@@ -1,18 +1,16 @@
 /*
- * Copyright (c) 2021, 2024, Oracle and/or its affiliates.
+ * Copyright (c) 2021, 2023, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms,
- * as designated in a particular file or component or in included license
+ * This program is also distributed with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
- *
+ * separately licensed software that they have included with MySQL.
  * This program is distributed in the hope that it will be useful,  but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
@@ -32,19 +30,17 @@
 
 #include "modules/adminapi/common/api_options.h"
 #include "modules/adminapi/common/async_replication_options.h"
+#include "modules/adminapi/common/async_topology.h"
 #include "modules/adminapi/common/clone_options.h"
 #include "modules/adminapi/common/cluster_types.h"
 #include "modules/adminapi/common/group_replication_options.h"
 #include "mysqlshdk/include/scripting/types_cpp.h"
 #include "mysqlshdk/libs/utils/utils_general.h"
 
-namespace mysqlsh {
-namespace dba {
-namespace cluster {
+namespace mysqlsh::dba::cluster {
 
 struct Add_instance_options : public Password_interactive_options,
-                              public Wait_recovery_option,
-                              public Recovery_progress_option {
+                              Wait_recovery_option {
   static const shcore::Option_pack_def<Add_instance_options> &options();
 
   void set_cert_subject(const std::string &value);
@@ -55,13 +51,17 @@ struct Add_instance_options : public Password_interactive_options,
   std::string cert_subject;
 };
 
-struct Rejoin_instance_options : public Password_interactive_options {
+struct Rejoin_instance_options : public Password_interactive_options,
+                                 Timeout_option,
+                                 Recovery_progress_option {
   static const shcore::Option_pack_def<Rejoin_instance_options> &options();
-
   Rejoin_group_replication_options gr_options;
+  Join_read_replica_clone_options clone_options;
+  bool dry_run = false;
 };
 
-struct Remove_instance_options : public Password_interactive_options {
+struct Remove_instance_options : public Password_interactive_options,
+                                 Timeout_option {
   static const shcore::Option_pack_def<Remove_instance_options> &options();
 
   bool get_force(bool default_value = false) const noexcept {
@@ -69,6 +69,7 @@ struct Remove_instance_options : public Password_interactive_options {
   }
 
   std::optional<bool> force;
+  bool dry_run = false;
 };
 
 struct Status_options {
@@ -105,7 +106,27 @@ struct Set_primary_instance_options {
   std::optional<uint32_t> running_transactions_timeout;
 };
 
-}  // namespace cluster
-}  // namespace dba
-}  // namespace mysqlsh
+// Read-Replicas
+
+enum class Source_type { PRIMARY, SECONDARY, CUSTOM };
+
+struct Replication_sources {
+  std::set<Managed_async_channel_source,
+           std::greater<Managed_async_channel_source>>
+      replication_sources;
+  std::optional<Source_type> source_type;
+};
+
+struct Add_replica_instance_options : public Timeout_option,
+                                      Recovery_progress_option {
+  static const shcore::Option_pack_def<Add_replica_instance_options> &options();
+  void set_replication_sources(const shcore::Value &value);
+
+  bool dry_run = false;
+  std::optional<std::string> label;
+  Replication_sources replication_sources_option;
+  Join_read_replica_clone_options clone_options;
+};
+
+}  // namespace mysqlsh::dba::cluster
 #endif  // MODULES_ADMINAPI_CLUSTER_API_OPTIONS_H_
