@@ -1,18 +1,16 @@
 /*
- * Copyright (c) 2018, 2024, Oracle and/or its affiliates.
+ * Copyright (c) 2018, 2023, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms,
- * as designated in a particular file or component or in included license
+ * This program is also distributed with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
- *
+ * separately licensed software that they have included with MySQL.
  * This program is distributed in the hope that it will be useful,  but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
@@ -29,17 +27,25 @@
 #include <optional>
 #include <string>
 
+#include "modules/adminapi/cluster/api_options.h"
 #include "modules/adminapi/cluster_set/cluster_set_impl.h"
+#include "modules/adminapi/common/async_topology.h"
 #include "modules/adminapi/common/common.h"
 #include "modules/adminapi/common/common_status.h"
+#include "modules/adminapi/common/dba_errors.h"
 #include "modules/adminapi/common/metadata_storage.h"
 #include "modules/adminapi/common/parallel_applier_options.h"
 #include "modules/adminapi/common/server_features.h"
 #include "modules/adminapi/common/sql.h"
+#include "mysqlshdk/include/scripting/types.h"
+#include "mysqlshdk/libs/mysql/async_replication.h"
 #include "mysqlshdk/libs/mysql/clone.h"
 #include "mysqlshdk/libs/mysql/group_replication.h"
 #include "mysqlshdk/libs/mysql/repl_config.h"
 #include "mysqlshdk/libs/utils/debug.h"
+#include "mysqlshdk/libs/utils/logger.h"
+#include "mysqlshdk/libs/utils/options.h"
+#include "mysqlshdk/libs/utils/utils_string.h"
 
 namespace mysqlsh {
 namespace dba {
@@ -115,7 +121,8 @@ inline bool set_ts(shcore::Dictionary_t dict, const std::string &prop,
 
 }  // namespace
 
-Status::Status(const Cluster_impl &cluster, std::optional<uint64_t> extended)
+Status::Status(const std::shared_ptr<Cluster_impl> &cluster,
+               std::optional<uint64_t> extended)
     : m_cluster(cluster), m_extended(extended) {}
 
 Status::~Status() = default;
@@ -125,8 +132,13 @@ void Status::connect_to_members() {
 
   for (const auto &inst : m_instances) {
     try {
-      m_member_sessions[inst.endpoint] =
-          ipool->connect_unchecked_endpoint(inst.endpoint);
+      if (inst.instance_type == Instance_type::READ_REPLICA) {
+        m_read_replica_sessions[inst.endpoint] =
+            ipool->connect_unchecked_endpoint(inst.endpoint);
+      } else {
+        m_member_sessions[inst.endpoint] =
+            ipool->connect_unchecked_endpoint(inst.endpoint);
+      }
     } catch (const shcore::Error &e) {
       m_member_connect_errors[inst.endpoint] = e.format();
     }
@@ -145,8 +157,13 @@ shcore::Dictionary_t Status::check_group_status(
   int quorum_size = 0;
   // count inconsistencies in the group vs metadata
   int missing_from_group = 0;
+  int cluster_instance_count = 0;
 
   for (const auto &inst : m_instances) {
+    if (inst.instance_type != Instance_type::GROUP_MEMBER) continue;
+
+    cluster_instance_count++;
+
     if (std::find_if(members.begin(), members.end(),
                      [&inst](const mysqlshdk::gr::Member &member) {
                        return member.uuid == inst.uuid;
@@ -182,12 +199,12 @@ shcore::Dictionary_t Status::check_group_status(
     desc_status = "Cluster has no quorum as visible from '" + instance.descr() +
                   "' and cannot process write transactions.";
   } else {
-    if (m_cluster.is_fenced_from_writes()) {
+    if (m_cluster->is_fenced_from_writes()) {
       rs_status = Cluster_status::FENCED_WRITES;
 
       desc_status = "Cluster is fenced from Write Traffic.";
-    } else if (m_cluster.is_cluster_set_member() &&
-               m_cluster.is_invalidated()) {
+    } else if (m_cluster->is_cluster_set_member() &&
+               m_cluster->is_invalidated()) {
       rs_status = Cluster_status::INVALIDATED;
 
       desc_status = "Cluster was invalidated by the ClusterSet it belongs to.";
@@ -215,12 +232,12 @@ shcore::Dictionary_t Status::check_group_status(
     }
   }
 
-  if (static_cast<int>(m_instances.size()) > online_count) {
-    if (m_instances.size() - online_count == 1) {
+  if (cluster_instance_count > online_count) {
+    if (cluster_instance_count - online_count == 1) {
       desc_status.append(" 1 member is not active.");
     } else {
       desc_status.append(" " +
-                         std::to_string(m_instances.size() - online_count) +
+                         std::to_string(cluster_instance_count - online_count) +
                          " members are not active.");
     }
   }
@@ -240,7 +257,7 @@ const Instance_metadata *Status::instance_with_uuid(const std::string &uuid) {
 
 Member_stats_map Status::query_member_stats() {
   Member_stats_map stats;
-  auto group_instance = m_cluster.get_cluster_server();
+  auto group_instance = m_cluster->get_cluster_server();
 
   if (group_instance) {
     try {
@@ -468,14 +485,14 @@ void Status::collect_basic_local_status(shcore::Dictionary_t dict,
   std::string sql;
 
   if (version >= Version(8, 0, 0)) {
-    if (m_cluster.is_cluster_set_member()) {
+    if (m_cluster->is_cluster_set_member()) {
       // PRIMARY of PC has no relevant replication lag info
       // PRIMARY of RC shows lag from clusterset_replication channel
       // SECONDARY members show replication from gr_applier channel
       std::string channel_name;
 
       if (is_primary) {
-        if (!m_cluster.is_primary_cluster()) {
+        if (!m_cluster->is_primary_cluster()) {
           channel_name = k_clusterset_async_channel_name;
         }
       } else {
@@ -678,8 +695,9 @@ void Status::feed_member_info(shcore::Dictionary_t dict,
                               std::optional<bool> super_read_only,
                               const std::vector<std::string> &fence_sysvars,
                               mysqlshdk::gr::Member_state self_state,
-                              bool is_auto_rejoin_running) {
-  (*dict)["readReplicas"] = shcore::Value(shcore::make_dict());
+                              bool is_auto_rejoin_running,
+                              const shcore::Dictionary_t read_replicas) {
+  (*dict)["readReplicas"] = shcore::Value(read_replicas);
 
   if (m_extended.value_or(0) >= 1) {
     // Set fenceSysVars array.
@@ -1005,7 +1023,8 @@ void check_auth_type_instance_ssl(shcore::Array_t issues,
   {
     auto instance_repl_account =
         cluster.get_metadata_storage()->get_instance_repl_account(
-            instance.get_uuid(), Cluster_type::GROUP_REPLICATION);
+            instance.get_uuid(), Cluster_type::GROUP_REPLICATION,
+            Replica_type::GROUP_MEMBER);
 
     if (std::get<0>(instance_repl_account).empty()) {
       // if this happens, a warning is already shown to the user in
@@ -1367,6 +1386,7 @@ shcore::Dictionary_t Status::get_topology(
   };
 
   std::vector<Instance_metadata_info> instances;
+  std::vector<Read_replica_info> read_replicas;
 
   // add placeholders for unmanaged members
   for (const auto &m : member_info) {
@@ -1413,7 +1433,7 @@ shcore::Dictionary_t Status::get_topology(
       log_debug("Instance %s with uuid=%s found in group but not in MD",
                 mdi.md.address.c_str(), m.uuid.c_str());
 
-      auto group_instance = m_cluster.get_cluster_server();
+      auto group_instance = m_cluster->get_cluster_server();
 
       mysqlshdk::db::Connection_options opts(mdi.md.endpoint);
       mysqlshdk::db::Connection_options group_session_copts(
@@ -1432,12 +1452,32 @@ shcore::Dictionary_t Status::get_topology(
   for (const auto &i : m_instances) {
     bool found = false;
     for (const auto &m : member_info) {
-      if (m.uuid == i.uuid ||
-          mysqlshdk::utils::are_endpoints_equal(
-              i.endpoint,
-              mysqlshdk::utils::make_host_and_port(m.host, m.port))) {
+      if (i.instance_type == Instance_type::READ_REPLICA) {
+        Read_replica_info read_replica = {};
+
+        read_replica.md = i;
+
+        Replication_sources replication_sources =
+            m_cluster->get_read_replica_replication_sources(i.uuid);
+
+        if (replication_sources.source_type != Source_type::CUSTOM) {
+          read_replica.managed_channel_info.automatic_sources = true;
+        } else {
+          read_replica.managed_channel_info.sources.swap(
+              replication_sources.replication_sources);
+        }
+
+        read_replicas.emplace_back(read_replica);
         found = true;
         break;
+      } else {
+        if (m.uuid == i.uuid ||
+            mysqlshdk::utils::are_endpoints_equal(
+                i.endpoint,
+                mysqlshdk::utils::make_host_and_port(m.host, m.port))) {
+          found = true;
+          break;
+        }
       }
     }
     if (!found) {
@@ -1460,14 +1500,19 @@ shcore::Dictionary_t Status::get_topology(
   }
 
   std::map<std::string, std::string> endpoints;
-  if (m_cluster.get_metadata_storage()->real_version().get_major() > 1) {
+  if (m_cluster->get_metadata_storage()->real_version().get_major() > 1) {
     endpoints =
-        m_cluster.get_metadata_storage()->get_instances_with_recovery_accounts(
-            m_cluster.get_id());
+        m_cluster->get_metadata_storage()->get_instances_with_recovery_accounts(
+            m_cluster->get_id());
   }
 
   auto mismatched_recovery_accounts =
-      m_cluster.get_mismatched_recovery_accounts();
+      m_cluster->get_mismatched_recovery_accounts();
+
+  // Flag to mark the primary instance was already feeded with rogue
+  // read-replicas info. Used to avoid all members being fed with the same
+  // read-replica when in multi-primary mode
+  bool already_feeded_primary = false;
 
   for (const auto &inst : instances) {
     shcore::Dictionary_t member = shcore::make_dict();
@@ -1491,7 +1536,8 @@ shcore::Dictionary_t Status::get_topology(
       // Get the current parallel-applier options
       parallel_applier_options = Parallel_applier_options(*instance);
 
-      // Get super_read_only value of each instance to set the mode accurately.
+      // Get super_read_only value of each instance to set the mode
+      // accurately.
       super_read_only = instance->get_sysvar_bool("super_read_only");
 
       // Get offline_mode value of each instance to set the mode accurately.
@@ -1529,7 +1575,7 @@ shcore::Dictionary_t Status::get_topology(
           std::string status;
           // Get the join timestamp from the Metadata
           shcore::Value join_time;
-          m_cluster.get_metadata_storage()->query_instance_attribute(
+          m_cluster->get_metadata_storage()->query_instance_attribute(
               instance->get_uuid(), k_instance_attribute_join_time, &join_time);
 
           std::tie(status, recovery_info) = recovery_status(
@@ -1593,14 +1639,22 @@ shcore::Dictionary_t Status::get_topology(
           shcore::Value(m_member_connect_errors[inst.md.endpoint]);
     }
     feed_metadata_info(member, inst.md);
-    feed_member_info(member, minfo, offline_mode, super_read_only,
-                     fence_sysvars, self_state, auto_rejoin);
+
+    bool is_primary = minfo.role == Member_role::PRIMARY;
+
+    feed_member_info(
+        member, minfo, offline_mode, super_read_only, fence_sysvars, self_state,
+        auto_rejoin,
+        get_read_replicas_info(inst, read_replicas,
+                               is_primary && !already_feeded_primary));
+
+    if (is_primary) already_feeded_primary = true;
 
     {
       shcore::Array_t issues = instance_diagnostics(
-          instance.get(), &m_cluster, inst, recovery_channel, applier_channel,
-          super_read_only, minfo, self_state, parallel_applier_options,
-          *m_cluster_transaction_size_limit);
+          instance.get(), m_cluster.get(), inst, recovery_channel,
+          applier_channel, super_read_only, minfo, self_state,
+          parallel_applier_options, *m_cluster_transaction_size_limit);
 
       if (offline_mode.value_or(false)) {
         issues->push_back(
@@ -1651,6 +1705,10 @@ shcore::Dictionary_t Status::get_topology(
       }
     }
 
+    if (inst.md.tags && !inst.md.tags->empty()) {
+      (*member)["tags"] = shcore::Value(inst.md.tags);
+    }
+
     if ((m_extended.value_or(0) >= 2) &&
         member_stats.find(inst.md.uuid) != member_stats.end()) {
       shcore::Dictionary_t mdict = member;
@@ -1682,15 +1740,15 @@ shcore::Dictionary_t Status::collect_replicaset_status() {
   shcore::Dictionary_t tmp = shcore::make_dict();
   shcore::Dictionary_t ret = shcore::make_dict();
 
-  auto group_instance = m_cluster.get_cluster_server();
+  auto group_instance = m_cluster->get_cluster_server();
   bool gr_running =
-      m_cluster.cluster_availability() != Cluster_availability::OFFLINE &&
-      m_cluster.cluster_availability() !=
+      m_cluster->cluster_availability() != Cluster_availability::OFFLINE &&
+      m_cluster->cluster_availability() !=
           Cluster_availability::SOME_UNREACHABLE &&
-      m_cluster.cluster_availability() != Cluster_availability::UNREACHABLE;
+      m_cluster->cluster_availability() != Cluster_availability::UNREACHABLE;
 
   std::string topology_mode =
-      mysqlshdk::gr::to_string(m_cluster.get_cluster_topology_type());
+      mysqlshdk::gr::to_string(m_cluster->get_cluster_topology_type());
 
   // Set Cluster name
   (*ret)["name"] = shcore::Value("default");
@@ -1711,7 +1769,7 @@ shcore::Dictionary_t Status::collect_replicaset_status() {
   }
 
   if (m_extended.value_or(0) >= 1) {
-    (*ret)["groupName"] = shcore::Value(m_cluster.get_group_name());
+    (*ret)["groupName"] = shcore::Value(m_cluster->get_group_name());
 
     if (group_instance &&
         group_instance->get_version() >= mysqlshdk::utils::Version(8, 0, 26)) {
@@ -1796,7 +1854,7 @@ shcore::Dictionary_t Status::collect_replicaset_status() {
       }
     }
   } else {
-    if (m_cluster.cluster_availability() == Cluster_availability::OFFLINE) {
+    if (m_cluster->cluster_availability() == Cluster_availability::OFFLINE) {
       (*ret)["status"] = shcore::Value("OFFLINE");
       (*ret)["statusText"] =
           shcore::Value("All members of the group are OFFLINE");
@@ -1807,25 +1865,25 @@ shcore::Dictionary_t Status::collect_replicaset_status() {
     }
   }
 
-  auto issues = cluster_diagnostics(m_cluster, member_info, protocol_version);
+  auto issues = cluster_diagnostics(*m_cluster, member_info, protocol_version);
 
   // Get the Cluster's transaction size_limit stored in the Metadata if the
   // Cluster is standalone or a Primary. Otherwise, it's a Replica so it
   // should be the value of the primary member
-  shcore::Value value;
   bool is_replica_cluster =
-      m_cluster.is_cluster_set_member() && !m_cluster.is_primary_cluster();
+      m_cluster->is_cluster_set_member() && !m_cluster->is_primary_cluster();
 
   if (group_instance) {
     m_cluster_transaction_size_limit =
-        m_cluster.get_cluster_server()
+        m_cluster->get_cluster_server()
             ->get_sysvar_int(kGrTransactionSizeLimit)
             .value_or(0);
   }
 
   if (!is_replica_cluster) {
-    if (!m_cluster.get_metadata_storage()->query_cluster_attribute(
-            m_cluster.get_id(), k_cluster_attribute_transaction_size_limit,
+    shcore::Value value;
+    if (!m_cluster->get_metadata_storage()->query_cluster_attribute(
+            m_cluster->get_id(), k_cluster_attribute_transaction_size_limit,
             &value)) {
       issues->push_back(
           shcore::Value("WARNING: Cluster's transaction size limit is not "
@@ -1866,7 +1924,7 @@ shcore::Array_t Status::validate_recovery_accounts_unused(
   auto issues = shcore::make_array();
 
   auto accounts =
-      m_cluster.get_unused_recovery_accounts(mismatched_recovery_accounts);
+      m_cluster->get_unused_recovery_accounts(mismatched_recovery_accounts);
   if (accounts.empty()) return issues;
 
   std::string msg{accounts.size() == 1
@@ -1883,15 +1941,409 @@ shcore::Array_t Status::validate_recovery_accounts_unused(
   return issues;
 }
 
+shcore::Array_t Status::read_replica_diagnostics(
+    Instance *instance, const Read_replica_info &rr_info,
+    bool is_primary) const {
+  using mysqlshdk::mysql::Replication_channel;
+
+  shcore::Array_t instance_errors = shcore::make_array();
+
+  auto append_error = [&instance_errors](const std::string &msg) {
+    if (!instance_errors) {
+      instance_errors = shcore::make_array();
+    }
+    instance_errors->push_back(shcore::Value(msg));
+  };
+
+  if (!instance) {
+    append_error(
+        "ERROR: Could not connect to the Read-Replica: The instance is "
+        "unreachable.");
+
+    return instance_errors;
+  }
+
+  // Check if super_read_only is disabled
+  if (!instance->is_read_only(true)) {
+    append_error(
+        "WARNING: Instance is a Read-Replica but super_read_only option is "
+        "OFF. Use Cluster.rejoinInstance() to fix it.");
+  }
+
+  // Check if the read-replica doesn't have any configured
+  // replicationSources
+  Replication_sources replication_sources =
+      m_cluster->get_read_replica_replication_sources(instance->get_uuid());
+
+  if (!replication_sources.source_type.has_value() ||
+      (*replication_sources.source_type == Source_type::CUSTOM &&
+       replication_sources.replication_sources.empty())) {
+    append_error(
+        "WARNING: Read Replica's replicationSources is empty. Use "
+        "Cluster.setInstanceOption() with the option "
+        "'replicationSources' to update the instance's sources.");
+  }
+
+  // Check if the channel is missing
+  if (rr_info.managed_channel_info.channel_name.empty()) {
+    append_error(
+        "WARNING: Read Replica's replication channel is missing. Use "
+        "Cluster.rejoinInstance() to restore it.");
+
+    return instance_errors;
+  }
+
+  if (!rr_info.managed_channel_info.automatic_failover) {
+    append_error(
+        "WARNING: Read Replica's replication channel is misconfigured: "
+        "automatic failover is disabled. Use Cluster.rejoinInstance() to "
+        "fix it.");
+  }
+
+  // Check if the read-replica doesn't have any active sources
+  if (rr_info.managed_channel_info.sources.empty()) {
+    append_error(
+        "WARNING: Read Replica has no active sources. Use "
+        "Cluster.rejoinInstance() to fix it.");
+  }
+
+  // Check if the read-replica active sources match the replicationSources
+  bool sources_mismatch = false;
+
+  if (replication_sources.source_type == Source_type::CUSTOM) {
+    if (replication_sources.replication_sources !=
+        rr_info.managed_channel_info.sources) {
+      sources_mismatch = true;
+    }
+  } else {
+    sources_mismatch =
+        (replication_sources.source_type == Source_type::PRIMARY &&
+         !is_primary);
+  }
+
+  if (sources_mismatch) {
+    append_error(
+        "WARNING: Read Replica's replication channel is misconfigured: the "
+        "current sources don't match the configured ones. Use "
+        "Cluster.rejoinInstance() to fix it.");
+  }
+
+  // Check the replication channel status
+  if (mysqlshdk::mysql::Replication_channel channel;
+      mysqlshdk::mysql::get_channel_status(
+          *instance, mysqlsh::dba::k_read_replica_async_channel_name,
+          &channel)) {
+    switch (channel.status()) {
+      case mysqlshdk::mysql::Replication_channel::OFF:
+      case mysqlshdk::mysql::Replication_channel::APPLIER_OFF:
+      case mysqlshdk::mysql::Replication_channel::RECEIVER_OFF: {
+        append_error(
+            "WARNING: Read Replica's replication channel is stopped. Use "
+            "Cluster.rejoinInstance() to restore it.");
+        break;
+      }
+      case mysqlshdk::mysql::Replication_channel::CONNECTION_ERROR: {
+        append_error(
+            "WARNING: Read Replica's replication channel stopped with a "
+            "connection error: '" +
+            channel.receiver.last_error.message +
+            "'. Use Cluster.rejoinInstance() to restore it.");
+        break;
+      }
+      case mysqlshdk::mysql::Replication_channel::APPLIER_ERROR: {
+        std::string applier_error_msg;
+
+        for (const auto &applier : channel.appliers) {
+          if (applier.state == Replication_channel::Applier::OFF &&
+              applier.last_error.code != 0) {
+            applier_error_msg = applier.last_error.message;
+            break;
+          }
+        }
+
+        append_error(
+            "WARNING: Read Replica's replication channel stopped with an "
+            "error: '" +
+            applier_error_msg +
+            "'. Use Cluster.rejoinInstance() to restore it.");
+        break;
+      }
+      case mysqlshdk::mysql::Replication_channel::ON:
+      case mysqlshdk::mysql::Replication_channel::CONNECTING:
+        break;
+    }
+  }
+
+  return instance_errors;
+}
+
+shcore::Dictionary_t Status::feed_read_replica_info(const Read_replica_info &rr,
+                                                    bool is_primary) {
+  shcore::Dictionary_t rr_dict = shcore::make_dict();
+
+  rr_dict->set("address", shcore::Value(rr.md.endpoint));
+
+  // Get the replication channel status
+  auto &rr_instance = m_read_replica_sessions[rr.md.endpoint];
+
+  std::string status;
+  if (!rr_instance) {
+    status = to_string(mysqlshdk::mysql::Read_replica_status::UNREACHABLE);
+  } else {
+    status = to_string(mysqlshdk::mysql::get_read_replica_status(*rr_instance));
+  }
+
+  rr_dict->set("status", shcore::Value(status));
+
+  if (rr_instance) {
+    rr_dict->set("version",
+                 shcore::Value(rr_instance->get_version().get_full()));
+  }
+
+  rr_dict->set("role", shcore::Value("READ_REPLICA"));
+
+  if (m_extended.value_or(0) >= 1) {
+    shcore::Array_t source_list = shcore::make_array();
+
+    // Check whether the channel was configured to follow the
+    // primary/secondary or a list of sources
+    bool automatic_sources = rr.managed_channel_info.automatic_sources;
+
+    if (automatic_sources && m_extended.value_or(0) < 2) {
+      shcore::Value source_attribute;
+      if (m_cluster->get_metadata_storage()->query_instance_attribute(
+              rr.md.uuid, k_instance_attribute_read_replica_replication_sources,
+              &source_attribute)) {
+        if (source_attribute.type == shcore::String) {
+          source_list->push_back(shcore::Value(source_attribute.as_string()));
+        }
+      }
+    } else {
+      for (const auto &source : rr.managed_channel_info.sources) {
+        if (m_extended.value_or(0) >= 2) {
+          shcore::Dictionary_t source_list_extended = shcore::make_dict();
+          (*source_list_extended)["address"] =
+              shcore::Value(source.to_string());
+          (*source_list_extended)["weight"] = shcore::Value(source.weight);
+
+          source_list->push_back(shcore::Value(source_list_extended));
+        } else {
+          source_list->push_back(shcore::Value(source.to_string()));
+        }
+      }
+    }
+
+    rr_dict->set("replicationSources", shcore::Value(source_list));
+
+    // SSL info
+    if (rr_instance) {
+      auto instance_repl_account =
+          m_cluster->get_metadata_storage()->get_instance_repl_account(
+              rr_instance->get_uuid(), Cluster_type::GROUP_REPLICATION,
+              Replica_type::READ_REPLICA);
+
+      std::string ssl_cipher, ssl_version;
+      mysqlshdk::mysql::iterate_thread_variables(
+          *(m_cluster->get_metadata_storage()->get_md_server()),
+          "Binlog Dump GTID", instance_repl_account.first, "Ssl%",
+          [&ssl_cipher, &ssl_version](const std::string &var_name,
+                                      const std::string &var_value) {
+            if (var_name == "Ssl_cipher") {
+              ssl_cipher = std::move(var_value);
+            } else if (var_name == "Ssl_version") {
+              ssl_version = std::move(var_value);
+            }
+
+            return (ssl_cipher.empty() || ssl_version.empty());
+          });
+
+      if (!(ssl_cipher.empty() && ssl_version.empty())) {
+        if (ssl_cipher.empty()) {
+          rr_dict->set("replicationSsl", shcore::Value(std::move(ssl_version)));
+        } else if (ssl_version.empty()) {
+          rr_dict->set("replicationSsl", shcore::Value(std::move(ssl_cipher)));
+        } else {
+          rr_dict->set("replicationSsl",
+                       shcore::Value(shcore::str_format(
+                           "%s %s", ssl_cipher.c_str(), ssl_version.c_str())));
+        }
+      }
+
+      mysqlshdk::mysql::Replication_channel_master_info master_info;
+      mysqlshdk::mysql::Replication_channel_relay_log_info relay_info;
+
+      if (mysqlshdk::mysql::get_channel_info(*rr_instance,
+                                             k_read_replica_async_channel_name,
+                                             &master_info, &relay_info)) {
+        auto channel_stats_map =
+            channel_status(&rr.repl_channel_info, &master_info, &relay_info, "",
+                           m_extended.value_or(0), false, true);
+
+        auto store_dict = [&](std::initializer_list<std::string> keys) {
+          for (const auto &key : keys) {
+            if (channel_stats_map->has_key(key)) {
+              rr_dict->set(key, channel_stats_map->at(key));
+            }
+          }
+        };
+
+        store_dict({"applierStatus", "applierThreadState",
+                    "applierWorkerThreads", "receiverStatus",
+                    "receiverThreadState", "replicationLag"});
+
+        if (m_extended.value_or(0) >= 2) {
+          store_dict({"applierQueuedTransactionSetSize",
+                      "applierQueuedTransactionSet",
+                      "receiverTimeSinceLastMessage", "coordinatorState",
+                      "coordinatorThreadState"});
+        }
+
+        if (m_extended.value_or(0) == 3) {
+          store_dict({"options"});
+        }
+      } else {
+        log_info("Failed to obtain information from replication channel '%s'",
+                 k_read_replica_async_channel_name);
+      }
+    }
+  }
+
+  // Run the diagnostics
+  shcore::Array_t instance_errors = nullptr;
+  instance_errors = read_replica_diagnostics(rr_instance.get(), rr, is_primary);
+
+  if (!instance_errors->empty()) {
+    rr_dict->set("instanceErrors", shcore::Value(instance_errors));
+  }
+
+  return rr_dict;
+}
+
+shcore::Dictionary_t Status::get_read_replicas_info(
+    const Instance_metadata_info &instance_md,
+    const std::vector<Read_replica_info> &read_replicas, bool is_primary) {
+  shcore::Dictionary_t dict = shcore::make_dict();
+
+  std::vector<Read_replica_info> member_read_replicas, rogue_read_replicas;
+
+  // if there are no read-replicas for this member and it's not the primary
+  // (to place rogue read-replicas under it) return right away
+  if (!is_primary && read_replicas.empty()) {
+    return dict;
+  }
+
+  // Get the list of read-replicas for this member
+  for (auto rr_info : read_replicas) {
+    auto &rr_session = m_read_replica_sessions[rr_info.md.endpoint];
+
+    if (rr_session) {
+      if (!rr_info.managed_channel_info.automatic_sources &&
+          rr_info.managed_channel_info.sources.empty()) {
+        rogue_read_replicas.push_back(rr_info);
+        continue;
+      }
+
+      // Get the current source member
+      mysqlshdk::mysql::Replication_channel channel = {};
+
+      if (mysqlshdk::mysql::get_channel_status(
+              *rr_session, k_read_replica_async_channel_name,
+              &rr_info.repl_channel_info)) {
+        rr_info.current_source_server_uuid =
+            std::move(rr_info.repl_channel_info.source_uuid);
+
+        Managed_async_channel channel_config = {};
+        channel_config.channel_name = k_read_replica_async_channel_name;
+
+        if (!get_managed_connection_failover_configuration(*rr_session,
+                                                           &channel_config)) {
+          log_info(
+              "Failed to get the information for the Read-Replica managed "
+              "replication channel at '%s'",
+              rr_info.md.endpoint.c_str());
+          rogue_read_replicas.push_back(rr_info);
+          continue;
+        }
+
+        rr_info.managed_channel_info = std::move(channel_config);
+
+      } else {
+        // Instance is a rogue
+        rogue_read_replicas.push_back(rr_info);
+        continue;
+      }
+
+      // If the current source is empty, the instance is a rogue
+      // The reason why the source is empty might be because the replica was in
+      // error state and the channel was then stopped
+      if (rr_info.current_source_server_uuid.empty()) {
+        rogue_read_replicas.push_back(rr_info);
+        continue;
+      }
+
+      // If the current source does not belong to the cluster anymore, the
+      // instance must be considered rogue. The instance might have
+      // entered an error state and during that period the current source was
+      // removed from the cluster
+      try {
+        // Attempt to get the current source from the Cluster
+        m_cluster->get_metadata_storage()->get_instance_by_uuid(
+            rr_info.current_source_server_uuid);
+      } catch (const shcore::Exception &e) {
+        if (e.code() != SHERR_DBA_MEMBER_METADATA_MISSING) {
+          log_info("Error querying metadata for %s: %s\n",
+                   rr_info.current_source_server_uuid.c_str(), e.what());
+        }
+
+        // The source does not belong to the Cluster anymore, place the
+        // read-replica under the primary, as rogue
+        rogue_read_replicas.push_back(rr_info);
+        continue;
+      }
+
+      if (rr_info.current_source_server_uuid ==
+          instance_md.actual_server_uuid) {
+        member_read_replicas.push_back(rr_info);
+        continue;
+      }
+    } else {
+      rogue_read_replicas.push_back(rr_info);
+      continue;
+    }
+  }
+
+  // If this is the primary member, place rogue read-replicas under it
+  if (is_primary) {
+    for (auto const &rr : rogue_read_replicas) {
+      shcore::Dictionary_t rr_dict = feed_read_replica_info(rr, is_primary);
+
+      (*dict)[rr.md.label] = shcore::Value(rr_dict);
+    }
+  }
+
+  // If there are no other read-replicas for this member, return right away
+  if (member_read_replicas.empty()) {
+    return dict;
+  }
+
+  for (auto const &rr : member_read_replicas) {
+    shcore::Dictionary_t rr_dict = feed_read_replica_info(rr, is_primary);
+
+    (*dict)[rr.md.label] = shcore::Value(rr_dict);
+  }
+
+  return dict;
+}
+
 void Status::prepare() {
   // Sanity check: Verify if the topology type changed and issue an error if
   // needed.
-  if (m_cluster.get_cluster_server()) m_cluster.sanity_check();
+  if (m_cluster->get_cluster_server()) m_cluster->sanity_check();
 
-  m_instances = m_cluster.get_instances();
+  m_instances = m_cluster->get_all_instances();
 
-  // Always connect to members to be able to get an accurate mode, based on
-  // their super_ready_only value.
+  // Always connect to members to be able to get an accurate mode, based
+  // on their super_ready_only value.
   connect_to_members();
 }
 
@@ -1916,7 +2368,7 @@ shcore::Value Status::get_default_replicaset_status() {
   //   - ERROR
   //
   // If that's the case, a warning must be added to the resulting JSON object
-  auto group_instance = m_cluster.get_cluster_server();
+  auto group_instance = m_cluster->get_cluster_server();
 
   if (group_instance) {
     auto state = get_replication_group_state(
@@ -1927,8 +2379,8 @@ shcore::Value Status::get_default_replicaset_status() {
 
     if (show_warning) {
       std::string warning =
-          "The cluster description may be inaccurate as it was generated from "
-          "an instance in ";
+          "The cluster description may be inaccurate as it was generated "
+          "from an instance in ";
       warning.append(ManagedInstance::describe(
           static_cast<ManagedInstance::State>(state.source_state)));
       warning.append(" state");
@@ -1942,36 +2394,36 @@ shcore::Value Status::get_default_replicaset_status() {
 shcore::Value Status::execute() {
   shcore::Dictionary_t dict = shcore::make_dict();
 
-  (*dict)["clusterName"] = shcore::Value(m_cluster.get_name());
+  (*dict)["clusterName"] = shcore::Value(m_cluster->get_name());
 
   // If the Cluster belongs to a ClusterSet, include relevant ClusterSet
   // information:
   //   - domainName
   //   - clusterRole
   //   - clusterSetReplicationStatus
-  if (m_cluster.is_cluster_set_member()) {
-    auto cs_md = m_cluster.get_clusterset_metadata();
+  if (m_cluster->is_cluster_set_member()) {
+    auto cs_md = m_cluster->get_clusterset_metadata();
     Cluster_set_metadata cset;
 
-    m_cluster.get_metadata_storage()->get_cluster_set(cs_md.cluster_set_id,
-                                                      true, &cset, nullptr);
+    m_cluster->get_metadata_storage()->get_cluster_set(cs_md.cluster_set_id,
+                                                       true, &cset, nullptr);
 
     (*dict)["domainName"] = shcore::Value(cset.domain_name);
     (*dict)["clusterRole"] =
-        shcore::Value(m_cluster.is_primary_cluster() ? "PRIMARY" : "REPLICA");
+        shcore::Value(m_cluster->is_primary_cluster() ? "PRIMARY" : "REPLICA");
 
     // Get the ClusterSet replication channel status
     Cluster_channel_status ch_status =
-        m_cluster.get_cluster_set_object()->get_replication_channel_status(
-            m_cluster);
+        m_cluster->get_cluster_set_object()->get_replication_channel_status(
+            *m_cluster);
 
     // If the Cluster is a Replica, add the status right away
-    if (!m_cluster.is_primary_cluster()) {
+    if (!m_cluster->is_primary_cluster()) {
       (*dict)["clusterSetReplicationStatus"] =
           shcore::Value(to_string(ch_status));
     } else {
-      // If it's a Primary, add the status in case is suspicious (not Missing or
-      // Stopped)
+      // If it's a Primary, add the status in case is suspicious (not Missing
+      // or Stopped)
       if (ch_status != Cluster_channel_status::MISSING &&
           ch_status != Cluster_channel_status::STOPPED) {
         (*dict)["clusterSetReplicationStatus"] =
@@ -1983,26 +2435,26 @@ shcore::Value Status::execute() {
   // Get the default replicaSet options
   (*dict)["defaultReplicaSet"] = shcore::Value(get_default_replicaset_status());
 
-  if (m_cluster.get_cluster_server()) {
+  if (m_cluster->get_cluster_server()) {
     // Gets the metadata version
     if (m_extended.value_or(0) >= 1) {
       auto version = mysqlsh::dba::metadata::installed_version(
-          m_cluster.get_cluster_server());
+          m_cluster->get_cluster_server());
       (*dict)["metadataVersion"] = shcore::Value(version.get_base());
     }
 
     // Iterate all replicasets and get the status for each one
 
-    std::string addr = m_cluster.get_cluster_server()->get_canonical_address();
+    std::string addr = m_cluster->get_cluster_server()->get_canonical_address();
     (*dict)["groupInformationSourceMember"] = shcore::Value(addr);
   }
 
-  auto md_server = m_cluster.get_metadata_storage()->get_md_server();
+  auto md_server = m_cluster->get_metadata_storage()->get_md_server();
 
   // metadata server, if its a different one
   if (md_server &&
-      (!m_cluster.get_cluster_server() ||
-       md_server->get_uuid() != m_cluster.get_cluster_server()->get_uuid())) {
+      (!m_cluster->get_cluster_server() ||
+       md_server->get_uuid() != m_cluster->get_cluster_server()->get_uuid())) {
     (*dict)["metadataServer"] =
         shcore::Value(md_server->get_canonical_address());
   }
