@@ -1,18 +1,16 @@
 /*
- * Copyright (c) 2021, 2024, Oracle and/or its affiliates.
+ * Copyright (c) 2021, 2023, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms,
- * as designated in a particular file or component or in included license
+ * This program is also distributed with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
- *
+ * separately licensed software that they have included with MySQL.
  * This program is distributed in the hope that it will be useful,  but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
@@ -48,7 +46,6 @@ struct Timeout_option {
 
 struct Interactive_option {
   static const shcore::Option_pack_def<Interactive_option> &options();
-  void set_interactive(bool interactive);
   bool interactive() const;
 
  private:
@@ -57,12 +54,11 @@ struct Interactive_option {
 
 struct Wait_recovery_option {
   static const shcore::Option_pack_def<Wait_recovery_option> &options();
-  void set_wait_recovery(const std::string &option, int value);
+  void set_wait_recovery(int value);
   Recovery_progress_style get_wait_recovery();
 
  private:
   std::optional<Recovery_progress_style> m_wait_recovery;
-  std::optional<Recovery_progress_style> m_recovery_progress;
 };
 
 struct Recovery_progress_option {
