@@ -1,18 +1,16 @@
 /*
- * Copyright (c) 2016, 2024, Oracle and/or its affiliates.
+ * Copyright (c) 2016, 2023, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms,
- * as designated in a particular file or component or in included license
+ * This program is also distributed with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
- *
+ * separately licensed software that they have included with MySQL.
  * This program is distributed in the hope that it will be useful,  but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
@@ -27,6 +25,7 @@
 #define MODULES_ADMINAPI_COMMON_COMMON_H_
 
 #include <array>
+#include <cstdint>
 #include <functional>
 #include <locale>
 #include <map>
@@ -40,16 +39,11 @@
 
 #include "modules/adminapi/common/clone_options.h"
 #include "modules/adminapi/common/cluster_types.h"
-
 #include "modules/adminapi/common/metadata_management_mysql.h"
 #include "modules/mod_utils.h"
 #include "mysqlshdk/libs/config/config.h"
 #include "mysqlshdk/libs/db/connection_options.h"
-#include "mysqlshdk/libs/db/session.h"
 #include "mysqlshdk/libs/mysql/replication.h"
-#include "mysqlshdk/libs/utils/enumset.h"
-#include "scripting/lang_base.h"
-#include "scripting/types.h"
 
 namespace mysqlsh {
 
@@ -91,6 +85,8 @@ enum Type {
   AsyncReplication = 1 << 8,
   Unknown = 1 << 9
 };
+
+std::string to_string(Type type);
 }  // namespace TargetType
 
 namespace ManagedInstance {
@@ -188,6 +184,14 @@ inline constexpr const char k_default_domain_name[] = "mydb";
 inline constexpr const char *k_replicaset_channel_name = "";
 inline constexpr const char *k_clusterset_async_channel_name =
     "clusterset_replication";
+inline constexpr const char *k_read_replica_async_channel_name =
+    "read_replica_replication";
+
+inline constexpr int k_read_replica_master_connect_retry = 3;
+inline constexpr int k_read_replica_master_retry_count = 10;
+inline constexpr uint64_t k_read_replica_max_weight = 100;
+inline constexpr uint64_t k_read_replica_higher_weight = 80;
+inline constexpr uint64_t k_read_replica_lower_weight = 60;
 
 // TODO(alfredo) - all server configuration constants and general functions
 // should be moved to repl_config.h
@@ -282,14 +286,17 @@ inline constexpr const char kSwitchCommunicationStack[] =
 inline constexpr const char kTransactionSizeLimit[] = "transactionSizeLimit";
 inline constexpr const char kGrTransactionSizeLimit[] =
     "group_replication_transaction_size_limit";
-constexpr const char kRequireCertIssuer[] = "requireCertIssuer";
-constexpr const char kRequireCertSubject[] = "requireCertSubject";
-constexpr const char kPasswordExpiration[] = "passwordExpiration";
-constexpr const char kMemberAuthType[] = "memberAuthType";
-constexpr const char kCertIssuer[] = "certIssuer";
-constexpr const char kCertSubject[] = "certSubject";
-constexpr const char kReplicationSslMode[] = "replicationSslMode";
+inline constexpr const char kRequireCertIssuer[] = "requireCertIssuer";
+inline constexpr const char kRequireCertSubject[] = "requireCertSubject";
+inline constexpr const char kPasswordExpiration[] = "passwordExpiration";
+inline constexpr const char kMemberAuthType[] = "memberAuthType";
+inline constexpr const char kCertIssuer[] = "certIssuer";
+inline constexpr const char kCertSubject[] = "certSubject";
+inline constexpr const char kReplicationSslMode[] = "replicationSslMode";
 inline constexpr const char kPaxosSingleLeader[] = "paxosSingleLeader";
+inline constexpr const char kReplicationSources[] = "replicationSources";
+inline constexpr const char kReplicationSourcesAutoPrimary[] = "primary";
+inline constexpr const char kReplicationSourcesAutoSecondary[] = "secondary";
 
 inline constexpr const int k_group_replication_members_limit = 9;
 
@@ -335,6 +342,11 @@ inline const std::map<std::string, Option_availability>
         {kIpAllowlist,
          {kGrIpAllowlist, mysqlshdk::utils::Version("8.0.24"), {}}}};
 
+/**
+ * List with the supported build-in tags for setOption and setInstanceOption
+ */
+typedef std::map<std::string, shcore::Value_type> built_in_tags_map_t;
+
 struct Instance_definition {
   int host_id;
   int replicaset_id;
@@ -414,16 +426,16 @@ inline constexpr const char *kReplicationMemberAuthCertSubjectPassword =
 std::string to_string(Replication_auth_type auth);
 Replication_auth_type to_replication_auth_type(std::string_view auth);
 
-constexpr const char *kCommunicationStackMySQL = "MYSQL";
-constexpr const char *kCommunicationStackXCom = "XCOM";
+inline constexpr const char *kCommunicationStackMySQL = "MYSQL";
+inline constexpr const char *kCommunicationStackXCom = "XCOM";
 
-const std::set<std::string> kCommunicationStackValidValues = {
+inline const std::set<std::string> kCommunicationStackValidValues = {
     kCommunicationStackXCom, kCommunicationStackMySQL};
 
 /**
  * Map of the supported Cluster capabilities
  */
-const std::map<std::string, std::set<std::string>>
+inline const std::map<std::string, std::set<std::string>>
     k_cluster_supported_capabilities{
         {kCommunicationStack, kCommunicationStackValidValues}};
 
@@ -499,13 +511,15 @@ bool SHCORE_PUBLIC validate_cluster_group_name(
     const mysqlshdk::mysql::IInstance &instance, const std::string &group_name);
 
 bool validate_super_read_only(const mysqlshdk::mysql::IInstance &instance,
-                              std::optional<bool> clear_read_only);
+                              mysqlshdk::null_bool clear_read_only,
+                              bool interactive);
 
 enum class Instance_rejoinability {
   REJOINABLE,
   NOT_MEMBER,
   ONLINE,
-  RECOVERING
+  RECOVERING,
+  READ_REPLICA
 };
 
 Instance_rejoinability validate_instance_rejoinable(
@@ -513,12 +527,17 @@ Instance_rejoinability validate_instance_rejoinable(
     const std::shared_ptr<MetadataStorage> &metadata, Cluster_id cluster_id,
     bool *out_uuid_mistmatch = nullptr);
 
+bool validate_instance_standalone(const Instance &instance,
+                                  TargetType::Type *target_type = nullptr);
+
 bool is_sandbox(const mysqlshdk::mysql::IInstance &instance,
                 std::string *cnfPath = nullptr);
 
 // AdminAPI interactive handling specific methods
 std::string prompt_cnf_path(const mysqlshdk::mysql::IInstance &instance);
 int prompt_menu(const std::vector<std::string> &options, int defopt);
+bool prompt_super_read_only(const mysqlshdk::mysql::IInstance &instance,
+                            bool throw_on_error = false);
 void dump_table(const std::vector<std::string> &column_names,
                 const std::vector<std::string> &column_labels,
                 shcore::Value::Array_type_ref documents);
@@ -754,6 +773,8 @@ mysqlshdk::db::Ssl_options prepare_replica_ssl_options(
     const mysqlshdk::mysql::IInstance &instance, Cluster_ssl_mode ssl_mode,
     Replication_auth_type auth_type);
 
+void validate_replication_sources_option(const shcore::Value &value);
+
 namespace cluster_topology_executor_ops {
 /**
  * Validate the options used in addInstance() and rejoinInstance()
@@ -795,6 +816,10 @@ void validate_local_address_ip_compatibility(
     const std::shared_ptr<mysqlsh::dba::Instance> &target_instance,
     const std::string &local_address, const std::string &group_seeds,
     mysqlshdk::utils::Version lowest_cluster_version);
+
+void validate_read_replica_version(
+    mysqlshdk::utils::Version target_instance_version,
+    mysqlshdk::utils::Version lowest_cluster_version);
 }  // namespace cluster_topology_executor_ops
 
 }  // namespace dba
