@@ -1,18 +1,16 @@
 /*
- * Copyright (c) 2016, 2024, Oracle and/or its affiliates.
+ * Copyright (c) 2016, 2023, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms,
- * as designated in a particular file or component or in included license
+ * This program is also distributed with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
- *
+ * separately licensed software that they have included with MySQL.
  * This program is distributed in the hope that it will be useful,  but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
@@ -29,6 +27,7 @@
 #include <list>
 #include <map>
 #include <memory>
+#include <optional>
 #include <set>
 #include <string>
 #include <utility>
@@ -38,6 +37,7 @@
 #include "modules/adminapi/common/common.h"
 #include "modules/adminapi/common/instance_pool.h"
 #include "modules/adminapi/common/metadata_management_mysql.h"
+#include "mysqlshdk/libs/db/session.h"
 #include "mysqlshdk/libs/mysql/group_replication.h"
 #include "mysqlshdk/libs/mysql/undo.h"
 #include "mysqlshdk/libs/mysql/utils.h"
@@ -50,8 +50,13 @@ inline constexpr const char *kNotifyClusterSetPrimaryChanged =
 
 inline constexpr const char *kNotifyDataClusterSetId = "CLUSTER_SET_ID";
 
+enum class Instance_type { GROUP_MEMBER, ASYNC_MEMBER, READ_REPLICA, NONE };
+
 enum class Instance_column_md { ATTRIBUTES, ADDRESSES };
 
+std::string to_string(const Instance_type instance_type);
+Instance_type to_instance_type(const std::string &instance_type);
+
 struct Instance_metadata {
   Cluster_id cluster_id;
   Instance_id id = 0;
@@ -65,9 +70,13 @@ struct Instance_metadata {
   uint32_t server_id = 0;
   bool hidden_from_router = false;
   std::string cert_subject;
+  Instance_type instance_type = Instance_type::NONE;
+
+  shcore::Dictionary_t tags = nullptr;
 
   // GR clusters only
   std::string group_name;
+  Replica_type replica_type = Replica_type::GROUP_MEMBER;
 
   // Async clusters only
   Instance_id master_id = 0;
@@ -86,6 +95,8 @@ struct Cluster_metadata {
   std::string description;
   bool enabled = true;
 
+  shcore::Dictionary_t tags = nullptr;
+
   std::string full_cluster_name() const { return cluster_name; }
 
   // GR specific
@@ -117,21 +128,24 @@ struct Cluster_set_metadata {
 
 struct Router_metadata {
   std::string name;
-  uint64_t id;
+  uint64_t id{0};
   std::string hostname;
-  mysqlshdk::utils::nullable<std::string> rw_port;
-  mysqlshdk::utils::nullable<std::string> ro_port;
-  mysqlshdk::utils::nullable<std::string> rw_x_port;
-  mysqlshdk::utils::nullable<std::string> ro_x_port;
-  mysqlshdk::utils::nullable<std::string> bootstrap_target_type;
-  mysqlshdk::utils::nullable<std::string> last_checkin;
-  mysqlshdk::utils::nullable<std::string> version;
-  mysqlshdk::utils::nullable<std::string> target_cluster;
+  std::optional<std::string> rw_port;
+  std::optional<std::string> ro_port;
+  std::optional<std::string> rw_x_port;
+  std::optional<std::string> ro_x_port;
+  std::optional<std::string> rw_split_port;
+  std::optional<std::string> bootstrap_target_type;
+  std::optional<std::string> last_checkin;
+  std::optional<std::string> version;
+  std::optional<std::string> target_cluster;
+
+  shcore::Dictionary_t tags = nullptr;
 };
 
 struct Router_options_metadata {
-  mysqlshdk::utils::nullable<std::string> router_label;
-  std::map<std::string, shcore::Value> defined_options;
+  std::map<std::string, shcore::Value> global;
+  std::map<std::string, std::map<std::string, shcore::Value>> routers;
 };
 
 class Cluster_impl;
@@ -183,7 +197,8 @@ class MetadataStorage {
    */
   bool check_instance_type(const std::string &uuid,
                            const mysqlshdk::utils::Version &md_version,
-                           Cluster_type *out_type) const;
+                           Cluster_type *out_type,
+                           Replica_type *out_replica_type) const;
 
   bool check_all_members_online() const;
 
@@ -212,22 +227,15 @@ class MetadataStorage {
    */
   void cleanup_for_cluster(Cluster_id cluster_id);
 
-  /**
-   * Deletes any dangling Clusters the ClusterSet's Metadata may have
-   *
-   * Ensures that any unrelated Cluster entries on the ClusterSet's Metadata are
-   * removed. Those can be resulting of failed actions, such as adding a new
-   * Replica Cluster.
-   */
-  void prune_clusterset_metadata(const Cluster_set_id &cs_id);
-
   Cluster_set_id create_cluster_set_record(
       Cluster_set_impl *cs, Cluster_id seed_cluster_id,
       shcore::Dictionary_t seed_attributes);
 
-  Instance_id insert_instance(const Instance_metadata &instance);
+  Instance_id insert_instance(const Instance_metadata &instance,
+                              Transaction_undo *undo = nullptr);
   void update_instance(const Instance_metadata &instance);
-  void remove_instance(const std::string &instance_address);
+  void remove_instance(const std::string &instance_address,
+                       Transaction_undo *undo = nullptr);
   void drop_cluster(const std::string &cluster_name,
                     Transaction_undo *undo = nullptr);
 
@@ -299,22 +307,9 @@ class MetadataStorage {
   void set_cluster_tag(const std::string &uuid, const std::string &tagname,
                        const shcore::Value &value);
 
-  /**
-   * Returns the list of tags for a given instance stored on the metadata
-   * @param uuid The uuid of the instance
-   * @return string with the json object representation of the tags for the
-   * given instance. Empty string means there are no tags.
-   */
-  std::string get_instance_tags(const std::string &uuid) const;
-
-  /**
-   * Returns the list of tags for a given cluster/replicaset stored on the
-   * metadata
-   * @param uuid The uuid of the cluster/replicaset
-   * @return string with the json object representation of the tags for the
-   * given cluster/replicaset. Empty string means there are no tags.
-   */
-  std::string get_cluster_tags(const std::string &uuid) const;
+  void set_router_tag(Cluster_type type, const std::string &cluster_id,
+                      const std::string &router, const std::string &tagname,
+                      const shcore::Value &value);
 
   /**
    * Update the information on the metadata about the recovery user being used
@@ -328,6 +323,7 @@ class MetadataStorage {
    */
   void update_instance_repl_account(const std::string &instance_uuid,
                                     Cluster_type type,
+                                    Replica_type replica_type,
                                     const std::string &recovery_account_user,
                                     const std::string &recovery_account_host,
                                     Transaction_undo *undo = nullptr);
@@ -336,11 +332,13 @@ class MetadataStorage {
    * with the given uuid.
    * @param instance_uuid the uuid of the instance
    * @param type whether account is for ReplicaSet or Cluster
+   * @para replica_type GROUP_MEMBER or READ_REPLICA
    * @return a pair with the recovery account user name and recovery account
    * hostname
    */
   std::pair<std::string, std::string> get_instance_repl_account(
-      const std::string &instance_uuid, Cluster_type type);
+      const std::string &instance_uuid, Cluster_type type,
+      Replica_type replica_type);
 
   void update_cluster_repl_account(const Cluster_id &cluster_id,
                                    const std::string &repl_account_user,
@@ -350,6 +348,14 @@ class MetadataStorage {
   std::pair<std::string, std::string> get_cluster_repl_account(
       const Cluster_id &cluster_id) const;
 
+  std::pair<std::string, std::string> get_read_replica_repl_account(
+      const std::string &instance_uuid) const;
+
+  void update_read_replica_repl_account(const std::string &instance_uuid,
+                                        const std::string &repl_account_user,
+                                        const std::string &repl_account_host,
+                                        Transaction_undo *undo = nullptr);
+
   /**
    * Fetch from the metadata list of server uuids
    * with recovery account information.
@@ -421,32 +427,45 @@ class MetadataStorage {
                                     std::string *out_primary_id,
                                     uint64_t *out_view_id);
 
-  std::vector<Instance_metadata> get_all_instances(Cluster_id cluster_id = "");
+  std::vector<Instance_metadata> get_all_instances(
+      Cluster_id cluster_id = "", bool include_read_replicas = false);
 
-  Instance_metadata get_instance_by_uuid(const std::string &uuid) const;
+  Instance_metadata get_instance_by_uuid(
+      const std::string &uuid, const Cluster_id &cluster_id = "") const;
 
   Instance_metadata get_instance_by_address(
-      const std::string &instance_address);
+      const std::string &instance_address) const;
 
   std::vector<Instance_metadata> get_replica_set_instances(
       const Cluster_id &rs_id);
 
   std::vector<Router_metadata> get_routers(const Cluster_id &cluster_id);
   std::vector<Router_metadata> get_clusterset_routers(const Cluster_set_id &cs);
-  std::vector<Router_options_metadata> get_routing_options(
-      const Cluster_set_id &clusterset_id);
+
+  Router_options_metadata get_routing_options(Cluster_type type,
+                                              const std::string &id);
+
   std::string get_cluster_name(const std::string &group_replication_group_name);
   std::string get_cluster_group_name(const std::string &cluster_name);
 
-  void set_clusterset_global_routing_option(const Cluster_set_id &id,
-                                            const std::string option,
-                                            const shcore::Value &value);
+  void set_global_routing_option(Cluster_type type, const std::string &id,
+                                 const std::string &option,
+                                 const shcore::Value &value);
 
-  void set_routing_option(const std::string &router,
-                          const std::string &clusterset_id,
+  void set_routing_option(Cluster_type type, const std::string &router,
+                          const std::string &cluster_id,
                           const std::string &option,
                           const shcore::Value &value);
 
+  shcore::Value get_global_routing_option(Cluster_type type,
+                                          const std::string &cluster_id,
+                                          const std::string &option);
+
+  shcore::Value get_routing_option(Cluster_type type,
+                                   const std::string &cluster_id,
+                                   const std::string &router,
+                                   const std::string &option);
+
   /**
    * Get the topology mode of the cluster from the metadata.
    *
@@ -503,6 +522,16 @@ class MetadataStorage {
   void migrate_routers_to_clusterset(const Cluster_id &cluster_id,
                                      const Cluster_set_id &cluster_set_id);
 
+  /**
+   * Migrates the Read Only Targets setting from a Cluster to a ClusterSet when
+   * it becomes a ClusterSet
+   *
+   * @param cluster_id The target Cluster ID
+   * @param cluster_set_id The target clusterSet ID to be set
+   */
+  void migrate_read_only_targets_to_clusterset(
+      const Cluster_id &cluster_id, const Cluster_set_id &cluster_set_id);
+
   /**
    * Gets all routers using a Cluster as Target Cluster
    *
@@ -513,6 +542,8 @@ class MetadataStorage {
   std::vector<Router_metadata> get_routers_using_cluster_as_target(
       const Cluster_id &cluster_id);
 
+  shcore::Value get_router_info(const std::string &router_name);
+
   // Async cluster view operations
   Instance_id record_async_member_added(const Instance_metadata &member);
   void record_async_member_rejoined(const Instance_metadata &member);
@@ -572,7 +603,8 @@ class MetadataStorage {
   }
 
   bool check_metadata(mysqlshdk::utils::Version *out_version = nullptr,
-                      Cluster_type *out_type = nullptr) const;
+                      Cluster_type *out_type = nullptr,
+                      Replica_type *out_replica_type = nullptr) const;
 
   bool check_cluster_set(
       const mysqlshdk::mysql::IInstance *target_instance = nullptr,
@@ -582,6 +614,8 @@ class MetadataStorage {
 
   bool supports_cluster_set() const;
 
+  bool supports_read_replicas() const;
+
   /**
    * Try to acquire an exclusive lock on the metadata.
    *
@@ -670,10 +704,6 @@ class MetadataStorage {
                      const std::string &uuid, const std::string &tagname,
                      const shcore::Value &value);
 
-  std::string get_table_tags(const std::string &tablename,
-                             const std::string &uuid_column_name,
-                             const std::string &uuid) const;
-
   bool cluster_sets_supported() const;
 
   void update_instance_metadata(std::string_view uuid,
@@ -697,7 +727,7 @@ class MetadataStorage {
 
   template <typename... Args>
   inline std::shared_ptr<mysqlshdk::db::IResult> execute_sqlf(
-      const std::string &sql, const Args &...args) const {
+      const std::string &sql, const Args &... args) const {
     return execute_sql(shcore::sqlformat(sql, args...));
   }
 
