@@ -1,18 +1,16 @@
 /*
- * Copyright (c) 2016, 2024, Oracle and/or its affiliates.
+ * Copyright (c) 2016, 2023, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms,
- * as designated in a particular file or component or in included license
+ * This program is also distributed with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
- *
+ * separately licensed software that they have included with MySQL.
  * This program is distributed in the hope that it will be useful,  but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
@@ -23,20 +21,28 @@
  * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
+#include <iomanip>
+#include <iostream>
+#include <map>
 #include <memory>
+#include <sstream>
 #include <string>
+#include <vector>
+
+#include "modules/adminapi/cluster_set/cluster_set_impl.h"
 
-#include "db/utils_connection.h"
-#include "modules/adminapi/common/accounts.h"
 #include "modules/adminapi/common/common.h"
 #include "modules/adminapi/common/dba_errors.h"
+#include "modules/adminapi/common/sql.h"
 #include "modules/adminapi/mod_dba_cluster.h"
 #include "modules/adminapi/mod_dba_cluster_set.h"
+#include "modules/mysqlxtest_utils.h"
 #include "mysqlshdk/include/scripting/type_info/custom.h"
+#include "mysqlshdk/include/scripting/type_info/generic.h"
 #include "mysqlshdk/include/shellcore/utils_help.h"
 #include "mysqlshdk/libs/utils/debug.h"
 #include "mysqlshdk/libs/utils/utils_general.h"
-#include "mysqlshdk/libs/utils/utils_json.h"
+#include "mysqlshdk/shellcore/shell_console.h"
 
 using std::placeholders::_1;
 
@@ -47,7 +53,11 @@ namespace dba {
 using mysqlshdk::db::uri::formats::only_transport;
 using mysqlshdk::db::uri::formats::user_transport;
 // Documentation of the Cluster Class
-REGISTER_HELP_CLASS(Cluster, adminapi);
+REGISTER_HELP_CLASS_KW(
+    Cluster, adminapi,
+    (std::map<std::string, std::string>({{"FullType", "InnoDB Cluster"},
+                                         {"Type", "Cluster"},
+                                         {"type", "cluster"}})));
 REGISTER_HELP(CLUSTER_BRIEF, "Represents an InnoDB Cluster.");
 REGISTER_HELP(CLUSTER_DETAIL,
               "The cluster object is the entry point to manage and monitor "
@@ -76,23 +86,6 @@ Cluster::Cluster(const std::shared_ptr<Cluster_impl> &impl) : m_impl(impl) {
 
 Cluster::~Cluster() { DEBUG_OBJ_DEALLOC(Cluster); }
 
-std::string &Cluster::append_descr(std::string &s_out, int UNUSED(indent),
-                                   int UNUSED(quote_strings)) const {
-  s_out.append("<" + class_name() + ":" + impl()->get_name() + ">");
-  return s_out;
-}
-
-void Cluster::append_json(shcore::JSON_dumper &dumper) const {
-  dumper.start_object();
-  dumper.append_string("class", class_name());
-  dumper.append_string("name", impl()->get_name());
-  dumper.end_object();
-}
-
-bool Cluster::operator==(const Object_bridge &other) const {
-  return class_name() == other.class_name() && this == &other;
-}
-
 void Cluster::init() {
   add_property("name", "getName");
   expose("addInstance", &Cluster::add_instance, "instance", "?options")->cli();
@@ -108,11 +101,6 @@ void Cluster::init() {
       ->cli();
   expose("checkInstanceState", &Cluster::check_instance_state, "instance")
       ->cli();
-  expose("setupAdminAccount", &Cluster::setup_admin_account, "user", "?options")
-      ->cli();
-  expose("setupRouterAccount", &Cluster::setup_router_account, "user",
-         "?options")
-      ->cli();
   expose("rescan", &Cluster::rescan, "?options")->cli();
   expose("forceQuorumUsingPartitionOf",
          &Cluster::force_quorum_using_partition_of, "instance", "?password")
@@ -130,9 +118,13 @@ void Cluster::init() {
   expose("setInstanceOption", &Cluster::set_instance_option, "instance",
          "option", "value")
       ->cli();
-  expose("listRouters", &Cluster::list_routers, "?options")->cli();
   expose("removeRouterMetadata", &Cluster::remove_router_metadata, "router")
       ->cli();
+  expose("routingOptions", &Cluster::routing_options, "?router")->cli();
+  expose("setRoutingOption", &Cluster::set_routing_option, "option", "value");
+  expose("setRoutingOption", &Cluster::set_routing_option, "router", "option",
+         "value");
+
   expose("createClusterSet", &Cluster::create_cluster_set, "domainName",
          "?options")
       ->cli();
@@ -140,6 +132,16 @@ void Cluster::init() {
   expose("fenceAllTraffic", &Cluster::fence_all_traffic)->cli();
   expose("fenceWrites", &Cluster::fence_writes)->cli();
   expose("unfenceWrites", &Cluster::unfence_writes)->cli();
+
+  expose("listRouters", &Cluster::list_routers, "?options")->cli();
+  expose("setupAdminAccount", &Cluster::setup_admin_account, "user", "?options")
+      ->cli();
+  expose("setupRouterAccount", &Cluster::setup_router_account, "user",
+         "?options")
+      ->cli();
+  expose("addReplicaInstance", &Cluster::add_replica_instance, "instance",
+         "?options")
+      ->cli();
 }
 
 // Documentation of the getName function
@@ -163,52 +165,24 @@ String Cluster::getName() {}
 str Cluster::get_name() {}
 #endif
 
-shcore::Value Cluster::get_member(const std::string &prop) const {
-  // Throw an error if the cluster has already been dissolved
-  assert_valid(prop);
-
-  if (prop == "name") return shcore::Value(impl()->get_name());
-  return shcore::Cpp_object_bridge::get_member(prop);
-}
-
 void Cluster::assert_valid(const std::string &option_name) const {
   std::string name;
-  bool obj_disconnected = false;
 
   if (option_name == "disconnect" || option_name == "name") return;
 
   if (has_member(option_name) && m_invalidated) {
     if (has_method(option_name)) {
       name = get_function_name(option_name, false);
-      throw shcore::Exception::runtime_error(class_name() + "." + name + ": " +
-                                             "Can't call function '" + name +
+      throw shcore::Exception::runtime_error("Can't call function '" + name +
                                              "' on an offline Cluster");
     } else {
       name = get_member_name(option_name, shcore::current_naming_style());
-      throw shcore::Exception::runtime_error(class_name() + "." + name + ": " +
-                                             "Can't access object member '" +
+      throw shcore::Exception::runtime_error("Can't access object member '" +
                                              name + "' on an offline Cluster");
     }
   }
 
-  if (!impl()->get_cluster_server() ||
-      !impl()->get_cluster_server()->get_session() ||
-      !impl()->get_cluster_server()->get_session()->is_open()) {
-    obj_disconnected = true;
-  } else {
-    // Check if the server is still reachable even though the session is open
-    try {
-      impl()->get_cluster_server()->execute("select 1");
-    } catch (const shcore::Error &e) {
-      if (mysqlshdk::db::is_mysql_client_error(e.code())) {
-        obj_disconnected = true;
-      } else {
-        throw;
-      }
-    }
-  }
-
-  if (obj_disconnected) {
+  if (!impl()->check_valid()) {
     throw shcore::Exception::runtime_error(
         "The cluster object is disconnected. Please use dba." +
         get_function_name("getCluster", false) +
@@ -235,9 +209,7 @@ The options dictionary may contain the following attributes:
 @li recoveryMethod: Preferred method of state recovery. May be auto, clone or
 incremental. Default is auto.
 @li waitRecovery: Integer value to indicate if the command shall wait for the
-recovery process to finish and its verbosity level. Deprecated.
-@li recoveryProgress: Integer value to indicate the recovery process verbosity
-level.
+recovery process to finish and its verbosity level.
 @li password: the instance connection password
 @li memberSslMode: SSL mode used on the instance
 ${CLUSTER_OPT_IP_WHITELIST}
@@ -277,12 +249,6 @@ The waitRecovery option supports the following values:
 @li 3: block until the recovery process finishes and show progress using
 progress bars.
 
-The recoveryProgress option supports the following values:
-
-@li 0: do not show any progress information.
-@li 1: show detailed static progress information.
-@li 2: show detailed dynamic progress information using progress bars.
-
 By default, if the standard output on which the Shell is running refers to a
 terminal, the waitRecovery option has the value of 3. Otherwise, it has the
 value of 2.
@@ -312,13 +278,6 @@ ${CLUSTER_OPT_AUTO_REJOIN_TRIES_EXTRA}
 Please use the ipAllowlist option instead.
 
 @attention The groupSeeds option will be removed in a future release.
-
-@attention The waitRecovery option will be removed in a future release.
-Please use the recoveryProgress option instead.
-
-@attention The interactive option will be removed in a future release.
-
-@attention The password option will be removed in a future release.
 )*");
 
 /**
@@ -381,15 +340,53 @@ ${TOPIC_CONNECTION_MORE_INFO}
 The options dictionary may contain the following attributes:
 
 @li password: the instance connection password
+@li recoveryMethod: Preferred method of state recovery. May be auto, clone or
+incremental. Default is auto.
+@li recoveryProgress: Integer value to indicate the recovery process verbosity
+level.
+recovery process to finish and its verbosity level.
 @li memberSslMode: SSL mode used on the instance
 ${OPT_INTERACTIVE}
 ${CLUSTER_OPT_IP_WHITELIST}
 ${CLUSTER_OPT_IP_ALLOWLIST}
 ${CLUSTER_OPT_LOCAL_ADDRESS}
+@li dryRun: boolean if true, all validations and steps for rejoining the
+instance are executed, but no changes are actually made.
+${CLUSTER_OPT_CLONE_DONOR}
+@li timeout: maximum number of seconds to wait for the instance to sync up
+with the PRIMARY after it's provisioned and the replication channel is
+established. If reached, the operation is rolled-back. Default is 0 (no
+timeout).
 
 The password may be contained on the instance definition, however, it can be
 overwritten if it is specified on the options.
 
+The recoveryMethod option supports the following values:
+
+@li incremental: uses distributed state recovery, which applies missing
+transactions copied from another cluster member. Clone will be disabled.
+@li clone: clone: uses built-in MySQL clone support, which completely replaces
+the state of the target instance with a full snapshot of another cluster member
+before distributed recovery starts. Requires MySQL 8.0.17 or newer.
+@li auto: let Group Replication choose whether or not a full snapshot has to be
+taken, based on what the target server supports and the
+group_replication_clone_threshold sysvar.
+This is the default value. A prompt will be shown if not possible to safely
+determine a safe way forward. If interaction is disabled, the operation will be
+canceled instead.
+
+If recoveryMethod is not specified 'auto' will be used by default.
+
+The recoveryProgress option supports the following values:
+
+@li 0: do not show any progress information.
+@li 1: show detailed static progress information.
+@li 2: show detailed dynamic progress information using progress bars.
+
+By default, if the standard output on which the Shell is running refers to a
+terminal, the recoveryProgress option has the value of 2. Otherwise, it has the
+value of 1.
+
 ${CLUSTER_OPT_IP_ALLOWLIST_EXTRA}
 
 The localAddress is an advanced option and its usage is discouraged
@@ -401,10 +398,6 @@ ${CLUSTER_OPT_LOCAL_ADDRESS_EXTRA}
 
 @attention The ipWhitelist option will be removed in a future release.
 Please use the ipAllowlist option instead.
-
-@attention The interactive option will be removed in a future release.
-
-@attention The password option will be removed in a future release.
 )*");
 
 /**
@@ -459,6 +452,12 @@ The options dictionary may contain the following attributes:
 @li force: boolean, indicating if the instance must be removed (even if only
 from metadata) in case it cannot be reached. By default, set to false.
 ${OPT_INTERACTIVE}
+@li dryRun: boolean if true, all validations and steps for removing the
+instance are executed, but no changes are actually made. An exception will be
+thrown when finished.
+@li timeout: maximum number of seconds to wait for the instance to sync up
+with the PRIMARY. If reached, the operation is rolled-back. Default is 0 (no
+timeout).
 
 The password may be contained in the instance definition, however, it can be
 overwritten if it is specified on the options.
@@ -469,10 +468,6 @@ a cluster. This allows to remove from the metadata an instance than can no
 longer be recovered. Otherwise, the instance must be brought back ONLINE and
 removed without the force option to avoid errors trying to add it back to a
 cluster.
-
-@attention The interactive option will be removed in a future release.
-
-@attention The password option will be removed in a future release.
 )*");
 
 /**
@@ -673,8 +668,6 @@ from the metadata, including instances than can no longer be recovered.
 Otherwise, the instances must be brought back ONLINE and the cluster dissolved
 without the force option to avoid errors trying to reuse the instances and add
 them back to a cluster.
-
-@attention The interactive option will be removed in a future release.
 )*");
 
 /**
@@ -729,8 +722,6 @@ if really needed when instances are permanently not available (no longer
 reachable) or never going to be reused again in a cluster. Prefer to bring the
 non available instances back ONLINE or remove them from the cluster if they will
 no longer be used.
-
-@attention The interactive option will be removed in a future release.
 )*");
 
 /**
@@ -787,8 +778,6 @@ communication protocol to the highest version possible.
 @li updateViewChangeUuid: boolean value used to indicate if the command should
 generate and set a value for Group Replication View Change UUID in the whole
 Cluster. Required for InnoDB ClusterSet usage.
-@li repairMetadata: boolean. Set to true to repair the Metadata if detected to
-be inconsistent.
 
 The value for addInstances and removeInstances is used to specify which
 instances to add or remove from the metadata, respectively. Both options accept
@@ -796,14 +785,7 @@ list connection data. In addition, the "auto" value can be used for both
 options in order to automatically add or remove the instances in the metadata,
 without having to explicitly specify them.
 
-'repairMetadata' is used to eliminate any inconsistencies detected in the
-Metadata. These inconsistencies may arise from a few scenarios, such as the
-failure of one or more commands. Clusters detected in the ClusterSet Metadata
-that do not qualify as valid members will be removed.
-
 @attention The updateTopologyMode option will be removed in a future release.
-
-@attention The interactive option will be removed in a future release.
 )*");
 
 /**
@@ -870,8 +852,6 @@ in the network, but not accessible from your location.
 
 When this function is used, all the members that are ONLINE from the point of
 view of the given instance definition will be added to the group.
-
-@attention The password option will be removed in a future release.
 )*");
 
 /**
@@ -892,13 +872,6 @@ void Cluster::force_quorum_using_partition_of(
   auto instance_def = instance_def_;
   set_password_from_string(&instance_def, password);
 
-  if (password) {
-    handle_deprecated_option(mysqlshdk::db::kPassword, "");
-
-    DBUG_EXECUTE_IF("dba_deprecated_option_fail",
-                    { throw std::logic_error("debug"); });
-  }
-
   // Throw an error if the cluster has already been dissolved
   assert_valid("forceQuorumUsingPartitionOf");
 
@@ -925,9 +898,6 @@ Verifies the instance gtid state in relation to the cluster.
 
 @returns resultset A JSON object with the status.
 
-@attention This function is deprecated and will be removed in a future release
-of MySQL Shell.
-
 Analyzes the instance executed GTIDs with the executed/purged GTIDs on the
 cluster to determine if the instance is valid for the cluster.
 
@@ -971,10 +941,6 @@ shcore::Value Cluster::check_instance_state(
     const Connection_options &instance_def) {
   assert_valid("checkInstanceState");
 
-  mysqlsh::current_console()->print_warning(
-      "This function is deprecated and will be removed in a future release of "
-      "MySQL Shell.");
-
   return execute_with_pool(
       [&]() { return impl()->check_instance_state(instance_def); }, false);
 }
@@ -1208,6 +1174,7 @@ ${CLUSTER_OPT_MEMBER_WEIGHT}
 ${CLUSTER_OPT_AUTO_REJOIN_TRIES}
 ${CLUSTER_OPT_IP_ALLOWLIST}
 @li label a string identifier of the instance.
+@li replicationSources: The list of sources for a Read Replica Instance.
 
 ${CLUSTER_OPT_EXIT_STATE_ACTION_DETAIL}
 
@@ -1222,6 +1189,20 @@ ${CLUSTER_OPT_IP_ALLOWLIST_EXTRA}
 ${NAMESPACE_TAG_DETAIL_CLUSTER}
 
 ${NAMESPACE_TAG_INSTANCE_DETAILS_EXTRA}
+
+The replicationSources is a comma separated list of instances (host:port) to
+act as sources of the replication channel, i.e. to provide source failover of
+the channel. The first member of the list is configured with the highest
+priority among all members so when the channel activates it will be chosen for
+the first connection attempt. By default, the source list is automatically
+managed by Group Replication according to the current group membership and the
+primary member of the Cluster is the current source for the replication
+channel, this is the same as setting to "primary". Alternatively, it's
+possible to set to "secondary" to instruct Group Replication to automatically
+manage the list but use a secondary member of the Cluster as source.
+
+For the change to be effective, the Read-Replica must be reconfigured after
+using Cluster.<<<rejoinInstance>>>().
 )*");
 
 /**
@@ -1249,26 +1230,7 @@ void Cluster::set_instance_option(const Connection_options &instance_def,
 }
 
 REGISTER_HELP_FUNCTION(listRouters, Cluster);
-REGISTER_HELP_FUNCTION_TEXT(CLUSTER_LISTROUTERS, R"*(
-Lists the Router instances.
-
-@param options Optional dictionary with options for the operation.
-
-@returns A JSON object listing the Router instances associated to the cluster.
-
-This function lists and provides information about all Router instances
-registered for the cluster.
-
-Whenever a Metadata Schema upgrade is necessary, the recommended process
-is to upgrade MySQL Router instances to the latest version before upgrading
-the Metadata itself, in order to minimize service disruption.
-
-The options dictionary may contain the following attributes:
-
-@li onlyUpgradeRequired: boolean, enables filtering so only router instances
-that support older version of the Metadata Schema and require upgrade are
-included.
-)*");
+REGISTER_HELP_FUNCTION_TEXT(CLUSTER_LISTROUTERS, LISTROUTERS_HELP_TEXT);
 
 /**
  * $(CLUSTER_LISTROUTERS_BRIEF)
@@ -1280,17 +1242,94 @@ String Cluster::listRouters(Dictionary options) {}
 #elif DOXYGEN_PY
 str Cluster::list_routers(dict options) {}
 #endif
-shcore::Dictionary_t Cluster::list_routers(
-    const shcore::Option_pack_ref<List_routers_options> &options) {
-  // Throw an error if the cluster has already been dissolved
-  assert_valid("listRouters");
 
-  auto ret_val = execute_with_pool(
-      [&]() { return impl()->list_routers(options->only_upgrade_required); },
-      false);
+REGISTER_HELP_FUNCTION(setRoutingOption, Cluster);
+REGISTER_HELP_FUNCTION_TEXT(CLUSTER_SETROUTINGOPTION, R"*(
+Changes the value of either a global Cluster Routing option or of a single
+Router instance.
 
-  return ret_val.as_map();
-}
+@param router optional identifier of the target router instance (e.g.
+192.168.45.70@::system).
+@param option The Router option to be changed.
+@param value The value that the option shall get (or null to unset).
+
+@returns Nothing.
+
+The accepted options are:
+
+@li tags: Associates an arbitrary JSON object with custom key/value pairs with
+the Cluster metadata.
+@li read_only_targets: Routing policy to define Router's usage of Read Only
+instance. Default is 'secondaries'.
+@li stats_updates_frequency: Number of seconds between updates that the Router
+is to make to its statistics in the InnoDB Cluster metadata.
+
+The read_only_targets option supports the following values:
+
+@li all: All Read Replicas of the target Cluster should be used along the
+other SECONDARY Cluster members for R/O traffic.
+@li read_replicas: Only Read Replicas of the target Cluster should be used for
+R/O traffic.
+@li secondaries: Only Secondary members of the target Cluster should be used
+for R/O traffic (default).
+
+The stats_updates_frequency option accepts positive integers and sets the
+frequency of updates of Router stats (timestamp, version, etc.), in seconds,
+in the Metadata. If set to 0 (default), no periodic updates are done. Router
+will round up the value to be a multiple of Router's TTL, i.e.:
+
+@li If lower than TTL its gets rounded up to TTL, e.g. TTL=30, and
+stats_updates_frequency=1, effective frequency is 30 seconds.
+
+@li If not a multiple of TTL it will be rounded up and adjusted according to
+the TTL, e.g. TTL=5, stats_updates_frequency=11, effective frequency is 15
+seconds; TTL=5, stats_updates_frequency=13, effective frequency is 15 seconds.
+
+If the value is null, the option value is cleared and the default value (0)
+takes effect.
+)*");
+
+#if DOXYGEN_JS
+Undefined Cluster::setRoutingOption(String option, String value) {}
+#elif DOXYGEN_PY
+None Cluster::set_routing_option(str option, str value) {}
+#endif
+
+/**
+ * $(CLUSTER_SETROUTINGOPTION_BRIEF)
+ *
+ * $(CLUSTER_SETROUTINGOPTION)
+ */
+#if DOXYGEN_JS
+Undefined Cluster::setRoutingOption(String router, String option,
+                                    String value) {}
+#elif DOXYGEN_PY
+None Cluster::set_routing_option(str router, str option, str value) {}
+#endif
+
+REGISTER_HELP_FUNCTION(routingOptions, Cluster);
+REGISTER_HELP_FUNCTION_TEXT(CLUSTER_ROUTINGOPTIONS, R"*(
+Lists the Cluster Routers configuration options.
+
+@param router Optional identifier of the router instance to query for the options.
+
+@returns A JSON object describing the configuration options of all router
+instances of the Cluster and its global options or just the given Router.
+
+This function lists the Router configuration options of all Routers of the
+Cluster or the target Router.
+)*");
+
+/**
+ * $(CLUSTER_ROUTINGOPTIONS_BRIEF)
+ *
+ * $(CLUSTER_ROUTINGOPTIONS)
+ */
+#if DOXYGEN_JS
+Dictionary Cluster::routingOptions(String router) {}
+#elif DOXYGEN_PY
+dict Cluster::routing_options(str router) {}
+#endif
 
 REGISTER_HELP_FUNCTION(removeRouterMetadata, Cluster);
 REGISTER_HELP_FUNCTION_TEXT(CLUSTER_REMOVEROUTERMETADATA, R"*(
@@ -1323,53 +1362,12 @@ void Cluster::remove_router_metadata(const std::string &router_def) {
   assert_valid("removeRouterMetadata");
 
   return execute_with_pool(
-      [&]() { impl()->remove_router_metadata(router_def); }, false);
+      [&]() { impl()->remove_router_metadata(router_def, true); }, false);
 }
 
 REGISTER_HELP_FUNCTION(setupAdminAccount, Cluster);
-REGISTER_HELP_FUNCTION_TEXT(CLUSTER_SETUPADMINACCOUNT, R"*(
-Create or upgrade an InnoDB Cluster admin account.
-
-@param user Name of the InnoDB cluster administrator account.
-@param options Dictionary with options for the operation.
-
-@returns Nothing.
-
-This function creates/upgrades a MySQL user account with the necessary
-privileges to administer an InnoDB cluster.
-
-This function also allows a user to upgrade an existing admin account
-with the necessary privileges before a dba.<<<upgradeMetadata>>>() call.
-
-The mandatory argument user is the name of the MySQL account we want to create
-or upgrade to be used as Administrator account. The accepted format is
-username[@@host] where the host part is optional and if not provided defaults to
-'%'.
-
-The options dictionary may contain the following attributes:
-
-@li password: The password for the InnoDB cluster administrator account.
-${OPT_SETUP_ACCOUNT_OPTIONS_PASSWORD_EXPIRATION}
-${OPT_SETUP_ACCOUNT_OPTIONS_REQUIRE_CERT_ISSUER}
-${OPT_SETUP_ACCOUNT_OPTIONS_REQUIRE_CERT_SUBJECT}
-${OPT_SETUP_ACCOUNT_OPTIONS_DRY_RUN}
-${OPT_INTERACTIVE}
-${OPT_SETUP_ACCOUNT_OPTIONS_UPDATE}
-
-If the user account does not exist, either the password, requireCertIssuer or
-requireCertSubject are mandatory.
-
-If the user account exists, the update option must be enabled.
-
-${OPT_SETUP_ACCOUNT_OPTIONS_DRY_RUN_DETAIL}
-
-The interactive option can be used to explicitly enable or disable the
-interactive prompts that help the user through the account setup process.
-
-${OPT_SETUP_ACCOUNT_OPTIONS_UPDATE_DETAIL}
-
-@attention The interactive option will be removed in a future release.
-)*");
+REGISTER_HELP_FUNCTION_TEXT(CLUSTER_SETUPADMINACCOUNT,
+                            SETUPADMINACCOUNT_HELP_TEXT);
 
 /**
  * $(CLUSTER_SETUPADMINACCOUNT_BRIEF)
@@ -1382,63 +1380,9 @@ Undefined Cluster::setupAdminAccount(String user, Dictionary options) {}
 None Cluster::setup_admin_account(str user, dict options) {}
 #endif
 
-void Cluster::setup_admin_account(
-    const std::string &user,
-    const shcore::Option_pack_ref<Setup_account_options> &options) {
-  // Throw an error if the cluster has already been dissolved
-  assert_valid("setupAdminAccount");
-
-  // split user into user/host
-  std::string username, host;
-  std::tie(username, host) = validate_account_name(user);
-
-  return execute_with_pool(
-      [&]() { impl()->setup_admin_account(username, host, *options); }, false);
-}
-
 REGISTER_HELP_FUNCTION(setupRouterAccount, Cluster);
-REGISTER_HELP_FUNCTION_TEXT(CLUSTER_SETUPROUTERACCOUNT, R"*(
-Create or upgrade a MySQL account to use with MySQL Router.
-
-@param user Name of the account to create/upgrade for MySQL Router.
-@param options Dictionary with options for the operation.
-
-@returns Nothing.
-
-This function creates/upgrades a MySQL user account with the necessary
-privileges to be used by MySQL Router.
-
-This function also allows a user to upgrade existing MySQL router accounts
-with the necessary privileges after a dba.<<<upgradeMetadata>>>() call.
-
-The mandatory argument user is the name of the MySQL account we want to create
-or upgrade to be used by MySQL Router. The accepted format is
-username[@@host] where the host part is optional and if not provided defaults to
-'%'.
-
-The options dictionary may contain the following attributes:
-
-@li password: The password for the MySQL Router account.
-${OPT_SETUP_ACCOUNT_OPTIONS_PASSWORD_EXPIRATION}
-${OPT_SETUP_ACCOUNT_OPTIONS_REQUIRE_CERT_ISSUER}
-${OPT_SETUP_ACCOUNT_OPTIONS_REQUIRE_CERT_SUBJECT}
-${OPT_SETUP_ACCOUNT_OPTIONS_DRY_RUN}
-${OPT_INTERACTIVE}
-${OPT_SETUP_ACCOUNT_OPTIONS_UPDATE}
-
-If the user account does not exist, either the password, requireCertIssuer or
-requireCertSubject are mandatory.
-
-If the user account exists, the update option must be enabled.
-
-${OPT_SETUP_ACCOUNT_OPTIONS_DRY_RUN_DETAIL}
-
-${OPT_SETUP_ACCOUNT_OPTIONS_INTERACTIVE_DETAIL}
-
-${OPT_SETUP_ACCOUNT_OPTIONS_UPDATE_DETAIL}
-
-@attention The interactive option will be removed in a future release.
-)*");
+REGISTER_HELP_FUNCTION_TEXT(CLUSTER_SETUPROUTERACCOUNT,
+                            SETUPROUTERACCOUNT_HELP_TEXT);
 
 /**
  * $(CLUSTER_SETUPROUTERACCOUNT_BRIEF)
@@ -1450,19 +1394,6 @@ Undefined Cluster::setupRouterAccount(String user, Dictionary options) {}
 #elif DOXYGEN_PY
 None Cluster::setup_router_account(str user, dict options) {}
 #endif
-void Cluster::setup_router_account(
-    const std::string &user,
-    const shcore::Option_pack_ref<Setup_account_options> &options) {
-  // Throw an error if the cluster has already been dissolved
-  assert_valid("setupRouterAccount");
-
-  // split user into user/host
-  std::string username, host;
-  std::tie(username, host) = validate_account_name(user);
-
-  return execute_with_pool(
-      [&]() { impl()->setup_router_account(username, host, *options); }, false);
-}
 
 REGISTER_HELP_FUNCTION(fenceAllTraffic, Cluster);
 REGISTER_HELP_FUNCTION_TEXT(CLUSTER_FENCEALLTRAFFIC, R"*(
@@ -1699,5 +1630,109 @@ std::shared_ptr<ClusterSet> Cluster::get_cluster_set() {
   return impl()->get_cluster_set();
 }
 
+// Read-Replicas
+
+REGISTER_HELP_FUNCTION(addReplicaInstance, Cluster);
+REGISTER_HELP_FUNCTION_TEXT(CLUSTER_ADDREPLICAINSTANCE, R"*(
+Adds a Read Replica Instance to the Cluster.
+
+@param instance host:port of the target instance to be added as a Read Replica.
+@param options optional Dictionary with additional parameters described below.
+
+@returns nothing
+
+This function adds an Instance acting as Read-Replica of an InnoDB Cluster.
+
+<b>Pre-requisites</b>
+
+The following is a list of requirements to create a REPLICA cluster:
+
+@li The target instance must be a standalone instance.
+@li The target instance and Cluster must running MySQL 8.0.23 or newer.
+@li Unmanaged replication channels are not allowed.
+@li The target instance server_id and server_uuid must be unique in the
+topology, including among OFFLINE or unreachable members
+
+<b>Options</b>
+
+The options dictionary may contain the following values:
+
+@li dryRun: boolean if true, all validations and steps for creating a
+Read Replica Instance are executed, but no changes are actually made. An
+exception will be thrown when finished.
+@li label: an identifier for the Read Replica Instance being added, used in
+the output of status() and describe().
+@li replicationSources: The list of sources for the Read Replica Instance. By
+default, the list is automatically managed by Group Replication and the
+primary member is used as source.
+@li recoveryMethod: Preferred method for state recovery/provisioning. May be
+auto, clone or incremental. Default is auto.
+@li recoveryProgress: Integer value to indicate the recovery process verbosity
+level.
+@li timeout: maximum number of seconds to wait for the instance to sync up
+with the PRIMARY after it's provisioned and the replication channel is
+established. If reached, the operation is rolled-back. Default is 0 (no
+timeout).
+${CLUSTER_OPT_CLONE_DONOR}
+
+The replicationSources is a comma separated list of instances (host:port) to
+act as sources of the replication channel, i.e. to provide source failover of
+the channel. The first member of the list is configured with the highest
+priority among all members so when the channel activates it will be chosen for
+the first connection attempt. By default, the source list is automatically
+managed by Group Replication according to the current group membership and the
+primary member of the Cluster is the current source for the replication
+channel, this is the same as setting to "primary". Alternatively, it's
+possible to set to "secondary" to instruct Group Replication to automatically
+manage the list too but use a secondary member of the Cluster as source.
+
+The recoveryMethod option supports the following values:
+
+@li incremental: waits until the new instance has applied missing transactions
+from the source.
+@li clone: uses MySQL clone to provision the instance, which completely
+replaces the state of the target instance with a full snapshot of the
+instance's source.
+@li auto: compares the transaction set of the instance with that of the
+source to determine if incremental recovery is safe to be automatically
+chosen as the most appropriate recovery method.
+A prompt will be shown if not possible to safely determine a safe way forward.
+If interaction is disabled, the operation will be canceled instead.
+
+If recoveryMethod is not specified 'auto' will be used by default.
+
+The recoveryProgress option supports the following values:
+
+@li 0: do not show any progress information.
+@li 1: show detailed static progress information.
+@li 2: show detailed dynamic progress information using progress bars.
+
+By default, if the standard output on which the Shell is running refers to a
+terminal, the recoveryProgress option has the value of 2. Otherwise, it has the
+value of 1.
+)*");
+
+/**
+ * $(CLUSTER_ADDREPLICAINSTANCE_BRIEF)
+ *
+ * $(CLUSTER_ADDREPLICAINSTANCE)
+ */
+#if DOXYGEN_JS
+Undefined Cluster::addReplicaInstance(InstanceDef instance,
+                                      Dictionary options) {}
+#elif DOXYGEN_PY
+None Cluster::add_replica_instance(InstanceDef instance, dict options);
+#endif
+void Cluster::add_replica_instance(
+    const std::string &instance_def,
+    const shcore::Option_pack_ref<cluster::Add_replica_instance_options>
+        &options) {
+  assert_valid("addReplicaInstance");
+
+  return execute_with_pool(
+      [&]() { return impl()->add_replica_instance(instance_def, *options); },
+      false);
+}
+
 }  // namespace dba
 }  // namespace mysqlsh
