@@ -1,18 +1,16 @@
 /*
- * Copyright (c) 2014, 2024, Oracle and/or its affiliates.
+ * Copyright (c) 2014, 2023, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms,
- * as designated in a particular file or component or in included license
+ * This program is also distributed with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
- *
+ * separately licensed software that they have included with MySQL.
  * This program is distributed in the hope that it will be useful,  but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
@@ -1029,7 +1027,9 @@ std::shared_ptr<mysqlshdk::db::mysqlx::Result> Session::execute_stmt(
 }
 
 std::shared_ptr<mysqlshdk::db::IResult> Session::execute_sql(
-    const std::string &statement, const shcore::Array_t &args) {
+    const std::string &statement, const shcore::Array_t &args,
+    [[maybe_unused]] const std::vector<mysqlshdk::db::Query_attribute>
+        &query_attributes) {
   try {
     return execute_stmt("sql", statement, convert_args(args));
   } catch (const mysqlshdk::db::Error &error) {
