@@ -1,18 +1,16 @@
 /*
- * Copyright (c) 2020, 2024, Oracle and/or its affiliates.
+ * Copyright (c) 2020, 2023, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms,
- * as designated in a particular file or component or in included license
+ * This program is also distributed with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
- *
+ * separately licensed software that they have included with MySQL.
  * This program is distributed in the hope that it will be useful,  but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
@@ -27,9 +25,11 @@
 #define MODULES_UTIL_DUMP_DUMPER_H_
 
 #include <atomic>
+#include <condition_variable>
 #include <functional>
 #include <memory>
 #include <mutex>
+#include <queue>
 #include <string>
 #include <string_view>
 #include <thread>
@@ -67,10 +67,16 @@ class Dumper {
   Dumper &operator=(const Dumper &) = delete;
   Dumper &operator=(Dumper &&) = delete;
 
-  virtual ~Dumper();
+  virtual ~Dumper() = default;
 
   void run();
 
+  void interrupt();
+
+  void abort();
+
+  Progress_thread *progress() { return &m_progress_thread; }
+
  protected:
   static inline std::shared_ptr<mysqlshdk::db::IResult> query(
       const std::shared_ptr<mysqlshdk::db::ISession> &session,
@@ -87,7 +93,7 @@ class Dumper {
   template <typename... Args>
   static inline void executef(
       const std::shared_ptr<mysqlshdk::db::ISession> &session, const char *sql,
-      Args &&...args) {
+      Args &&... args) {
     session->executef(sql, std::forward<Args>(args)...);
   }
 
@@ -120,11 +126,6 @@ class Dumper {
     std::string basename;
   };
 
-  struct Index_info {
-    const Instance_cache::Index *info = nullptr;
-    bool is_pke = false;
-  };
-
   struct Partition_info {
     std::string basename;
     const Instance_cache::Partition *info = nullptr;
@@ -148,7 +149,6 @@ class Dumper {
     std::string task_name;
     std::string schema;
     std::string where;
-    Index_info index;
   };
 
   struct Table_data_task : Table_task {
@@ -163,8 +163,6 @@ class Dumper {
     std::function<void(Table_worker *)> task;
   };
 
-  class Synchronize_workers;
-
   class Dump_info;
 
   class Memory_dumper;
@@ -224,13 +222,15 @@ class Dumper {
 
   void initialize_dump();
 
+  void finalize_dump();
+
   void create_output_directory();
 
   void close_output_directory();
 
-  void create_worker_threads();
+  void create_worker_sessions();
 
-  void wait_for_workers();
+  void create_worker_threads();
 
   void maybe_push_shutdown_tasks();
 
@@ -287,7 +287,7 @@ class Dumper {
 
   void push_table_chunking_task(Table_task &&task);
 
-  bool should_dump_data(const Table_task &table);
+  bool should_dump_data(const Table_task &table) const;
 
   std::unique_ptr<Dump_writer_controller> table_dump_controller(
       const std::string &filename) const;
@@ -324,7 +324,7 @@ class Dumper {
                                       const std::size_t idx,
                                       const bool last_chunk) const;
 
-  void initialize_progress();
+  void initialize_throughput_progress();
 
   void update_progress(const Dump_write_result &progress);
 
@@ -370,6 +370,11 @@ class Dumper {
 
   void throw_if_cannot_dump_users() const;
 
+  shcore::Synchronized_queue<std::shared_ptr<mysqlshdk::db::ISession>>
+      &session_pool() {
+    return m_session_pool;
+  }
+
   // session
   std::shared_ptr<mysqlshdk::db::ISession> m_session;
   std::vector<std::shared_ptr<mysqlshdk::db::ISession>> m_lock_sessions;
@@ -435,10 +440,10 @@ class Dumper {
   // threads
   std::vector<std::thread> m_workers;
   std::vector<std::exception_ptr> m_worker_exceptions;
+  std::atomic<bool> m_worker_exception_thrown = false;
   shcore::Synchronized_queue<Task_info> m_worker_tasks;
   std::atomic<uint64_t> m_chunking_tasks;
   std::atomic<bool> m_main_thread_finished_producing_chunking_tasks;
-  std::unique_ptr<Synchronize_workers> m_worker_synchronization;
   std::function<std::unique_ptr<Dump_writer>()> m_writer_creator;
   volatile bool m_worker_interrupt = false;
 
@@ -448,6 +453,9 @@ class Dumper {
   mutable Progress_thread m_progress_thread;
   mutable Progress_thread::Stage *m_current_stage = nullptr;
   Progress_thread::Stage *m_data_dump_stage = nullptr;
+
+  shcore::Synchronized_queue<std::shared_ptr<mysqlshdk::db::ISession>>
+      m_session_pool;
 };
 
 }  // namespace dump
