@@ -1,18 +1,16 @@
 /*
- * Copyright (c) 2020, 2024, Oracle and/or its affiliates.
+ * Copyright (c) 2020, 2023, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms,
- * as designated in a particular file or component or in included license
+ * This program is also distributed with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
- *
+ * separately licensed software that they have included with MySQL.
  * This program is distributed in the hope that it will be useful,  but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
@@ -36,11 +34,15 @@
 #include <unordered_set>
 #include <vector>
 
+#include "modules/util/common/dump/filtering_options.h"
+#include "mysqlshdk/libs/db/column.h"
 #include "mysqlshdk/libs/db/session.h"
 #include "mysqlshdk/libs/utils/utils_general.h"
 #include "mysqlshdk/libs/utils/version.h"
 
-#include "modules/util/common/dump/filtering_options.h"
+namespace mysqlshdk::db {
+class SHCORE_PUBLIC IRow;
+}
 
 namespace mysqlsh {
 namespace dump {
@@ -68,18 +70,26 @@ struct Instance_cache {
 
     ~Index() = default;
 
-    inline const std::vector<const Column *> &columns() const {
-      return m_columns;
-    }
+    inline bool valid() const noexcept { return !m_columns.empty(); }
+
+    inline const std::vector<Column *> &columns() const { return m_columns; }
 
     inline const std::string &columns_sql() const { return m_columns_sql; }
 
-    void add_column(const Column *column);
+    inline bool primary() const { return m_primary; }
+
+    void reset();
+
+    void add_column(Column *column);
+
+    void set_primary(bool primary);
 
    private:
-    std::vector<const Column *> m_columns;
+    std::vector<Column *> m_columns;
 
     std::string m_columns_sql;
+
+    bool m_primary = false;
   };
 
   struct Histogram {
@@ -100,12 +110,8 @@ struct Instance_cache {
     std::string engine;
     std::string create_options;
     std::string comment;
-    std::unordered_map<std::string, Index> indexes;
-    const Index *primary_key = nullptr;
-    // indexes are ordered to ensure repeatability of the selection algorithm
-    std::vector<const Index *> primary_key_equivalents;
-    std::vector<const Index *> unique_keys;
-    std::vector<const Column *> columns;
+    Index index;
+    std::vector<Column *> columns;
     std::vector<Column> all_columns;
     std::vector<Histogram> histograms;
     std::vector<std::string> triggers;  // order of triggers is important
