@@ -1,18 +1,16 @@
 /*
- * Copyright (c) 2018, 2025, Oracle and/or its affiliates.
+ * Copyright (c) 2018, 2023, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms,
- * as designated in a particular file or component or in included license
+ * This program is also distributed with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
- *
+ * separately licensed software that they have included with MySQL.
  * This program is distributed in the hope that it will be useful,  but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
@@ -40,14 +38,19 @@ using off64_t = off_t;
 #endif
 
 #include <mysql.h>
+
 #include <algorithm>
+#include <cinttypes>
 #include <memory>
 #include <utility>
+
 #include "modules/util/import_table/helpers.h"
 #include "mysqlshdk/include/shellcore/console.h"
 #include "mysqlshdk/include/shellcore/scoped_contexts.h"
 #include "mysqlshdk/include/shellcore/shell_init.h"
 #include "mysqlshdk/libs/rest/error.h"
+#include "mysqlshdk/libs/storage/backend/in_memory/synchronized_file.h"
+#include "mysqlshdk/libs/storage/backend/in_memory/virtual_file.h"
 #include "mysqlshdk/libs/utils/utils_path.h"
 #include "mysqlshdk/libs/utils/utils_string.h"
 
@@ -176,6 +179,7 @@ int Transaction_buffer::consume(char *buffer, unsigned int length) {
     }
 
     m_trx_size += length;
+    m_current_offset += length;
 
     if (length > m_options.max_trx_size) {
       // in a single read, we got more bytes than the transaction limit, either
@@ -198,6 +202,10 @@ int Transaction_buffer::read(char *buffer, unsigned int length) {
     return m_file->read(buffer, length);
   }
 
+  if (m_options.fast_sub_chunking) {
+    return fast_sub_chunking(buffer, length);
+  }
+
   const auto read_more = [this](unsigned int count) -> int64_t {
     int64_t bytes = 0;
 
@@ -347,6 +355,7 @@ uint64_t Transaction_buffer::find_last_row_boundary_before_impl_default(
 
   if (p == 0) return 0;
 
+  p = adjust_line_offset(p);
   p = m_data.rfind(needle, p);
 
   if (p >= m_data.length()) return 0;
@@ -378,6 +387,7 @@ uint64_t Transaction_buffer::find_last_row_boundary_before_impl_no_escape(
 
   if (p < needle.size()) return 0;
 
+  p = adjust_line_offset(p);
   p = m_data.rfind(needle, p);
 
   if (p >= m_data.length()) return 0;
@@ -416,6 +426,7 @@ uint64_t Transaction_buffer::find_last_row_boundary_before_impl_escape(
 
   if (p < needle.size()) return 0;
 
+  p = adjust_line_offset(p);
   p = m_data.rfind(needle, p);
 
   while (p != std::string::npos) {
@@ -434,6 +445,103 @@ uint64_t Transaction_buffer::find_last_row_boundary_before_impl_escape(
 
   return 0;
 }
+
+uint64_t Transaction_buffer::adjust_line_offset(uint64_t offset) {
+  // find boundary of the last row that will fit within the given limit
+  if (m_options.offsets) {
+    const auto size = m_options.offsets->size();
+    bool index_changed = false;
+
+    for (; m_offset_index < size; ++m_offset_index, index_changed = true) {
+      const auto &o = (*m_options.offsets)[m_offset_index];
+
+      if (o > m_current_offset) {
+        const auto new_offset = o - m_current_offset;
+
+        if (new_offset > offset) {
+          if (index_changed) {
+            --m_offset_index;
+          }
+
+          break;
+        }
+
+        return new_offset;
+      }
+    }
+  }
+
+  return offset;
+}
+
+int Transaction_buffer::fast_sub_chunking(char *buffer, unsigned int length) {
+  if (!m_data.empty()) {
+    // if data is stored locally, then a row didn't fit into an empty buffer,
+    // consume it first
+    return consume(buffer, length);
+  }
+
+  if (trx_bytes_left() <= 0) {
+    // the row we had stored was longer than the transaction size, or we read
+    // exactly that many bytes
+    return 0;
+  }
+
+  auto bytes =
+      m_file->read(buffer, std::min<uint64_t>(length, trx_bytes_left()));
+
+  if (0 == bytes) {
+    m_eof = true;
+    return bytes;
+  }
+
+  if (bytes < 0) {
+    // it was not possible to fit a single row into the buffer
+    if (m_trx_size) {
+      // we have some data in transaction, finish it, we'll see if this row fits
+      // into an empty buffer
+      return 0;
+    } else {
+      // row doesn't fit into an empty buffer, store it locally
+      const auto virtual_file =
+          dynamic_cast<mysqlshdk::storage::in_memory::Virtual_file *>(m_file);
+      assert(virtual_file);
+
+      if (!virtual_file) {
+        throw std::logic_error(
+            "Fast sub-chunking is only possible with virtual FS");
+      }
+
+      const auto handle =
+          dynamic_cast<mysqlshdk::storage::in_memory::Synchronized_file *>(
+              virtual_file->file());
+      assert(handle);
+
+      if (!handle) {
+        throw std::logic_error(
+            "Fast sub-chunking is only possible with synchronized I/O");
+      }
+
+      const auto row_length = handle->pending_write_size();
+
+      m_data.resize(row_length);
+      bytes = m_file->read(m_data.data(), row_length);
+
+      // this read should succeed
+      assert(static_cast<std::size_t>(bytes) == row_length);
+
+      if (bytes <= 0) {
+        return bytes;
+      }
+
+      return consume(buffer, length);
+    }
+  }
+
+  m_trx_size += bytes;
+  return bytes;
+}
+
 // ------
 
 int local_infile_init(void **buffer, const char * /* filename */,
@@ -576,6 +684,9 @@ void Load_data_worker::execute(
     const std::shared_ptr<mysqlshdk::db::mysql::Session> &session,
     std::unique_ptr<mysqlshdk::storage::IFile> file,
     const Transaction_options &options) {
+  const auto worker_name =
+      shcore::str_format("[Worker%03" PRId64 "]: ", m_thread_id);
+
   try {
     File_info fi;
     fi.worker_id = m_thread_id;
@@ -588,7 +699,7 @@ void Load_data_worker::execute(
       return session->query(sql);
     };
     const auto execute = [&session](const auto &sql) { session->execute(sql); };
-    const auto executef = [&session](const auto &sql, auto &&...args) {
+    const auto executef = [&session](const auto &sql, auto &&... args) {
       session->executef(sql, std::forward<decltype(args)>(args)...);
     };
 
@@ -714,10 +825,6 @@ void Load_data_worker::execute(
       query_columns.pop_back();  // strip the last ,
     }
 
-    char worker_name[64];
-    snprintf(worker_name, sizeof(worker_name), "[Worker%03u] ",
-             static_cast<unsigned int>(m_thread_id));
-
     uint64_t subchunk = 0;
     while (true) {
       if (!fi.continuation) {
@@ -731,8 +838,9 @@ void Load_data_worker::execute(
       const auto format_error_message = [&worker_name, &fi, &r](
                                             const std::string &error,
                                             const std::string &extra = {}) {
-        const auto task = fi.filehandler ? fi.filehandler->filename() : "";
-        auto msg = worker_name + task + ": " + error;
+        const auto task =
+            fi.filehandler ? fi.filehandler->filename() + ": " : "";
+        auto msg = worker_name + task + error;
 
         if (fi.range_read) {
           msg += " @ file bytes range [" + std::to_string(r.range.first) +
@@ -800,7 +908,7 @@ void Load_data_worker::execute(
         throw std::exception(e);
       }
 
-      const auto filename = fi.filehandler->filename();
+      const std::string task = fi.filehandler->filename();
       std::shared_ptr<mysqlshdk::db::IResult> load_result = nullptr;
       const std::string query_prefix = shcore::sqlformat(
           "LOAD DATA LOCAL INFILE ? ", fi.filehandler->full_path().masked());
@@ -808,7 +916,7 @@ void Load_data_worker::execute(
           query_prefix + query_body + query_ignore_lines + query_columns;
 
 #ifndef NDEBUG
-      log_debug("%s %s %i", worker_name, full_query.c_str(),
+      log_debug("%s %s %i", worker_name.c_str(), full_query.c_str(),
                 m_range_queue != nullptr);
 #endif
 
@@ -862,8 +970,7 @@ void Load_data_worker::execute(
       {
         const char *mysql_info = session->get_mysql_info();
         const auto status =
-            worker_name + filename + ": " +
-            (mysql_info ? mysql_info : "ERROR") +
+            worker_name + task + ": " + (mysql_info ? mysql_info : "ERROR") +
             ((options.max_trx_size == 0 && max_trx_size == 0)
                  ? ""
                  : (fi.continuation
@@ -893,54 +1000,49 @@ void Load_data_worker::execute(
         }
 
         if (warnings_num > 0) {
-          // number of warnings to show
-          constexpr std::size_t k_warnings_to_show = 5;
-          std::size_t warnings_count = 0;
+          // show first k warnings, where k = warnings_to_show
+          constexpr int warnings_to_show = 5;
+          auto w = load_result->fetch_one_warning();
 
-          while (const auto w = load_result->fetch_one_warning()) {
-            const std::string msg = shcore::str_format(
-                "LOAD DATA error for `%s`.`%s` in file '%s', error %" PRIu32
-                ": %s",
-                m_opt.schema().c_str(), m_opt.table().c_str(), filename.c_str(),
-                w->code, w->msg.c_str());
+          for (int i = 0; w && i < warnings_to_show;
+               w = load_result->fetch_one_warning(), i++) {
+            const std::string msg =
+                task + " error " + std::to_string(w->code) + ": " + w->msg;
 
-            if (warnings_count++ < k_warnings_to_show) {
-              // show first k warnings
-              switch (w->level) {
-                case mysqlshdk::db::Warning::Level::Error:
-                  mysqlsh::current_console()->print_error(msg);
-                  break;
-
-                case mysqlshdk::db::Warning::Level::Warn:
-                  mysqlsh::current_console()->print_warning(msg);
-                  break;
-
-                case mysqlshdk::db::Warning::Level::Note:
-                  mysqlsh::current_console()->print_note(msg);
-                  break;
-              }
-            } else {
-              // log remaining warnings
-              switch (w->level) {
-                case mysqlshdk::db::Warning::Level::Error:
-                  log_error("%s", msg.c_str());
-                  break;
-
-                case mysqlshdk::db::Warning::Level::Warn:
-                  log_warning("%s", msg.c_str());
-                  break;
-
-                case mysqlshdk::db::Warning::Level::Note:
-                  log_info("%s", msg.c_str());
-                  break;
-              }
+            switch (w->level) {
+              case mysqlshdk::db::Warning::Level::Error:
+                mysqlsh::current_console()->print_error(msg);
+                break;
+              case mysqlshdk::db::Warning::Level::Warn:
+                mysqlsh::current_console()->print_warning(msg);
+                break;
+              case mysqlshdk::db::Warning::Level::Note:
+                mysqlsh::current_console()->print_note(msg);
+                break;
             }
           }
 
-          if (warnings_count > k_warnings_to_show) {
-            const auto remaining_warnings_count =
-                warnings_count - k_warnings_to_show;
+          // log remaining warnings
+          size_t remaining_warnings_count = 0;
+          for (; w; w = load_result->fetch_one_warning()) {
+            remaining_warnings_count++;
+            const std::string msg =
+                task + " error " + std::to_string(w->code) + ": " + w->msg;
 
+            switch (w->level) {
+              case mysqlshdk::db::Warning::Level::Error:
+                log_error("%s", msg.c_str());
+                break;
+              case mysqlshdk::db::Warning::Level::Warn:
+                log_warning("%s", msg.c_str());
+                break;
+              case mysqlshdk::db::Warning::Level::Note:
+                log_info("%s", msg.c_str());
+                break;
+            }
+          }
+
+          if (remaining_warnings_count > 0) {
             mysqlsh::current_console()->print_info(
                 "Check mysqlsh.log for " +
                 std::to_string(remaining_warnings_count) + " more warning" +
@@ -951,6 +1053,11 @@ void Load_data_worker::execute(
 
       if (!m_range_queue && !fi.continuation) break;
     }
+  } catch (const std::exception &e) {
+    if (!m_thread_exception[m_thread_id]) {
+      mysqlsh::current_console()->print_error(worker_name + e.what());
+      m_thread_exception[m_thread_id] = std::current_exception();
+    }
   } catch (...) {
     m_thread_exception[m_thread_id] = std::current_exception();
   }
