@@ -1,18 +1,16 @@
 /*
- * Copyright (c) 2020, 2024, Oracle and/or its affiliates.
+ * Copyright (c) 2020, 2023, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms,
- * as designated in a particular file or component or in included license
+ * This program is also distributed with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
- *
+ * separately licensed software that they have included with MySQL.
  * This program is distributed in the hope that it will be useful,  but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
@@ -27,6 +25,7 @@
 #define MODULES_UTIL_LOAD_DUMP_LOADER_H_
 
 #include <atomic>
+#include <chrono>
 #include <list>
 #include <memory>
 #include <queue>
@@ -49,6 +48,7 @@
 
 #include "mysqlshdk/libs/db/mysql/session.h"
 #include "mysqlshdk/libs/storage/ifile.h"
+#include "mysqlshdk/libs/utils/priority_queue.h"
 #include "mysqlshdk/libs/utils/synchronized_queue.h"
 
 namespace mysqlsh {
@@ -68,8 +68,16 @@ class Dump_loader {
 
   void interrupt();
 
+  void hard_interrupt();
+
+  void abort();
+
   void run();
 
+  void show_metadata(bool force = false) const;
+
+  dump::Progress_thread *progress() { return &m_progress_thread; }
+
  private:
   class Worker {
    public:
@@ -86,6 +94,7 @@ class Dump_loader {
           Dump_loader *) = 0;
 
       size_t id() const { return m_id; }
+      const char *log_id() const { return m_log_id.c_str(); }
       const std::string &schema() const { return m_schema; }
       const std::string &table() const { return m_table; }
       const std::string &key() const { return m_key; }
@@ -99,6 +108,7 @@ class Dump_loader {
 
      protected:
       size_t m_id = std::numeric_limits<size_t>::max();
+      std::string m_log_id;
       std::string m_schema;
       std::string m_table;
       std::string m_key;
@@ -106,7 +116,7 @@ class Dump_loader {
      private:
       friend class Worker;
 
-      void set_id(size_t id) { m_id = id; }
+      void set_id(size_t id);
 
       uint64_t m_weight = 1;
     };
@@ -131,11 +141,12 @@ class Dump_loader {
      public:
       Table_ddl_task(const std::string &schema, const std::string &table,
                      std::string &&script, bool placeholder,
-                     Load_progress_log::Status status)
+                     Load_progress_log::Status status, bool exists)
           : Task(schema, table),
             m_script(std::move(script)),
             m_placeholder(placeholder),
-            m_status(status) {}
+            m_status(status),
+            m_exists(exists) {}
 
       bool execute(const std::shared_ptr<mysqlshdk::db::mysql::Session> &,
                    Worker *, Dump_loader *) override;
@@ -170,6 +181,8 @@ class Dump_loader {
       Load_progress_log::Status m_status;
 
       std::unique_ptr<compatibility::Deferred_statements> m_deferred_statements;
+
+      bool m_exists = false;
     };
 
     class Load_chunk_task : public Task {
@@ -291,23 +304,8 @@ class Dump_loader {
     }
   };
 
-  class Priority_queue
-      : public std::priority_queue<Task_ptr, std::vector<Task_ptr>,
-                                   Task_comparator> {
-   public:
-    using priority_queue::priority_queue;
-
-    Task_ptr pop_top() {
-      if (c.empty()) {
-        throw std::runtime_error("Trying to pop from an empty queue.");
-      }
-
-      std::pop_heap(c.begin(), c.end(), comp);
-      Task_ptr task = std::move(c.back());
-      c.pop_back();
-      return task;
-    }
-  };
+  using Queue =
+      shcore::Priority_queue<Task_ptr, std::vector<Task_ptr>, Task_comparator>;
 
   struct Worker_event {
     enum Event {
@@ -339,7 +337,9 @@ class Dump_loader {
   void execute_table_ddl_tasks();
   void execute_view_ddl_tasks();
 
-  bool wait_for_more_data();
+  void wait_for_metadata();
+  bool scan_for_more_data(bool wait = true);
+  void wait_for_dump(std::chrono::steady_clock::time_point start_time);
 
   std::shared_ptr<mysqlshdk::db::mysql::Session> create_session();
 
@@ -368,6 +368,7 @@ class Dump_loader {
 
   void check_existing_objects();
   bool report_duplicates(const std::string &what, const std::string &schema,
+                         std::list<Dump_reader::Object_info *> *objects,
                          mysqlshdk::db::IResult *result);
 
   void open_dump();
@@ -456,7 +457,7 @@ class Dump_loader {
   template <typename... Args>
   static inline void executef(
       const std::shared_ptr<mysqlshdk::db::ISession> &session, const char *sql,
-      Args &&...args) {
+      Args &&... args) {
     session->executef(sql, std::forward<Args>(args)...);
   }
 
@@ -470,7 +471,7 @@ class Dump_loader {
   }
 
   template <typename... Args>
-  inline void executef(const char *sql, Args &&...args) {
+  inline void executef(const char *sql, Args &&... args) {
     executef(m_session, sql, std::forward<Args>(args)...);
   }
 
@@ -557,7 +558,7 @@ class Dump_loader {
 
   std::vector<std::thread> m_worker_threads;
   std::list<Worker> m_workers;
-  Priority_queue m_pending_tasks;
+  Queue m_pending_tasks;
   uint64_t m_current_weight = 0;
 
   std::mutex m_tables_being_loaded_mutex;
@@ -588,6 +589,7 @@ class Dump_loader {
 
   size_t m_num_bytes_previously_loaded = 0;
   std::atomic<size_t> m_num_rows_loaded;
+  std::atomic<size_t> m_num_rows_deleted = 0;
   std::atomic<size_t> m_num_bytes_loaded;
   std::atomic<size_t> m_num_raw_bytes_loaded;
   std::atomic<size_t> m_num_chunks_loaded;
