@@ -1,18 +1,16 @@
 /*
- * Copyright (c) 2020, 2024, Oracle and/or its affiliates.
+ * Copyright (c) 2020, 2023, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms,
- * as designated in a particular file or component or in included license
+ * This program is also distributed with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
- *
+ * separately licensed software that they have included with MySQL.
  * This program is distributed in the hope that it will be useful,  but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
@@ -28,11 +26,8 @@
 #include <mysqld_error.h>
 
 #include <algorithm>
-#include <regex>
-#include <utility>
 
 #include "modules/mod_utils.h"
-#include "modules/util/common/dump/constants.h"
 #include "modules/util/common/dump/utils.h"
 #include "modules/util/dump/dump_manifest_config.h"
 #include "modules/util/load/load_errors.h"
@@ -53,34 +48,13 @@ namespace {
 
 using Version = mysqlshdk::utils::Version;
 
+const char *k_excluded_users[] = {"mysql.infoschema", "mysql.session",
+                                  "mysql.sys"};
+const char *k_oci_excluded_users[] = {"administrator", "ociadmin", "ocimonitor",
+                                      "ocirpl"};
+
 constexpr auto k_minimum_max_bytes_per_transaction = 4096;
 
-bool is_mds(const Version &version) {
-  return shcore::str_endswith(version.get_extra(), "cloud");
-}
-
-std::shared_ptr<mysqlshdk::oci::IPAR_config> par_config(
-    const std::string &url) {
-  mysqlshdk::oci::PAR_structure par;
-
-  switch (mysqlshdk::oci::parse_par(url, &par)) {
-    case mysqlshdk::oci::PAR_type::MANIFEST:
-      return std::make_shared<dump::Dump_manifest_read_config>(par);
-
-    case mysqlshdk::oci::PAR_type::PREFIX:
-      return std::make_shared<
-          mysqlshdk::storage::backend::oci::Oci_par_directory_config>(par);
-
-    case mysqlshdk::oci::PAR_type::GENERAL:
-      return std::make_shared<mysqlshdk::oci::General_par_config>(par);
-
-    case mysqlshdk::oci::PAR_type::NONE:
-      break;
-  }
-
-  return {};
-}
-
 }  // namespace
 
 Load_dump_options::Load_dump_options() : Load_dump_options("") {}
@@ -135,7 +109,7 @@ const shcore::Option_pack_def<Load_dump_options> &Load_dump_options::options() {
                     {{"append", Update_gtid_set::APPEND},
                      {"replace", Update_gtid_set::REPLACE},
                      {"off", Update_gtid_set::OFF}})
-          .optional("showMetadata", &Load_dump_options::m_show_metadata)
+          .optional("showMetadata", &Load_dump_options::set_show_metadata)
           .optional("createInvisiblePKs",
                     &Load_dump_options::m_create_invisible_pks)
           .optional("maxBytesPerTransaction",
@@ -181,7 +155,7 @@ void Load_dump_options::set_max_bytes_per_transaction(
 void Load_dump_options::set_progress_file(const std::string &value) {
   m_progress_file = value;
 
-  auto config = par_config(value);
+  auto config = dump::common::get_par_config(value);
 
   if (config && config->valid()) {
     if (mysqlshdk::oci::PAR_type::GENERAL != config->par().type()) {
@@ -195,10 +169,8 @@ void Load_dump_options::set_progress_file(const std::string &value) {
 }
 
 void Load_dump_options::set_session(
-    const std::shared_ptr<mysqlshdk::db::ISession> &session,
-    const std::string &current_schema) {
+    const std::shared_ptr<mysqlshdk::db::ISession> &session) {
   m_base_session = session;
-  m_current_schema = current_schema;
 
   m_target = get_classic_connection_options(m_base_session);
 
@@ -227,7 +199,7 @@ void Load_dump_options::set_session(
   m_target_server_version =
       Version(query("SELECT @@version")->fetch_one()->get_string(0));
 
-  m_is_mds = ::mysqlsh::is_mds(m_target_server_version);
+  m_is_mds = m_target_server_version.is_mds();
   DBUG_EXECUTE_IF("dump_loader_force_mds", { m_is_mds = true; });
 
   m_sql_generate_invisible_primary_key =
@@ -271,10 +243,10 @@ void Load_dump_options::set_session(
 
   if (m_load_users) {
     // some users are always excluded
-    filters().users().exclude(dump::common::k_excluded_users);
+    filters().users().exclude(k_excluded_users);
 
     if (is_mds()) {
-      filters().users().exclude(dump::common::k_mhs_excluded_users);
+      filters().users().exclude(k_oci_excluded_users);
     }
 
     shcore::Account account;
@@ -284,10 +256,6 @@ void Load_dump_options::set_session(
     filters().users().exclude(std::move(account));
   }
 
-  if (is_mds()) {
-    filters().schemas().exclude(dump::common::k_mhs_excluded_schemas);
-  }
-
   if (m_target_server_version >= Version(8, 0, 27)) {
     // adding indexes in parallel was added in 8.0.27,
     // innodb_parallel_read_threads threads are used during the first stage,
@@ -310,16 +278,11 @@ void Load_dump_options::set_session(
 
 void Load_dump_options::validate() {
   if (!m_storage_config || !m_storage_config->valid()) {
-    auto config = par_config(m_url);
+    auto config = dump::common::get_par_config(m_url);
 
     if (config && config->valid()) {
       if (mysqlshdk::oci::PAR_type::MANIFEST == config->par().type() ||
           mysqlshdk::oci::PAR_type::PREFIX == config->par().type()) {
-        if (mysqlshdk::oci::PAR_type::MANIFEST == config->par().type()) {
-          current_console()->print_warning(
-              "Support for PAR Manifest is deprecated and will be removed in a "
-              "future release. Please use a prefix PAR instead.");
-        }
         if (!m_progress_file.has_value()) {
           throw shcore::Exception::argument_error(
               "When using a PAR to load a dump, the progressFile option must "
@@ -331,7 +294,7 @@ void Load_dump_options::validate() {
             "file.");
       }
 
-      m_storage_config = std::move(config);
+      set_storage_config(std::move(config));
     }
   }
 
@@ -353,7 +316,8 @@ void Load_dump_options::validate() {
   }
 }
 
-std::string Load_dump_options::target_import_info() const {
+std::string Load_dump_options::target_import_info(
+    const char *operation, const std::string &extra) const {
   std::string action;
 
   std::vector<std::string> what_to_load;
@@ -376,14 +340,14 @@ std::string Load_dump_options::target_import_info() const {
 
   if (what_to_load.size() == 3) {
     action = shcore::str_format(
-        "Loading %s, %s and %s from %s", what_to_load[0].c_str(),
+        "%s %s, %s and %s from %s", operation, what_to_load[0].c_str(),
         what_to_load[1].c_str(), what_to_load[2].c_str(), where.c_str());
   } else if (what_to_load.size() == 2) {
-    action =
-        shcore::str_format("Loading %s and %s from %s", what_to_load[0].c_str(),
-                           what_to_load[1].c_str(), where.c_str());
+    action = shcore::str_format("%s %s and %s from %s", operation,
+                                what_to_load[0].c_str(),
+                                what_to_load[1].c_str(), where.c_str());
   } else if (!what_to_load.empty()) {
-    action = shcore::str_format("Loading %s only from %s",
+    action = shcore::str_format("%s %s only from %s", operation,
                                 what_to_load[0].c_str(), where.c_str());
   } else {
     if (m_analyze_tables == Analyze_table_mode::HISTOGRAM)
@@ -398,15 +362,17 @@ std::string Load_dump_options::target_import_info() const {
     }
   }
 
-  std::string detail;
+  std::string detail = extra;
 
-  if (threads_count() == 1)
-    detail = " using 1 thread.";
-  else
-    detail = shcore::str_format(" using %s threads.",
-                                std::to_string(threads_count()).c_str());
+  if (detail.empty()) {
+    detail = " using " + std::to_string(threads_count()) + " thread";
 
-  return action + detail;
+    if (threads_count() != 1) {
+      detail += 's';
+    }
+  }
+
+  return action + detail + '.';
 }
 
 void Load_dump_options::on_unpacked_options() {
@@ -415,11 +381,11 @@ void Load_dump_options::on_unpacked_options() {
   m_blob_storage_options.throw_on_conflict(m_oci_bucket_options);
 
   if (m_oci_bucket_options) {
-    m_storage_config = m_oci_bucket_options.config();
+    set_storage_config(m_oci_bucket_options.config());
   }
 
   if (m_s3_bucket_options) {
-    m_storage_config = m_s3_bucket_options.config();
+    set_storage_config(m_s3_bucket_options.config());
   }
 
   if (m_blob_storage_options) {
@@ -474,13 +440,10 @@ void Load_dump_options::on_unpacked_options() {
 
 void Load_dump_options::on_log_options(const char *msg) const {
   std::string s = msg;
+  const auto pos = s.find("\"progressFile\":\"https://objectstorage.");
 
-  static const std::regex k_par_progress_file(
-      R"("progressFile":"https://(?:[^\.]+\.)?objectstorage\.)");
-  std::smatch match;
-
-  if (std::regex_search(s, match, k_par_progress_file)) {
-    s = mysqlshdk::oci::hide_par_secret(s, match.position());
+  if (std::string::npos != pos) {
+    s = mysqlshdk::oci::hide_par_secret(s, pos);
   }
 
   log_info("Load options: %s", s.c_str());
@@ -535,4 +498,9 @@ void Load_dump_options::set_handle_grant_errors(const std::string &action) {
   }
 }
 
+void Load_dump_options::set_storage_config(
+    std::shared_ptr<mysqlshdk::storage::Config> storage_config) {
+  m_storage_config = std::move(storage_config);
+}
+
 }  // namespace mysqlsh
