@@ -1,18 +1,16 @@
 /*
- * Copyright (c) 2014, 2025, Oracle and/or its affiliates.
+ * Copyright (c) 2014, 2023, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms,
- * as designated in a particular file or component or in included license
+ * This program is also distributed with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
- *
+ * separately licensed software that they have included with MySQL.
  * This program is distributed in the hope that it will be useful,  but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
@@ -30,7 +28,6 @@
 
 #include <algorithm>
 #include <cassert>
-#include <cstdint>
 #include <iterator>
 #include <list>
 #include <map>
@@ -440,7 +437,7 @@ typedef Value::Array_type_ref Array_t;
 inline Dictionary_t make_dict() { return std::make_shared<Value::Map_type>(); }
 
 template <typename K, typename V, typename... Arg>
-inline Dictionary_t make_dict(K &&key, V &&value, Arg &&...args) {
+inline Dictionary_t make_dict(K &&key, V &&value, Arg &&... args) {
   Dictionary_t dict = make_dict(std::forward<Arg>(args)...);
   dict->emplace(std::forward<K>(key), std::forward<V>(value));
   return dict;
@@ -449,7 +446,7 @@ inline Dictionary_t make_dict(K &&key, V &&value, Arg &&...args) {
 inline Array_t make_array() { return std::make_shared<Value::Array_type>(); }
 
 template <typename... Arg>
-inline Array_t make_array(Arg &&...args) {
+inline Array_t make_array(Arg &&... args) {
   auto array = make_array();
   (void)std::initializer_list<int>{
       (array->emplace_back(std::forward<Arg>(args)), 0)...};
@@ -465,15 +462,6 @@ inline Array_t make_array(const C<T> &container) {
   return array;
 }
 
-template <template <typename...> class C, typename T>
-inline Array_t make_array(C<T> &&container) {
-  auto array = make_array();
-  for (auto &item : container) {
-    array->emplace_back(std::move(item));
-  }
-  return array;
-}
-
 class SHCORE_PUBLIC Exception : public shcore::Error {
   std::shared_ptr<Value::Map_type> _error;
 
@@ -891,12 +879,21 @@ class Option_unpacker {
 
   void set_options(const shcore::Dictionary_t &options);
 
+  void set_ignored(std::unordered_set<std::string_view> ignored) {
+    m_ignored = std::move(ignored);
+  }
+
+  bool is_ignored(std::string_view option) const {
+    return m_ignored.count(option) > 0;
+  }
+
   const shcore::Dictionary_t &options() const { return m_options; }
 
  protected:
   Dictionary_t m_options;
   std::set<std::string> m_unknown;
   std::set<std::string> m_missing;
+  std::unordered_set<std::string_view> m_ignored;
 
   template <typename T, typename S>
   void extract_value(const char *name, S *out_value, const Value &value) {
@@ -983,9 +980,6 @@ class SHCORE_PUBLIC Object_bridge {
   //! Sets the value of a member
   virtual void set_member(size_t index, Value value) = 0;
 
-  //! Returns the number of indexable members
-  virtual size_t length() const = 0;
-
   //! Returns true if a method with the given name exists.
   virtual bool has_method(const std::string &name) const = 0;
 
