@@ -1,18 +1,16 @@
 /*
- * Copyright (c) 2014, 2024, Oracle and/or its affiliates.
+ * Copyright (c) 2014, 2023, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms,
- * as designated in a particular file or component or in included license
+ * This program is also distributed with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
- *
+ * separately licensed software that they have included with MySQL.
  * This program is distributed in the hope that it will be useful,  but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
@@ -34,7 +32,10 @@
 #include <memory>
 #include <mutex>
 #include <string>
+#include <string_view>
 #include <tuple>
+#include <unordered_map>
+#include <unordered_set>
 #include <utility>
 #include <vector>
 
@@ -187,6 +188,15 @@ struct Parameter final {
 
   bool is_deprecated;
 
+  bool operator==(const Parameter &r) const {
+    return name == r.name && m_type == r.m_type && flag == r.flag &&
+           short_name == r.short_name &&
+           cmd_line_enabled == r.cmd_line_enabled &&
+           is_deprecated == r.is_deprecated;
+  }
+
+  bool operator!=(const Parameter &r) const { return !operator==(r); }
+
   void validate(const Value &data, Parameter_context *context) const;
 
   bool valid_type(Value_type type) const;
@@ -614,6 +624,8 @@ class Option_pack_def : public IOption_pack_def {
       T::options().unpack(unpacker, &((*instance).*var));
     });
 
+    ignore(T::options().ignored());
+
     return *this;
   }
 
@@ -648,9 +660,52 @@ class Option_pack_def : public IOption_pack_def {
         [](const Option_pack_def<C> *, shcore::Option_unpacker *unpacker,
            C *instance) { T::options().unpack(unpacker, instance); });
 
+    ignore(T::options().ignored());
+
     return *this;
   }
 
+  /**
+   * Ignores given options. Options in this list are treated as unknown.
+   *
+   * @params options Options to be ignored
+   */
+  Option_pack_def<C> &ignore(
+      const std::unordered_set<std::string_view> &ignored_options) {
+    if (!ignored_options.empty()) {
+      m_ignored.insert(ignored_options.begin(), ignored_options.end());
+
+      m_option_md.erase(std::remove_if(m_option_md.begin(), m_option_md.end(),
+                                       [this](const auto &param) {
+                                         return is_ignored(param->name);
+                                       }),
+                        m_option_md.end());
+    }
+
+    return *this;
+  }
+
+  /**
+   * Ignores all options from the given option pack type.
+   */
+  template <typename T>
+  Option_pack_def<C> &ignore() {
+    const auto &options = T::options().options();
+    std::unordered_set<std::string_view> ignored;
+
+    for (const auto &o : options) {
+      ignored.emplace(o->name);
+    }
+
+    ignore(ignored);
+
+    return *this;
+  }
+
+  const std::unordered_set<std::string_view> &ignored() const {
+    return m_ignored;
+  }
+
   /**
    * Allows defining a callback for when the unpacking starts.
    *
@@ -696,6 +751,7 @@ class Option_pack_def : public IOption_pack_def {
 
     Option_unpacker unpacker;
     unpacker.set_options(options);
+    unpacker.set_ignored(m_ignored);
 
     unpack(&unpacker, instance);
 
@@ -720,8 +776,9 @@ class Option_pack_def : public IOption_pack_def {
  private:
   template <typename T>
   void add_options() {
-    auto options = T::options().options();
-    m_option_md.insert(m_option_md.end(), options.begin(), options.end());
+    for (const auto &o : T::options().options()) {
+      add_option(o);
+    }
   }
 
   template <typename T>
@@ -743,15 +800,36 @@ class Option_pack_def : public IOption_pack_def {
   template <typename T>
   void add_option(const std::string &name, const std::string &sname,
                   Option_scope scope, shcore::Param_flag flag) {
-    m_option_md.push_back(std::make_shared<shcore::Parameter>(
-        name, shcore::Type_info<T>::vtype(), flag, sname,
-        scope != Option_scope::CLI_DISABLED,
-        scope == Option_scope::DEPRECATED));
+    if (is_ignored(name)) {
+      return;
+    }
 
-    auto validator = Type_info<T>::validator();
-    if (validator) {
+    const auto option = std::make_shared<shcore::Parameter>(
+        name, shcore::Type_info<T>::vtype(), flag, sname,
+        scope != Option_scope::CLI_DISABLED, scope == Option_scope::DEPRECATED);
+    if (auto validator = Type_info<T>::validator()) {
       validator->set_enabled(false);
-      m_option_md.back()->set_validator(std::move(validator));
+      option->set_validator(std::move(validator));
+    }
+
+    add_option(option);
+  }
+
+  void add_option(const std::shared_ptr<shcore::Parameter> &option) {
+    if (is_ignored(option->name)) {
+      return;
+    }
+
+    if (const auto o = m_all_options.find(option->name);
+        o != m_all_options.end()) {
+      if (*option != *m_option_md[o->second]) {
+        throw std::logic_error(
+            "Trying to overwrite an option which already exists: " +
+            option->name);
+      }
+    } else {
+      m_all_options.emplace(option->name, m_option_md.size());
+      m_option_md.emplace_back(option);
     }
   }
 
@@ -799,6 +877,10 @@ class Option_pack_def : public IOption_pack_def {
     }
   }
 
+  bool is_ignored(std::string_view option) const {
+    return m_ignored.count(option) > 0;
+  }
+
   std::vector<std::function<void(const Option_pack_def<C> *,
                                  shcore::Option_unpacker *, C *)>>
       m_unpack_callbacks;
@@ -806,6 +888,8 @@ class Option_pack_def : public IOption_pack_def {
   std::function<void(C *, const shcore::Dictionary_t &)> m_start_callback;
   std::function<void(C *)> m_done_callback;
   std::function<void(C *, const char *)> m_log_callback;
+  std::unordered_set<std::string_view> m_ignored;
+  std::unordered_map<std::string_view, std::size_t> m_all_options;
 };
 
 /**
@@ -930,7 +1014,6 @@ class SHCORE_PUBLIC Cpp_object_bridge : public Object_bridge {
   bool is_indexed() const override;
   Value get_member(size_t index) const override;
   void set_member(size_t index, Value value) override;
-  size_t length() const override;
 
   bool has_method(const std::string &name) const override;
 
@@ -1543,7 +1626,7 @@ class SHCORE_PUBLIC Cpp_object_bridge : public Object_bridge {
   template <typename R, typename C, typename... A>
   Cpp_function::Metadata *expose_(const std::string &name, R (C::*func)(A...),
                                   const std::vector<std::string> &docs,
-                                  Type_info_t<A> &&...defs) {
+                                  Type_info_t<A> &&... defs) {
     return expose__<R, std::function<R(A...)>, A...>(
         name,
         [this, func](A... args) -> R {
@@ -1556,7 +1639,7 @@ class SHCORE_PUBLIC Cpp_object_bridge : public Object_bridge {
   Cpp_function::Metadata *expose_(const std::string &name,
                                   R (C::*func)(A...) const,
                                   const std::vector<std::string> &docs,
-                                  Type_info_t<A> &&...defs) {
+                                  Type_info_t<A> &&... defs) {
     return expose__<R, std::function<R(A...)>, A...>(
         name,
         [this, func](A... args) -> R {
@@ -1568,7 +1651,7 @@ class SHCORE_PUBLIC Cpp_object_bridge : public Object_bridge {
   template <typename R, typename... A>
   Cpp_function::Metadata *expose_(const std::string &name, R (*func)(A...),
                                   const std::vector<std::string> &docs,
-                                  Type_info_t<A> &&...defs) {
+                                  Type_info_t<A> &&... defs) {
     return expose__<R, std::function<R(A...)>, A...>(name, func, docs,
                                                      std::move(defs)...);
   }
@@ -1577,7 +1660,7 @@ class SHCORE_PUBLIC Cpp_object_bridge : public Object_bridge {
   Cpp_function::Metadata *expose_(const std::string &name,
                                   std::function<R(A...)> &&func,
                                   const std::vector<std::string> &docs,
-                                  Type_info_t<A> &&...defs) {
+                                  Type_info_t<A> &&... defs) {
     return expose__<R, std::function<R(A...)>, A...>(name, std::move(func),
                                                      docs, std::move(defs)...);
   }
@@ -1606,7 +1689,7 @@ class SHCORE_PUBLIC Cpp_object_bridge : public Object_bridge {
   template <typename R, typename F, typename... A>
   Cpp_function::Metadata *expose__(const std::string &name, F &&func,
                                    const std::vector<std::string> &docs,
-                                   Type_info_t<A> &&...defs) {
+                                   Type_info_t<A> &&... defs) {
     assert(func != nullptr);
     assert(!name.empty());
     assert(docs.size() == sizeof...(A));
