@@ -1,18 +1,16 @@
 /*
- * Copyright (c) 2017, 2024, Oracle and/or its affiliates.
+ * Copyright (c) 2017, 2023, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms,
- * as designated in a particular file or component or in included license
+ * This program is also distributed with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
- *
+ * separately licensed software that they have included with MySQL.
  * This program is distributed in the hope that it will be useful,  but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
@@ -23,8 +21,11 @@
  * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 #include "mysqlshdk/libs/db/connection_options.h"
+
 #include <algorithm>
 #include <cassert>
+#include <map>
+
 #include "mysqlshdk/include/shellcore/shell_options.h"
 #include "mysqlshdk/libs/db/uri_encoder.h"
 #include "mysqlshdk/libs/db/uri_parser.h"
@@ -40,6 +41,32 @@ using mysqlshdk::utils::nullable_options::Set_mode;
 namespace mysqlshdk {
 namespace db {
 
+namespace {
+const std::set<std::string> uri_extra_options = {
+    kAuthMethod,
+    kGetServerPublicKey,
+    kServerPublicKeyPath,
+    kConnectTimeout,
+    kNetReadTimeout,
+    kNetWriteTimeout,
+    kCompression,
+    kCompressionAlgorithms,
+    kLocalInfile,
+    kNetBufferLength,
+    kMaxAllowedPacket,
+    kMysqlPluginDir,
+#ifdef _WIN32
+    kKerberosClientAuthMode,
+#endif
+    kOciConfigFile,
+    kOciAuthenticationClientConfigProfile,
+};
+
+const std::map<std::string, std::string, shcore::Case_insensitive_comparator>
+    deprecated_connection_attributes = {{kDbUser, kUser},
+                                        {kDbPassword, kPassword}};
+}  // namespace
+
 std::string to_string(Transport_type type) {
   switch (type) {
     case Transport_type::Tcp:
@@ -346,7 +373,13 @@ void Connection_options::set(const std::string &name,
     set_compression(value);
   } else if (iname == kCompressionAlgorithms) {
     set_compression_algorithms(value);
-  } else if (is_extra_option(iname)) {
+  }
+#ifdef _WIN32
+  else if (iname == kKerberosClientAuthMode) {
+    set_kerberos_auth_mode(value);
+  }
+#endif
+  else if (is_extra_option(iname)) {
     if (iname == kGetServerPublicKey) {
       if (!is_bool_value(value)) {
         throw std::invalid_argument(
@@ -715,6 +748,61 @@ bool Connection_options::is_auth_method(const std::string &method_id) const {
   return ret_val;
 }
 
+bool Connection_options::has_oci_config_file() const {
+  return has_value(mysqlshdk::db::kOciConfigFile);
+}
+
+void Connection_options::set_oci_config_file(const std::string &path) {
+  set(mysqlshdk::db::kOciConfigFile, path);
+}
+
+std::string Connection_options::get_oci_config_file() const {
+  return m_extra_options.get_value(mysqlshdk::db::kOciConfigFile);
+}
+
+bool Connection_options::has_oci_client_config_profile() const {
+  return has_value(mysqlshdk::db::kOciAuthenticationClientConfigProfile);
+}
+
+void Connection_options::set_oci_client_config_profile(
+    const std::string &path) {
+  set(mysqlshdk::db::kOciAuthenticationClientConfigProfile, path);
+}
+
+std::string Connection_options::get_oci_client_config_profile() const {
+  return m_extra_options.get_value(
+      mysqlshdk::db::kOciAuthenticationClientConfigProfile);
+}
+
+#ifdef _WIN32
+bool Connection_options::has_kerberos_auth_mode() const {
+  return has_value(mysqlshdk::db::kKerberosClientAuthMode);
+}
+
+void Connection_options::set_kerberos_auth_mode(const std::string &mode) {
+  std::string uppermode = shcore::str_upper(mode);
+  if (uppermode != mysqlshdk::db::kKerberosAuthModeSSPI &&
+      uppermode != mysqlshdk::db::kKerberosAuthModeGSSAPI) {
+    throw std::invalid_argument(shcore::str_format(
+        "Invalid value: %s. Allowed values: SSPI, GSSAPI.", mode.c_str()));
+  }
+
+  m_extra_options.set(mysqlshdk::db::kKerberosClientAuthMode, uppermode,
+                      Set_mode::CREATE);
+}
+
+std::string Connection_options::get_kerberos_auth_mode() const {
+  if (has_kerberos_auth_mode()) {
+    auto result =
+        m_extra_options.get_value(mysqlshdk::db::kKerberosClientAuthMode);
+    if (result == mysqlshdk::db::kKerberosAuthModeSSPI ||
+        result == mysqlshdk::db::kKerberosAuthModeGSSAPI)
+      return result;
+  }
+  return mysqlshdk::db::kKerberosAuthModeSSPI;
+}
+#endif
+
 void Connection_options::set_connection_attribute(const std::string &attribute,
                                                   const std::string &value) {
   std::string error;
