@@ -1,18 +1,16 @@
 /*
- * Copyright (c) 2017, 2024, Oracle and/or its affiliates.
+ * Copyright (c) 2017, 2023, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms,
- * as designated in a particular file or component or in included license
+ * This program is also distributed with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
- *
+ * separately licensed software that they have included with MySQL.
  * This program is distributed in the hope that it will be useful,  but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
@@ -229,6 +227,8 @@ class IInstance {
 
   virtual std::string get_plugin_library_extension() const = 0;
 
+  virtual std::string generate_uuid() const = 0;
+
  public:
   // Convenience interface for session
   virtual std::shared_ptr<mysqlshdk::db::IResult> query(
@@ -241,12 +241,12 @@ class IInstance {
 
   template <typename... Args>
   inline std::shared_ptr<mysqlshdk::db::IResult> queryf(
-      const std::string &sql, const Args &...args) const {
+      const std::string &sql, const Args &... args) const {
     return query(shcore::sqlformat(sql, args...));
   }
 
   template <typename... Args>
-  inline void executef(const std::string &sql, const Args &...args) const {
+  inline void executef(const std::string &sql, const Args &... args) const {
     execute(shcore::sqlformat(sql, args...));
   }
 
@@ -254,7 +254,7 @@ class IInstance {
   inline std::string queryf_one_string(int32_t column_index,
                                        const std::string &default_if_null,
                                        const std::string &sql,
-                                       const Args &...args) const {
+                                       const Args &... args) const {
     auto result = query(shcore::sqlformat(sql, args...));
     if (auto row = result->fetch_one())
       return row->get_as_string(column_index, default_if_null);
@@ -264,7 +264,7 @@ class IInstance {
   template <typename... Args>
   inline int64_t queryf_one_int(int32_t column_index, int64_t default_if_null,
                                 const std::string &sql,
-                                const Args &...args) const {
+                                const Args &... args) const {
     auto result = query(shcore::sqlformat(sql, args...));
     if (auto row = result->fetch_one())
       return row->get_int(column_index, default_if_null);
@@ -497,6 +497,16 @@ class Instance : public IInstance {
 
   std::string get_plugin_library_extension() const override;
 
+  /**
+   * Generate a UUID to use for the group name / view change uuid
+   *
+   * The UUID is generated from the target instance (MySQL server) using the
+   * UUID() SQL function.
+   *
+   * @return A string with a new UUID to be used,
+   */
+  std::string generate_uuid() const override;
+
  public:
   std::shared_ptr<mysqlshdk::db::IResult> query(
       const std::string &sql, bool buffered = false) const override;
