@@ -1,18 +1,16 @@
 /*
- * Copyright (c) 2017, 2024, Oracle and/or its affiliates.
+ * Copyright (c) 2017, 2023, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms,
- * as designated in a particular file or component or in included license
+ * This program is also distributed with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
- *
+ * separately licensed software that they have included with MySQL.
  * This program is distributed in the hope that it will be useful,  but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
@@ -40,6 +38,7 @@
 #include <ifaddrs.h>
 #include <net/if.h>
 #include <netdb.h>
+#include <poll.h>
 #include <sys/socket.h>
 #include <sys/types.h>
 #include <unistd.h>
@@ -604,32 +603,30 @@ bool Net::is_port_listening_impl(const std::string &address, int port) const {
   }
 
 #ifdef _WIN32
+  const auto poll = WSAPoll;
   auto last_error = []() { return WSAGetLastError(); };
   const int connection_refused = WSAECONNREFUSED;
-  auto would_block = [](int err) {
+  const auto would_block = [](int err) {
     return err == WSAEWOULDBLOCK || err == WSAEINPROGRESS;
   };
 #else
   auto last_error = []() { return errno; };
   const int connection_refused = ECONNREFUSED;
-  auto would_block = [](int err) {
+  const auto would_block = [](int err) {
     return err == EWOULDBLOCK || err == EINPROGRESS;
   };
 #endif
   int err;
 
-  struct timeval timeout;
-  timeout.tv_sec = k_port_check_timeout;
-  timeout.tv_usec = 0;
+  pollfd fds;
+  fds.fd = sock;
+  fds.events = POLLIN | POLLOUT;
 
   while (would_block((err = last_error()))) {
-    fd_set wfds, efds;
-    FD_ZERO(&wfds);
-    FD_SET(sock, &wfds);
-    FD_ZERO(&efds);
-    FD_SET(sock, &efds);
+    fds.revents = 0;
+
+    err = poll(&fds, 1, k_port_check_timeout * 1000);  // milliseconds
 
-    err = select(sock + 1, nullptr, &wfds, &efds, &timeout);
     if (err == 0) {
       // timeout
       closesocket(sock);
@@ -638,6 +635,7 @@ bool Net::is_port_listening_impl(const std::string &address, int port) const {
 
     if (err > 0) {
       socklen_t len = sizeof(err);
+
       if (getsockopt(sock, SOL_SOCKET, SO_ERROR, (char *)&err, &len) < 0) {
         err = last_error();
       } else {
@@ -646,6 +644,7 @@ bool Net::is_port_listening_impl(const std::string &address, int port) const {
           return true;
         }
       }
+
       break;
     }
   }
