@@ -1,18 +1,16 @@
 /*
- * Copyright (c) 2017, 2024, Oracle and/or its affiliates.
+ * Copyright (c) 2017, 2023, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms,
- * as designated in a particular file or component or in included license
+ * This program is also distributed with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
- *
+ * separately licensed software that they have included with MySQL.
  * This program is distributed in the hope that it will be useful,  but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
@@ -42,7 +40,8 @@
 namespace shcore {
 
 void clear_buffer(char *buffer, size_t size);
-void clear_buffer(std::string &buffer);
+
+void clear_buffer(std::string *buffer);
 
 namespace internal {
 
@@ -162,7 +161,7 @@ inline bool str_caseeq_pair(std::basic_string_view<Char> a,
 }
 
 template <typename Char, typename... T>
-inline bool str_caseeq(std::basic_string_view<Char> a, T &&...tokens) {
+inline bool str_caseeq(std::basic_string_view<Char> a, T &&... tokens) {
   return (str_caseeq_pair<Char>(a, std::forward<T>(tokens)) || ...);
 }
 
@@ -170,13 +169,13 @@ inline bool str_caseeq(std::basic_string_view<Char> a, T &&...tokens) {
 
 template <typename... T>
 inline bool str_caseeq(std::string_view a, std::string_view token,
-                       T &&...tokens) {
+                       T &&... tokens) {
   return internal::str_caseeq(a, token, std::forward<T>(tokens)...);
 }
 
 template <typename... T>
 inline bool str_caseeq(std::wstring_view a, std::wstring_view token,
-                       T &&...tokens) {
+                       T &&... tokens) {
   return internal::str_caseeq(a, token, std::forward<T>(tokens)...);
 }
 
@@ -190,7 +189,7 @@ inline bool str_beginswith_pair(std::basic_string_view<Char> s,
 }
 
 template <typename Char, typename... T>
-inline bool str_beginswith(std::basic_string_view<Char> s, T &&...prefixes) {
+inline bool str_beginswith(std::basic_string_view<Char> s, T &&... prefixes) {
   return (str_beginswith_pair<Char>(s, std::forward<T>(prefixes)) || ...);
 }
 
@@ -198,13 +197,13 @@ inline bool str_beginswith(std::basic_string_view<Char> s, T &&...prefixes) {
 
 template <typename... T>
 inline bool str_beginswith(std::string_view s, std::string_view prefix,
-                           T &&...prefixes) {
+                           T &&... prefixes) {
   return internal::str_beginswith(s, prefix, std::forward<T>(prefixes)...);
 }
 
 template <typename... T>
 inline bool str_beginswith(std::wstring_view s, std::wstring_view prefix,
-                           T &&...prefixes) {
+                           T &&... prefixes) {
   return internal::str_beginswith(s, prefix, std::forward<T>(prefixes)...);
 }
 
@@ -218,7 +217,7 @@ inline bool str_ibeginswith_pair(std::basic_string_view<Char> s,
 }
 
 template <typename Char, typename... T>
-inline bool str_ibeginswith(std::basic_string_view<Char> s, T &&...prefixes) {
+inline bool str_ibeginswith(std::basic_string_view<Char> s, T &&... prefixes) {
   return (str_ibeginswith_pair<Char>(s, std::forward<T>(prefixes)) || ...);
 }
 
@@ -226,13 +225,13 @@ inline bool str_ibeginswith(std::basic_string_view<Char> s, T &&...prefixes) {
 
 template <typename... T>
 inline bool str_ibeginswith(std::string_view s, std::string_view prefix,
-                            T &&...prefixes) {
+                            T &&... prefixes) {
   return internal::str_ibeginswith(s, prefix, std::forward<T>(prefixes)...);
 }
 
 template <typename... T>
 inline bool str_ibeginswith(std::wstring_view s, std::wstring_view prefix,
-                            T &&...prefixes) {
+                            T &&... prefixes) {
   return internal::str_ibeginswith(s, prefix, std::forward<T>(prefixes)...);
 }
 
@@ -247,7 +246,7 @@ inline bool str_endswith_pair(std::basic_string_view<Char> s,
 }
 
 template <typename Char, typename... T>
-inline bool str_endswith(std::basic_string_view<Char> s, T &&...suffixes) {
+inline bool str_endswith(std::basic_string_view<Char> s, T &&... suffixes) {
   return (str_endswith_pair<Char>(s, std::forward<T>(suffixes)) || ...);
 }
 
@@ -255,13 +254,13 @@ inline bool str_endswith(std::basic_string_view<Char> s, T &&...suffixes) {
 
 template <typename... T>
 inline bool str_endswith(std::string_view s, std::string_view suffix,
-                         T &&...suffixes) {
+                         T &&... suffixes) {
   return internal::str_endswith(s, suffix, std::forward<T>(suffixes)...);
 }
 
 template <typename... T>
 inline bool str_endswith(std::wstring_view s, std::wstring_view suffix,
-                         T &&...suffixes) {
+                         T &&... suffixes) {
   return internal::str_endswith(s, suffix, std::forward<T>(suffixes)...);
 }
 
@@ -276,7 +275,7 @@ inline bool str_iendswith_pair(std::basic_string_view<Char> s,
 }
 
 template <typename Char, typename... T>
-inline bool str_iendswith(std::basic_string_view<Char> s, T &&...suffixes) {
+inline bool str_iendswith(std::basic_string_view<Char> s, T &&... suffixes) {
   return (str_iendswith_pair<Char>(s, std::forward<T>(suffixes)) || ...);
 }
 
@@ -284,13 +283,13 @@ inline bool str_iendswith(std::basic_string_view<Char> s, T &&...suffixes) {
 
 template <typename... T>
 inline bool str_iendswith(std::string_view s, std::string_view suffix,
-                          T &&...suffixes) {
+                          T &&... suffixes) {
   return internal::str_iendswith(s, suffix, std::forward<T>(suffixes)...);
 }
 
 template <typename... T>
 inline bool str_iendswith(std::wstring_view s, std::wstring_view suffix,
-                          T &&...suffixes) {
+                          T &&... suffixes) {
   return internal::str_iendswith(s, suffix, std::forward<T>(suffixes)...);
 }
 
@@ -434,13 +433,6 @@ inline bool str_itersplit(const std::string &input,
 }
 
 /** Strip a string out of blank chars */
-std::string_view SHCORE_PUBLIC
-str_strip_view(std::string_view s, std::string_view chars = " \r\n\t");
-std::string_view SHCORE_PUBLIC
-str_lstrip_view(std::string_view s, std::string_view chars = " \r\n\t");
-std::string_view SHCORE_PUBLIC
-str_rstrip_view(std::string_view s, std::string_view chars = " \r\n\t");
-
 std::string SHCORE_PUBLIC str_strip(std::string_view s,
                                     std::string_view chars = " \r\n\t");
 std::string SHCORE_PUBLIC str_lstrip(std::string_view s,
@@ -520,6 +512,12 @@ inline std::string str_join(const C &container, std::string_view sep,
 std::string SHCORE_PUBLIC str_replace(std::string_view s, std::string_view from,
                                       std::string_view to);
 
+template <typename... Args>
+inline std::string str_replace(std::string_view s, std::string_view from,
+                               std::string_view to, const Args &... rest) {
+  return str_replace(str_replace(s, from, to), rest...);
+}
+
 std::string SHCORE_PUBLIC bits_to_string(uint64_t bits, int nbits);
 std::pair<uint64_t, int> SHCORE_PUBLIC string_to_bits(std::string_view s);
 
