@@ -1,18 +1,16 @@
 /*
- * Copyright (c) 2014, 2024, Oracle and/or its affiliates.
+ * Copyright (c) 2014, 2023, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms,
- * as designated in a particular file or component or in included license
+ * This program is also distributed with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
- *
+ * separately licensed software that they have included with MySQL.
  * This program is distributed in the hope that it will be useful,  but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
@@ -306,7 +304,7 @@ Shell_options::Shell_options(
   add_startup_options(true)
     (cmdline("--ssh=<value>"), "Make SSH tunnel connection using"
         "Uniform Resource Identifier. "
-        "Format: [user@]host[:port]")
+        "Format: [user[:pass]@]host[:port]")
     (&storage.ssh.identity_file, "", cmdline("--ssh-identity-file=<file>"),
         "File from which the private key for public key authentication is "
         "read.",
@@ -339,7 +337,32 @@ Shell_options::Shell_options(
     (&storage.ssh.buffer_size, 10240, "ssh.bufferSize",
     "Set buffer size in bytes for data transfer, default is 10240 (10Kb)",
       shcore::opts::Range<int>(0, std::numeric_limits<int>::max()));
+
+#ifdef _WIN32
+  add_startup_options()
+    (cmdline("--plugin-authentication-kerberos-client-mode=<mode>"), 
+      "Allows defining the kerberos client mode (SSPI, GSSAPI) when using "
+      "kerberos authentication.",
+      [this](const std::string&, const char* value){
+        storage.connection_data.set_kerberos_auth_mode(value);
+      });
+#endif
   
+  add_startup_options()
+    (cmdline("--oci-config-file=<file>"),
+      "Allows defining the OCI configuration file for OCI authentication.",
+      [this](const std::string&, const char* value) {
+        auto path = shcore::path::expand_user(value);
+        if (!shcore::is_file(path))
+          throw shcore::Exception::argument_error("Invalid path or file for OCI authentication configuration file.");
+        storage.connection_data.set_oci_config_file(value);
+      })
+    (cmdline("--authentication-oci-client-config-profile=<name>"),
+      "Allows defining the OCI profile used from the configuration for client side OCI authentication.",
+      [this](const std::string&, const char* value) {
+        storage.connection_data.set_oci_client_config_profile(value);
+      });
+
   add_startup_options(true)
     (cmdline("--uri=<value>"), "Connect to Uniform Resource Identifier. "
         "Format: [user[:pass]@]host[:port][/db]")
@@ -458,10 +481,9 @@ Shell_options::Shell_options(
         cmdline("--fido-register-factor=<name>"),
         "Specifies authentication factor, for which registration needs to be "
         "done.")
-    (cmdline("--recreate-schema"), "Drop and recreate the specified schema. "
-        "Schema will be deleted if it exists!", deprecated(m_on_warning, nullptr, [this](const std::string&, const char* ) {
-          storage.recreate_database = true;
-        }));
+    (&storage.recreate_database, false, "recreateDatabase",
+        cmdline("--recreate-schema"), "Drop and recreate the specified schema. "
+        "Schema will be deleted if it exists!");
 
   add_startup_options(true)
     (cmdline("--mx", "--mysqlx"),
@@ -508,7 +530,7 @@ Shell_options::Shell_options(
         std::bind(
             &Shell_options::override_session_type, this, _1, _2))
     (cmdline("--js", "--javascript"), "Start in JavaScript mode.",
-#ifdef HAVE_JS
+#ifdef HAVE_V8
         [this](const std::string&, const char*) {
           storage.initial_mode = shcore::IShell_core::Mode::JavaScript;
         }
@@ -627,6 +649,7 @@ Shell_options::Shell_options(
         }
       });
 
+
   // make sure hack for accessing log_level via Value works
   static_assert(
       std::is_integral<
@@ -944,6 +967,7 @@ Shell_options::Shell_options(
 
     check_password_conflicts();
     check_ssh_conflicts();
+
     if (!flags.is_set(Option_flags::CONNECTION_ONLY)) {
       check_file_execute_conflicts();
       check_import_options();
@@ -1205,12 +1229,6 @@ bool Shell_options::custom_cmdline_handler(Iterator *iterator) {
       iterator->next_no_value();
     }
   } else if ("--import" == option) {
-    m_on_warning(
-        "WARNING: The --import option was deprecated and will be removed in "
-        "a future version of the MySQL Shell. Please consider using the CLI "
-        "call for import-json instead.\nFor additional information: mysqlsh -- "
-        "util import-json --help");
-
     const auto cmdline = iterator->iterator();
 
     storage.import_args.push_back(cmdline->get());  // omit --import
