@@ -1,17 +1,15 @@
-/* Copyright (c) 2017, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2017, 2023, Oracle and/or its affiliates.
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2.0,
  as published by the Free Software Foundation.
 
- This program is designed to work with certain software (including
- but not limited to OpenSSL) that is licensed under separate terms,
- as designated in a particular file or component or in included license
+ This program is also distributed with certain software (including
+ but not limited to OpenSSL) that is licensed under separate terms, as
+ designated in a particular file or component or in included license
  documentation.  The authors of MySQL hereby grant you an additional
  permission to link the program and your derivative works with the
- separately licensed software that they have either included with
- the program or referenced in the documentation.
-
+ separately licensed software that they have included with MySQL.
  This program is distributed in the hope that it will be useful,  but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
@@ -22,6 +20,8 @@
  51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA */
 
 #include <string>
+
+#include "mysqlshdk/libs/utils/utils_general.h"
 #include "unittest/test_utils/command_line_test.h"
 #include "utils/utils_string.h"
 
@@ -836,4 +836,25 @@ TEST_F(Command_line_connection_test, invalid_port) {
   MY_EXPECT_CMD_OUTPUT_CONTAINS("Invalid value for -P: invalidPort3");
 }
 
+TEST_F(Command_line_connection_test, kerberos_auth_mode) {
+  execute({_mysqlsh, _uri.c_str(),
+           "--plugin-authentication-kerberos-client-mode=InvalidMode", "--port",
+           "invalidPort", NULL});
+#ifdef _WIN32
+  MY_EXPECT_CMD_OUTPUT_CONTAINS(
+      "Invalid value: InvalidMode. Allowed values: SSPI, GSSAPI.");
+
+  execute({_mysqlsh, _uri.c_str(),
+           "--plugin-authentication-kerberos-client-mode=GsSApi", "--port",
+           "invalidPort", NULL});
+
+  MY_EXPECT_CMD_OUTPUT_NOT_CONTAINS("GsSApi");
+  MY_EXPECT_CMD_OUTPUT_CONTAINS("Invalid value for --port: invalidPort");
+
+#else
+  MY_EXPECT_CMD_OUTPUT_CONTAINS(
+      "unknown option --plugin-authentication-kerberos-client-mod");
+#endif
+}
+
 }  // namespace tests
