@@ -1,18 +1,16 @@
 /*
- * Copyright (c) 2018, 2024, Oracle and/or its affiliates.
+ * Copyright (c) 2018, 2023, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms,
- * as designated in a particular file or component or in included license
+ * This program is also distributed with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
- *
+ * separately licensed software that they have included with MySQL.
  * This program is distributed in the hope that it will be useful,  but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
@@ -109,15 +107,7 @@ TEST_F(Preconditions, check_session) {
   // Test invalid server version
   // Test different invalid server versions
   std::vector<std::string> min_invalid_versions = {"5.6", "5.1", "5.6.99"};
-
-  std::vector<std::string> max_invalid_versions = {
-      shcore::str_format("%d.0",
-                         mysqlshdk::utils::k_shell_version.get_major() + 1),
-      shcore::str_format("%d.%d.0",
-                         mysqlshdk::utils::k_shell_version.get_major(),
-                         mysqlshdk::utils::k_shell_version.get_minor() + 1),
-      shcore::str_format("%d.%d", mysqlshdk::utils::k_shell_version.get_major(),
-                         mysqlshdk::utils::k_shell_version.get_minor() + 2)};
+  std::vector<std::string> max_invalid_versions = {"9.0", "8.2.0", "8.3"};
 
   EXPECT_CALL(*m_mock_metadata, get_md_server())
       .Times(min_invalid_versions.size());
@@ -153,45 +143,21 @@ TEST_F(Preconditions, check_session) {
       ADD_FAILURE();
     } catch (const shcore::Exception &e) {
       EXPECT_STREQ(
-          shcore::str_format(
-              "Unsupported server version: AdminAPI operations in this version "
-              "of MySQL Shell support MySQL Server up to version %d.%d",
-              mysqlshdk::utils::k_shell_version.get_major(),
-              mysqlshdk::utils::k_shell_version.get_minor())
-              .c_str(),
+          "Unsupported server version: AdminAPI operations in this version of "
+          "MySQL Shell support MySQL Server up to version 8.1",
           e.what());
     }
   }
 
-  std::vector<std::string> valid_versions = {"5.7", "5.7.22"};
-
-  valid_versions.push_back(shcore::str_format(
-      "%d.%d.%d", mysqlshdk::utils::k_shell_version.get_major(),
-      mysqlshdk::utils::k_shell_version.get_minor(),
-      mysqlshdk::utils::k_shell_version.get_patch()));
-
-  // Now inserts corner cases for the different versions starting at 8.0
-  // Valid versions start at 8.0
-  int lower_major = 8;
-  for (auto major = lower_major;
-       major <= mysqlshdk::utils::k_shell_version.get_major(); major++) {
-    // Valid minor versions end at current minor for the last release or at 6
-    // for previous releases
-    int max_minor = major < mysqlshdk::utils::k_shell_version.get_major()
-                        ? 6
-                        : mysqlshdk::utils::k_shell_version.get_minor();
-    for (auto minor = 0; minor <= max_minor; minor++) {
-      valid_versions.push_back(shcore::str_format("%d.%d", major, minor));
-      valid_versions.push_back(shcore::str_format("%d.%d.1", major, minor));
-      valid_versions.push_back(shcore::str_format("%d.%d.99", major, minor));
-    }
-  }
+  std::vector<std::string> valid_versions = {
+      "8.0", "8.0.1", "8.0.11", "8.0.99", "5.7", "5.7.22", "8.1", "8.1.15"};
   EXPECT_CALL(*m_mock_metadata, get_md_server()).Times(valid_versions.size());
   EXPECT_CALL(*m_mock_session, is_open()).Times(valid_versions.size());
 
   for (const auto &version : valid_versions) {
     EXPECT_CALL(*m_mock_session, get_server_version())
         .WillOnce(Return(mysqlshdk::utils::Version(version)));
+
     try {
       m_preconditions->check_session();
 
@@ -558,6 +524,7 @@ TEST_F(Preconditions, check_cluster_set_preconditions) {
       "Cluster.listRouters",
       "Cluster.setPrimaryInstance",
       "Cluster.options",
+      "Cluster.routingOptions",
       "Cluster.status",
       "Cluster.dissolve",
       "Dba.getCluster",
@@ -567,6 +534,7 @@ TEST_F(Preconditions, check_cluster_set_preconditions) {
 
   std::set<std::string> cset_sometimes_allowed_expected = {
       "Cluster.addInstance",
+      "Cluster.addReplicaInstance",
       "Cluster.rejoinInstance",
       "Cluster.removeInstance",
       "Cluster.removeRouterMetadata",
@@ -574,6 +542,7 @@ TEST_F(Preconditions, check_cluster_set_preconditions) {
       "Cluster.resetRecoveryAccountsPassword",
       "Cluster.setInstanceOption",
       "Cluster.setOption",
+      "Cluster.setRoutingOption",
       "Cluster.setupAdminAccount",
       "Cluster.setupRouterAccount"};
 
@@ -630,7 +599,7 @@ TEST_F(Preconditions, check_cluster_set_preconditions) {
       cset_sometimes_allowed.insert(precondition.first);
       for (const auto &global_state : global_cluster_states) {
         EXPECT_CALL(checker, get_cluster_global_state())
-            .WillRepeatedly(Return(global_state));
+            .WillOnce(Return(global_state));
         if (precondition.second.cluster_set_state.is_set(global_state.first)) {
           EXPECT_NO_THROW(checker.check_cluster_set_preconditions(
               precondition.second.cluster_set_state));
