@@ -1,18 +1,16 @@
 /*
- * Copyright (c) 2017, 2024, Oracle and/or its affiliates.
+ * Copyright (c) 2017, 2023, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms,
- * as designated in a particular file or component or in included license
+ * This program is also distributed with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
- *
+ * separately licensed software that they have included with MySQL.
  * This program is distributed in the hope that it will be useful,  but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
@@ -66,6 +64,7 @@ class mod_shell_test : public Shell_core_test_wrapper {
   std::shared_ptr<mysqlsh::Shell> _shell;
 };
 
+// clang-format off
 TEST_F(mod_shell_test, parse_uri) {
   {
     std::string args;
@@ -283,6 +282,60 @@ TEST_F(mod_shell_test, parse_uri) {
         "ECDSA-AES256-GCM-SHA384&tls-version=TLSv1.1%2CTLSv1.2",
         _shell->unparse_uri(dict));
   }
+  // clang-format on
+
+#ifdef _WIN32
+  {
+    std::string args;
+    args = "user@host?plugin-authentication-kerberos-client-mode=SSPI";
+
+    auto dict = _shell->parse_uri(args);
+
+    EXPECT_TRUE(dict->has_key(mysqlshdk::db::kKerberosClientAuthMode));
+    EXPECT_STREQ(
+        mysqlshdk::db::kKerberosAuthModeSSPI,
+        dict->get_string(mysqlshdk::db::kKerberosClientAuthMode).c_str());
+
+    // test for case-sensitivity
+    args = "user@host?plugin-authentication-kerberos-client-mode=GsSapi";
+
+    dict = _shell->parse_uri(args);
+
+    EXPECT_TRUE(dict->has_key(mysqlshdk::db::kKerberosClientAuthMode));
+    EXPECT_STREQ(
+        mysqlshdk::db::kKerberosAuthModeGSSAPI,
+        dict->get_string(mysqlshdk::db::kKerberosClientAuthMode).c_str());
+  }
+#else
+  {
+    std::string args;
+    args = "user@host?plugin-authentication-kerberos-client-mode=SSPI";
+
+    EXPECT_THROW_MSG_CONTAINS(_shell->parse_uri(args), std::exception,
+                              "Invalid URI: Invalid connection option "
+                              "'plugin-authentication-kerberos-client-mode'.");
+  }
+#endif
+
+  {
+    std::string args;
+    args =
+        "user@host?oci-config-file=config.cfg&authentication-oci-client-config-"
+        "profile=name";
+
+    auto dict = _shell->parse_uri(args);
+
+    EXPECT_TRUE(dict->has_key(mysqlshdk::db::kOciConfigFile));
+    EXPECT_TRUE(
+        dict->has_key(mysqlshdk::db::kOciAuthenticationClientConfigProfile));
+
+    EXPECT_STREQ("config.cfg",
+                 dict->get_string(mysqlshdk::db::kOciConfigFile).c_str());
+    EXPECT_STREQ(
+        "name",
+        dict->get_string(mysqlshdk::db::kOciAuthenticationClientConfigProfile)
+            .c_str());
+  }
 }
 
 TEST_F(mod_shell_test, connect) {
@@ -442,5 +495,4 @@ TEST_F(mod_shell_test, dump_rows) {
     }
   }
 }
-
 }  // namespace testing
