@@ -21,12 +21,12 @@ FUNCTIONS
       addInstance(instance[, options])
             Adds an Instance to the cluster.
 
+      addReplicaInstance(instance[, options])
+            Adds a Read Replica Instance to the Cluster.
+
       checkInstanceState(instance)
             Verifies the instance gtid state in relation to the cluster.
 
-            ATTENTION: This function is deprecated and will be removed in a
-                       future release of MySQL Shell.
-
       createClusterSet(domainName[, options])
             Creates a MySQL InnoDB ClusterSet from an existing standalone
             InnoDB Cluster.
@@ -79,6 +79,9 @@ FUNCTIONS
       resetRecoveryAccountsPassword(options)
             Reset the password of the recovery accounts of the cluster.
 
+      routingOptions([router])
+            Lists the Cluster Routers configuration options.
+
       setInstanceOption(instance, option, value)
             Changes the value of an option in a Cluster member.
 
@@ -88,6 +91,10 @@ FUNCTIONS
       setPrimaryInstance(instance[, options])
             Elects a specific cluster member as the new primary.
 
+      setRoutingOption([router], option, value)
+            Changes the value of either a global Cluster Routing option or of a
+            single Router instance.
+
       setupAdminAccount(user, options)
             Create or upgrade an InnoDB Cluster admin account.
 
@@ -142,9 +149,7 @@ DESCRIPTION
       - recoveryMethod: Preferred method of state recovery. May be auto, clone
         or incremental. Default is auto.
       - waitRecovery: Integer value to indicate if the command shall wait for
-        the recovery process to finish and its verbosity level. Deprecated.
-      - recoveryProgress: Integer value to indicate the recovery process
-        verbosity level.
+        the recovery process to finish and its verbosity level.
       - password: the instance connection password
       - memberSslMode: SSL mode used on the instance
       - ipWhitelist: The list of hosts allowed to connect to the instance for
@@ -159,7 +164,7 @@ DESCRIPTION
       - interactive: boolean value used to disable/enable the wizards in the
         command execution, i.e. prompts and confirmations will be provided or
         not according to the value set. The default value is equal to MySQL
-        Shell wizard mode. Deprecated.
+        Shell wizard mode.
       - exitStateAction: string value indicating the group replication exit
         state action.
       - memberWeight: integer value with a percentage weight for automatic
@@ -198,12 +203,6 @@ DESCRIPTION
       - 3: block until the recovery process finishes and show progress using
         progress bars.
 
-      The recoveryProgress option supports the following values:
-
-      - 0: do not show any progress information.
-      - 1: show detailed static progress information.
-      - 2: show detailed dynamic progress information using progress bars.
-
       By default, if the standard output on which the Shell is running refers
       to a terminal, the waitRecovery option has the value of 3. Otherwise, it
       has the value of 2.
@@ -285,13 +284,6 @@ DESCRIPTION
 
       ATTENTION: The groupSeeds option will be removed in a future release.
 
-      ATTENTION: The waitRecovery option will be removed in a future release.
-                 Please use the recoveryProgress option instead.
-
-      ATTENTION: The interactive option will be removed in a future release.
-
-      ATTENTION: The password option will be removed in a future release.
-
 //@<OUT> Check Instance State
 NAME
       checkInstanceState - Verifies the instance gtid state in relation to the
@@ -307,9 +299,6 @@ RETURNS
       resultset A JSON object with the status.
 
 DESCRIPTION
-      ATTENTION: This function is deprecated and will be removed in a future
-                 release of MySQL Shell.
-
       Analyzes the instance executed GTIDs with the executed/purged GTIDs on
       the cluster to determine if the instance is valid for the cluster.
 
@@ -413,7 +402,7 @@ DESCRIPTION
       - interactive: boolean value used to disable/enable the wizards in the
         command execution, i.e. prompts and confirmations will be provided or
         not according to the value set. The default value is equal to MySQL
-        Shell wizard mode. Deprecated.
+        Shell wizard mode.
 
       The force option (set to true) must only be used to dissolve a cluster
       with instances that are permanently not available (no longer reachable)
@@ -423,8 +412,6 @@ DESCRIPTION
       and the cluster dissolved without the force option to avoid errors trying
       to reuse the instances and add them back to a cluster.
 
-      ATTENTION: The interactive option will be removed in a future release.
-
 //@<OUT> Force Quorum Using Partition Of
 NAME
       forceQuorumUsingPartitionOf - Restores the cluster from quorum loss.
@@ -457,8 +444,6 @@ DESCRIPTION
       point of view of the given instance definition will be added to the
       group.
 
-      ATTENTION: The password option will be removed in a future release.
-
 //@<OUT> Get Name
 NAME
       getName - Retrieves the name of the cluster.
@@ -490,11 +475,11 @@ WHERE
       options: Dictionary with options for the operation.
 
 RETURNS
-      A JSON object listing the Router instances associated to the cluster.
+      A JSON object listing the Router instances associated to the Cluster.
 
 DESCRIPTION
       This function lists and provides information about all Router instances
-      registered for the cluster.
+      registered for the Cluster.
 
       Whenever a Metadata Schema upgrade is necessary, the recommended process
       is to upgrade MySQL Router instances to the latest version before
@@ -574,21 +559,61 @@ DESCRIPTION
       The options dictionary may contain the following attributes:
 
       - password: the instance connection password
+      - recoveryMethod: Preferred method of state recovery. May be auto, clone
+        or incremental. Default is auto.
+      - recoveryProgress: Integer value to indicate the recovery process
+        verbosity level. recovery process to finish and its verbosity level.
       - memberSslMode: SSL mode used on the instance
       - interactive: boolean value used to disable/enable the wizards in the
         command execution, i.e. prompts and confirmations will be provided or
         not according to the value set. The default value is equal to MySQL
-        Shell wizard mode. Deprecated.
+        Shell wizard mode.
       - ipWhitelist: The list of hosts allowed to connect to the instance for
         group replication. Deprecated.
       - ipAllowlist: The list of hosts allowed to connect to the instance for
         group replication. Only valid if communicationStack=XCOM.
       - localAddress: string value with the Group Replication local address to
         be used instead of the automatically generated one.
+      - dryRun: boolean if true, all validations and steps for rejoining the
+        instance are executed, but no changes are actually made.
+      - cloneDonor: The Cluster member to be used as donor when performing
+        clone-based recovery.
+      - timeout: maximum number of seconds to wait for the instance to sync up
+        with the PRIMARY after it's provisioned and the replication channel is
+        established. If reached, the operation is rolled-back. Default is 0 (no
+        timeout).
 
       The password may be contained on the instance definition, however, it can
       be overwritten if it is specified on the options.
 
+      The recoveryMethod option supports the following values:
+
+      - incremental: uses distributed state recovery, which applies missing
+        transactions copied from another cluster member. Clone will be
+        disabled.
+      - clone: clone: uses built-in MySQL clone support, which completely
+        replaces the state of the target instance with a full snapshot of
+        another cluster member before distributed recovery starts. Requires
+        MySQL 8.0.17 or newer.
+      - auto: let Group Replication choose whether or not a full snapshot has
+        to be taken, based on what the target server supports and the
+        group_replication_clone_threshold sysvar. This is the default value. A
+        prompt will be shown if not possible to safely determine a safe way
+        forward. If interaction is disabled, the operation will be canceled
+        instead.
+
+      If recoveryMethod is not specified 'auto' will be used by default.
+
+      The recoveryProgress option supports the following values:
+
+      - 0: do not show any progress information.
+      - 1: show detailed static progress information.
+      - 2: show detailed dynamic progress information using progress bars.
+
+      By default, if the standard output on which the Shell is running refers
+      to a terminal, the recoveryProgress option has the value of 2. Otherwise,
+      it has the value of 1.
+
       The ipAllowlist format is a comma separated list of IP addresses or
       subnet CIDR notation, for example: 192.168.1.0/24,10.0.0.1. By default
       the value is set to AUTOMATIC, allowing addresses from the instance
@@ -618,10 +643,6 @@ DESCRIPTION
       ATTENTION: The ipWhitelist option will be removed in a future release.
                  Please use the ipAllowlist option instead.
 
-      ATTENTION: The interactive option will be removed in a future release.
-
-      ATTENTION: The password option will be removed in a future release.
-
 //@<OUT> Remove Instance
 NAME
       removeInstance - Removes an Instance from the cluster.
@@ -652,7 +673,13 @@ DESCRIPTION
       - interactive: boolean value used to disable/enable the wizards in the
         command execution, i.e. prompts and confirmations will be provided or
         not according to the value set. The default value is equal to MySQL
-        Shell wizard mode. Deprecated.
+        Shell wizard mode.
+      - dryRun: boolean if true, all validations and steps for removing the
+        instance are executed, but no changes are actually made. An exception
+        will be thrown when finished.
+      - timeout: maximum number of seconds to wait for the instance to sync up
+        with the PRIMARY. If reached, the operation is rolled-back. Default is
+        0 (no timeout).
 
       The password may be contained in the instance definition, however, it can
       be overwritten if it is specified on the options.
@@ -664,10 +691,6 @@ DESCRIPTION
       back ONLINE and removed without the force option to avoid errors trying
       to add it back to a cluster.
 
-      ATTENTION: The interactive option will be removed in a future release.
-
-      ATTENTION: The password option will be removed in a future release.
-
 //@<OUT> SetInstanceOption
 NAME
       setInstanceOption - Changes the value of an option in a Cluster member.
@@ -703,6 +726,7 @@ DESCRIPTION
       - ipAllowlist: The list of hosts allowed to connect to the instance for
         group replication. Only valid if communicationStack=XCOM.
       - label a string identifier of the instance.
+      - replicationSources: The list of sources for a Read Replica Instance.
 
       The exitStateAction option supports the following values:
 
@@ -773,6 +797,21 @@ DESCRIPTION
             useful to shut down the instance and perform maintenance on it
             without disrupting incoming application traffic.
 
+      The replicationSources is a comma separated list of instances (host:port)
+      to act as sources of the replication channel, i.e. to provide source
+      failover of the channel. The first member of the list is configured with
+      the highest priority among all members so when the channel activates it
+      will be chosen for the first connection attempt. By default, the source
+      list is automatically managed by Group Replication according to the
+      current group membership and the primary member of the Cluster is the
+      current source for the replication channel, this is the same as setting
+      to "primary". Alternatively, it's possible to set to "secondary" to
+      instruct Group Replication to automatically manage the list but use a
+      secondary member of the Cluster as source.
+
+      For the change to be effective, the Read-Replica must be reconfigured
+      after using Cluster.rejoinInstance().
+
 //@<OUT> SetOption
 NAME
       setOption - Changes the value of an option for the whole Cluster.
@@ -965,7 +1004,7 @@ SYNTAX
       <Cluster>.setupAdminAccount(user, options)
 
 WHERE
-      user: Name of the InnoDB cluster administrator account.
+      user: Name of the InnoDB Cluster administrator account.
       options: Dictionary with options for the operation.
 
 RETURNS
@@ -973,7 +1012,7 @@ RETURNS
 
 DESCRIPTION
       This function creates/upgrades a MySQL user account with the necessary
-      privileges to administer an InnoDB cluster.
+      privileges to administer an InnoDB Cluster.
 
       This function also allows a user to upgrade an existing admin account
       with the necessary privileges before a dba.upgradeMetadata() call.
@@ -985,7 +1024,7 @@ DESCRIPTION
 
       The options dictionary may contain the following attributes:
 
-      - password: The password for the InnoDB cluster administrator account.
+      - password: The password for the InnoDB Cluster administrator account.
       - passwordExpiration: Password expiration setting for the account. May be
         set to the number of days for expiration, 'NEVER' to disable expiration
         and 'DEFAULT' to use the system default.
@@ -996,7 +1035,7 @@ DESCRIPTION
       - interactive: boolean value used to disable/enable the wizards in the
         command execution, i.e. prompts and confirmations will be provided or
         not according to the value set. The default value is equal to MySQL
-        Shell wizard mode. Deprecated.
+        Shell wizard mode.
       - update: boolean value that must be enabled to allow updating the
         privileges and/or password of existing accounts. Default value is
         False.
@@ -1018,8 +1057,6 @@ DESCRIPTION
       options or vice-versa but certificate options can only be changed
       together.
 
-      ATTENTION: The interactive option will be removed in a future release.
-
 //@<OUT> setupRouterAccount
 NAME
       setupRouterAccount - Create or upgrade a MySQL account to use with MySQL
@@ -1061,7 +1098,7 @@ DESCRIPTION
       - interactive: boolean value used to disable/enable the wizards in the
         command execution, i.e. prompts and confirmations will be provided or
         not according to the value set. The default value is equal to MySQL
-        Shell wizard mode. Deprecated.
+        Shell wizard mode.
       - update: boolean value that must be enabled to allow updating the
         privileges and/or password of existing accounts. Default value is
         False.
@@ -1083,8 +1120,6 @@ DESCRIPTION
       options or vice-versa but certificate options can only be changed
       together.
 
-      ATTENTION: The interactive option will be removed in a future release.
-
 //@<OUT> Rescan
 NAME
       rescan - Rescans the cluster.
@@ -1111,7 +1146,7 @@ DESCRIPTION
       - interactive: boolean value used to disable/enable the wizards in the
         command execution, i.e. prompts and confirmations will be provided or
         not according to the value set. The default value is equal to MySQL
-        Shell wizard mode. Deprecated.
+        Shell wizard mode.
       - removeInstances: List with the connection data of the obsolete
         instances to remove from the metadata, or "auto" to automatically
         remove obsolete instances from the metadata.
@@ -1124,8 +1159,6 @@ DESCRIPTION
       - updateViewChangeUuid: boolean value used to indicate if the command
         should generate and set a value for Group Replication View Change UUID
         in the whole Cluster. Required for InnoDB ClusterSet usage.
-      - repairMetadata: boolean. Set to true to repair the Metadata if detected
-        to be inconsistent.
 
       The value for addInstances and removeInstances is used to specify which
       instances to add or remove from the metadata, respectively. Both options
@@ -1133,16 +1166,9 @@ DESCRIPTION
       for both options in order to automatically add or remove the instances in
       the metadata, without having to explicitly specify them.
 
-      'repairMetadata' is used to eliminate any inconsistencies detected in the
-      Metadata. These inconsistencies may arise from a few scenarios, such as
-      the failure of one or more commands. Clusters detected in the ClusterSet
-      Metadata that do not qualify as valid members will be removed.
-
       ATTENTION: The updateTopologyMode option will be removed in a future
                  release.
 
-      ATTENTION: The interactive option will be removed in a future release.
-
 //@<OUT> Status
 NAME
       status - Describe the status of the cluster.
@@ -1285,7 +1311,7 @@ DESCRIPTION
       - interactive: boolean value used to disable/enable the wizards in the
         command execution, i.e. prompts and confirmations will be provided or
         not according to the value set. The default value is equal to MySQL
-        Shell wizard mode. Deprecated.
+        Shell wizard mode.
 
       The use of the force option (set to true) is not recommended. Use it only
       if really needed when instances are permanently not available (no longer
@@ -1293,8 +1319,6 @@ DESCRIPTION
       bring the non available instances back ONLINE or remove them from the
       cluster if they will no longer be used.
 
-      ATTENTION: The interactive option will be removed in a future release.
-
 //@<OUT> createClusterSet
 NAME
       createClusterSet - Creates a MySQL InnoDB ClusterSet from an existing
@@ -1472,3 +1496,161 @@ DESCRIPTION
       ATTENTION: This function does not unfence Clusters that have been fenced
                  to ALL traffic. Those Cluster are completely shut down and can
                  only be restored using dba.rebootClusterFromCompleteOutage().
+
+//@<OUT> addReplicaInstance
+NAME
+      addReplicaInstance - Adds a Read Replica Instance to the Cluster.
+
+SYNTAX
+      <Cluster>.addReplicaInstance(instance[, options])
+
+WHERE
+      instance: host:port of the target instance to be added as a Read Replica.
+      options: Dictionary with additional parameters described below.
+
+RETURNS
+      nothing
+
+DESCRIPTION
+      This function adds an Instance acting as Read-Replica of an InnoDB
+      Cluster.
+
+      Pre-requisites
+
+      The following is a list of requirements to create a REPLICA cluster:
+
+      - The target instance must be a standalone instance.
+      - The target instance and Cluster must running MySQL 8.0.23 or newer.
+      - Unmanaged replication channels are not allowed.
+      - The target instance server_id and server_uuid must be unique in the
+        topology, including among OFFLINE or unreachable members
+
+      Options
+
+      The options dictionary may contain the following values:
+
+      - dryRun: boolean if true, all validations and steps for creating a Read
+        Replica Instance are executed, but no changes are actually made. An
+        exception will be thrown when finished.
+      - label: an identifier for the Read Replica Instance being added, used in
+        the output of status() and describe().
+      - replicationSources: The list of sources for the Read Replica Instance.
+        By default, the list is automatically managed by Group Replication and
+        the primary member is used as source.
+      - recoveryMethod: Preferred method for state recovery/provisioning. May
+        be auto, clone or incremental. Default is auto.
+      - recoveryProgress: Integer value to indicate the recovery process
+        verbosity level.
+      - timeout: maximum number of seconds to wait for the instance to sync up
+        with the PRIMARY after it's provisioned and the replication channel is
+        established. If reached, the operation is rolled-back. Default is 0 (no
+        timeout).
+      - cloneDonor: The Cluster member to be used as donor when performing
+        clone-based recovery.
+
+      The replicationSources is a comma separated list of instances (host:port)
+      to act as sources of the replication channel, i.e. to provide source
+      failover of the channel. The first member of the list is configured with
+      the highest priority among all members so when the channel activates it
+      will be chosen for the first connection attempt. By default, the source
+      list is automatically managed by Group Replication according to the
+      current group membership and the primary member of the Cluster is the
+      current source for the replication channel, this is the same as setting
+      to "primary". Alternatively, it's possible to set to "secondary" to
+      instruct Group Replication to automatically manage the list too but use a
+      secondary member of the Cluster as source.
+
+      The recoveryMethod option supports the following values:
+
+      - incremental: waits until the new instance has applied missing
+        transactions from the source.
+      - clone: uses MySQL clone to provision the instance, which completely
+        replaces the state of the target instance with a full snapshot of the
+        instance's source.
+      - auto: compares the transaction set of the instance with that of the
+        source to determine if incremental recovery is safe to be automatically
+        chosen as the most appropriate recovery method. A prompt will be shown
+        if not possible to safely determine a safe way forward. If interaction
+        is disabled, the operation will be canceled instead.
+
+      If recoveryMethod is not specified 'auto' will be used by default.
+
+      The recoveryProgress option supports the following values:
+
+      - 0: do not show any progress information.
+      - 1: show detailed static progress information.
+      - 2: show detailed dynamic progress information using progress bars.
+
+      By default, if the standard output on which the Shell is running refers
+      to a terminal, the recoveryProgress option has the value of 2. Otherwise,
+      it has the value of 1.
+
+//@<OUT> routingOptions
+NAME
+      routingOptions - Lists the Cluster Routers configuration options.
+
+SYNTAX
+      <Cluster>.routingOptions([router])
+
+WHERE
+      router: Identifier of the router instance to query for the options.
+
+RETURNS
+      A JSON object describing the configuration options of all router
+      instances of the Cluster and its global options or just the given Router.
+
+DESCRIPTION
+      This function lists the Router configuration options of all Routers of
+      the Cluster or the target Router.
+
+//@<OUT> setRoutingOption
+NAME
+      setRoutingOption - Changes the value of either a global Cluster Routing
+                         option or of a single Router instance.
+
+SYNTAX
+      <Cluster>.setRoutingOption([router], option, value)
+
+WHERE
+      router: Identifier of the target router instance (e.g.
+              192.168.45.70::system).
+      option: The Router option to be changed.
+      value: The value that the option shall get (or null to unset).
+
+RETURNS
+      Nothing.
+
+DESCRIPTION
+      The accepted options are:
+
+      - tags: Associates an arbitrary JSON object with custom key/value pairs
+        with the Cluster metadata.
+      - read_only_targets: Routing policy to define Router's usage of Read Only
+        instance. Default is 'secondaries'.
+      - stats_updates_frequency: Number of seconds between updates that the
+        Router is to make to its statistics in the InnoDB Cluster metadata.
+
+      The read_only_targets option supports the following values:
+
+      - all: All Read Replicas of the target Cluster should be used along the
+        other SECONDARY Cluster members for R/O traffic.
+      - read_replicas: Only Read Replicas of the target Cluster should be used
+        for R/O traffic.
+      - secondaries: Only Secondary members of the target Cluster should be
+        used for R/O traffic (default).
+
+      The stats_updates_frequency option accepts positive integers and sets the
+      frequency of updates of Router stats (timestamp, version, etc.), in
+      seconds, in the Metadata. If set to 0 (default), no periodic updates are
+      done. Router will round up the value to be a multiple of Router's TTL,
+      i.e.:
+
+      - If lower than TTL its gets rounded up to TTL, e.g. TTL=30, and
+        stats_updates_frequency=1, effective frequency is 30 seconds.
+      - If not a multiple of TTL it will be rounded up and adjusted according
+        to the TTL, e.g. TTL=5, stats_updates_frequency=11, effective frequency
+        is 15 seconds; TTL=5, stats_updates_frequency=13, effective frequency
+        is 15 seconds.
+
+      If the value is null, the option value is cleared and the default value
+      (0) takes effect.
