@@ -42,6 +42,9 @@ FUNCTIONS
       removeRouterMetadata(routerDef)
             Removes metadata for a router instance.
 
+      routingOptions([router])
+            Lists the ReplicaSet Routers configuration options.
+
       setInstanceOption(instance, option, value)
             Changes the value of an option in a ReplicaSet member.
 
@@ -51,6 +54,10 @@ FUNCTIONS
       setPrimaryInstance(instance, options)
             Performs a safe PRIMARY switchover, promoting the given instance.
 
+      setRoutingOption([router], option, value)
+            Changes the value of either a global ReplicaSet Routing option or
+            of a single Router instance.
+
       setupAdminAccount(user, options)
             Create or upgrade an InnoDB ReplicaSet admin account.
 
@@ -136,14 +143,12 @@ DESCRIPTION
         or incremental. Default is auto.
       - waitRecovery: Integer value to indicate the recovery process verbosity
         level.
-      - recoveryProgress: Integer value to indicate the recovery process
-        verbosity level.
       - cloneDonor: host:port of an existing replicaSet member to clone from.
         IPv6 addresses are not supported for this option.
       - interactive: boolean value used to disable/enable the wizards in the
         command execution, i.e. prompts and confirmations will be provided or
         not according to the value set. The default value is equal to MySQL
-        Shell wizard mode. Deprecated.
+        Shell wizard mode.
       - timeout: timeout in seconds for transaction sync operations; 0 disables
         timeout and force the Shell to wait until the transaction sync
         finishes. Defaults to 0.
@@ -172,12 +177,6 @@ DESCRIPTION
       - 2: show detailed static progress information.
       - 3: show detailed dynamic progress information using progress bars.
 
-      The recoveryProgress option supports the following values:
-
-      - 0: do not show any progress information.
-      - 1: show detailed static progress information.
-      - 2: show detailed dynamic progress information using progress bars.
-
       By default, if the standard output on which the Shell is running refers
       to a terminal, the waitRecovery option has the value of 3. Otherwise, it
       has the value of 2.
@@ -188,11 +187,6 @@ DESCRIPTION
       members are available the PRIMARY will be selected. The option accepts
       values in the format: 'host:port'. IPv6 addresses are not supported.
 
-      ATTENTION: The waitRecovery option will be removed in a future release.
-                 Please use the recoveryProgress option instead.
-
-      ATTENTION: The interactive option will be removed in a future release.
-
 //@<OUT> Disconnect
 NAME
       disconnect - Disconnects all internal sessions used by the replicaset
@@ -341,14 +335,12 @@ DESCRIPTION
         or incremental. Default is auto.
       - waitRecovery: Integer value to indicate the recovery process verbosity
         level.
-      - recoveryProgress: Integer value to indicate the recovery process
-        verbosity level.
       - cloneDonor: host:port of an existing replicaSet member to clone from.
         IPv6 addresses are not supported for this option.
       - interactive: boolean value used to disable/enable the wizards in the
         command execution, i.e. prompts and confirmations will be provided or
         not according to the value set. The default value is equal to MySQL
-        Shell wizard mode. Deprecated.
+        Shell wizard mode.
       - timeout: timeout in seconds for transaction sync operations; 0 disables
         timeout and force the Shell to wait until the transaction sync
         finishes. Defaults to 0.
@@ -375,12 +367,6 @@ DESCRIPTION
       - 2: show detailed static progress information.
       - 3: show detailed dynamic progress information using progress bars.
 
-      The recoveryProgress option supports the following values:
-
-      - 0: do not show any progress information.
-      - 1: show detailed static progress information.
-      - 2: show detailed dynamic progress information using progress bars.
-
       By default, if the standard output on which the Shell is running refers
       to a terminal, the waitRecovery option has the value of 3. Otherwise, it
       has the value of 2.
@@ -391,11 +377,6 @@ DESCRIPTION
       members are available the PRIMARY will be selected. The option accepts
       values in the format: 'host:port'. IPv6 addresses are not supported.
 
-      ATTENTION: The waitRecovery option will be removed in a future release.
-                 Please use the recoveryProgress option instead.
-
-      ATTENTION: The interactive option will be removed in a future release.
-
 //@<OUT> Remove Instance
 NAME
       removeInstance - Removes an Instance from the replicaset.
@@ -521,7 +502,7 @@ DESCRIPTION
       - interactive: boolean value used to disable/enable the wizards in the
         command execution, i.e. prompts and confirmations will be provided or
         not according to the value set. The default value is equal to MySQL
-        Shell wizard mode. Deprecated.
+        Shell wizard mode.
       - update: boolean value that must be enabled to allow updating the
         privileges and/or password of existing accounts. Default value is
         False.
@@ -543,8 +524,6 @@ DESCRIPTION
       options or vice-versa but certificate options can only be changed
       together.
 
-      ATTENTION: The interactive option will be removed in a future release.
-
 //@<OUT> setupRouterAccount
 NAME
       setupRouterAccount - Create or upgrade a MySQL account to use with MySQL
@@ -586,7 +565,7 @@ DESCRIPTION
       - interactive: boolean value used to disable/enable the wizards in the
         command execution, i.e. prompts and confirmations will be provided or
         not according to the value set. The default value is equal to MySQL
-        Shell wizard mode. Deprecated.
+        Shell wizard mode.
       - update: boolean value that must be enabled to allow updating the
         privileges and/or password of existing accounts. Default value is
         False.
@@ -608,6 +587,66 @@ DESCRIPTION
       options or vice-versa but certificate options can only be changed
       together.
 
+//@<OUT> routingOptions
+NAME
+      routingOptions - Lists the ReplicaSet Routers configuration options.
+
+SYNTAX
+      <ReplicaSet>.routingOptions([router])
+
+WHERE
+      router: Identifier of the router instance to query for the options.
+
+RETURNS
+      A JSON object describing the configuration options of all router
+      instances of the ReplicaSet and its global options or just the given
+      Router.
+
+DESCRIPTION
+      This function lists the Router configuration options of all Routers of
+      the ReplicaSet or the target Router.
+
+//@<OUT> setRoutingOption
+NAME
+      setRoutingOption - Changes the value of either a global ReplicaSet
+                         Routing option or of a single Router instance.
+
+SYNTAX
+      <ReplicaSet>.setRoutingOption([router], option, value)
+
+WHERE
+      router: Identifier of the target router instance (e.g.
+              192.168.45.70::system).
+      option: The Router option to be changed.
+      value: The value that the option shall get (or null to unset).
+
+RETURNS
+      Nothing.
+
+DESCRIPTION
+      The accepted options are:
+
+      - tags: Associates an arbitrary JSON object with custom key/value pairs
+        with the ReplicaSet metadata.
+      - stats_updates_frequency: Number of seconds between updates that the
+        Router is to make to its statistics in the InnoDB Cluster metadata.
+
+      The stats_updates_frequency option accepts positive integers and sets the
+      frequency of updates of Router stats (timestamp, version, etc.), in
+      seconds, in the Metadata. If set to 0 (default), no periodic updates are
+      done. Router will round up the value to be a multiple of Router's TTL,
+      i.e.:
+
+      - If lower than TTL its gets rounded up to TTL, e.g. TTL=30, and
+        stats_updates_frequency=1, effective frequency is 30 seconds.
+      - If not a multiple of TTL it will be rounded up and adjusted according
+        to the TTL, e.g. TTL=5, stats_updates_frequency=11, effective frequency
+        is 15 seconds; TTL=5, stats_updates_frequency=13, effective frequency
+        is 15 seconds.
+
+      If the value is null, the option value is cleared and the default value
+      (0) takes effect.
+
 //@<OUT> Status
 NAME
       status - Describe the status of the replicaset.
