@@ -58,10 +58,6 @@ Creating user admin@%.
 Account admin@% was successfully created.
 
 Configuring instance...
-?{(VER(>=8.0.35) && VER(<8.1.0)) || VER(>=8.2.0)}
-
-WARNING: '@@binlog_transaction_dependency_tracking' is deprecated and will be removed in a future release. (Code 1287).
-?{}
 The instance '<<<__address1>>>' was configured to be used in an InnoDB ReplicaSet.
 NOTE: MySQL server needs to be restarted for configuration changes to take effect.
 
@@ -143,7 +139,7 @@ This instance reports its own address as 127.0.0.1:<<<__mysql_sandbox_port3>>>
 
 * Checking transaction state of the instance...
 
-NOTE: The target instance '<<<__address3>>>' has not been pre-provisioned (GTID set is empty). The Shell is unable to decide whether replication can completely recover its state.
+NOTE: The target instance '<<<__address3>>>' has not been pre-provisioned (GTID set is empty).
 
 Incremental state recovery selected through the recoveryMethod option
 
