@@ -58,10 +58,6 @@ Creating user admin@%.
 Account admin@% was successfully created.
 
 Configuring instance...
-?{((VER(>=8.0.35) && VER(<8.1.0)) || VER(>=8.2.0)) && !__replaying}
-
-WARNING: '@@binlog_transaction_dependency_tracking' is deprecated and will be removed in a future release. (Code 1287).
-?{}
 The instance '127.0.0.1:<<<__mysql_sandbox_port1>>>' was configured to be used in an InnoDB ReplicaSet.
 
 //@<OUT> configureReplicaSetInstance
@@ -120,10 +116,6 @@ NOTE: Some configuration options need to be fixed:
 
 Some variables need to be changed, but cannot be done dynamically on the server.
 Configuring instance...
-?{((VER(>=8.0.35) && VER(<8.1.0)) || VER(>=8.2.0)) && !__replaying}
-
-WARNING: '@@binlog_transaction_dependency_tracking' is deprecated and will be removed in a future release. (Code 1287).
-?{}
 The instance '127.0.0.1:<<<__mysql_sandbox_port2>>>' was configured to be used in an InnoDB ReplicaSet.
 NOTE: MySQL server needs to be restarted for configuration changes to take effect.
 
@@ -173,7 +165,7 @@ You are connected to a member of replicaset 'myrs'.
 |Adding instance to the replicaset...|
 |This instance reports its own address as 127.0.0.1:<<<__mysql_sandbox_port2>>>|
 |127.0.0.1:<<<__mysql_sandbox_port2>>>: Instance configuration is suitable.|
-|NOTE: The target instance '127.0.0.1:<<<__mysql_sandbox_port2>>>' has not been pre-provisioned (GTID set is empty). The Shell is unable to decide whether replication can completely recover its state.|
+|NOTE: The target instance '127.0.0.1:<<<__mysql_sandbox_port2>>>' has not been pre-provisioned (GTID set is empty).|
 |** Changing replication source of 127.0.0.1:<<<__mysql_sandbox_port2>>> to 127.0.0.1:<<<__mysql_sandbox_port1>>>|
 |The instance '127.0.0.1:<<<__mysql_sandbox_port2>>>' was added to the replicaset and is replicating from 127.0.0.1:<<<__mysql_sandbox_port1>>>.|
 
