@@ -1,18 +1,16 @@
 /*
- * Copyright (c) 2014, 2024, Oracle and/or its affiliates.
+ * Copyright (c) 2014, 2023, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is designed to work with certain software (including
- * but not limited to OpenSSL) that is licensed under separate terms,
- * as designated in a particular file or component or in included license
+ * This program is also distributed with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have either included with
- * the program or referenced in the documentation.
- *
+ * separately licensed software that they have included with MySQL.
  * This program is distributed in the hope that it will be useful,  but
  * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
@@ -189,6 +187,10 @@ class Shell_cmdline_options : public tests::Shell_base_test {
       return options->mysql_plugin_dir;
     else if (option == "log-sql")
       return options->log_sql;
+#ifdef _WIN32
+    else if (option == "plugin-authentication-kerberos-client-mode")
+      return options->connection_options().get_kerberos_auth_mode();
+#endif
 
     throw std::logic_error(option);
   }
@@ -805,6 +807,10 @@ TEST_F(Shell_cmdline_options, default_values) {
   EXPECT_EQ("error", options.log_sql);
   EXPECT_EQ("*SELECT*:SHOW*", options.log_sql_ignore);
   EXPECT_EQ("*IDENTIFIED*:*PASSWORD*", options.log_sql_ignore_unsafe);
+
+#ifdef _WIN32
+  EXPECT_FALSE(options.connection_options().has_kerberos_auth_mode());
+#endif
 }
 
 TEST_F(Shell_cmdline_options, app) {
@@ -891,7 +897,7 @@ TEST_F(Shell_cmdline_options, app) {
   test_option_with_no_value("--sqlx", "initial-mode",
                             shell_mode_name(IShell_core::Mode::SQL));
 
-#ifdef HAVE_JS
+#ifdef HAVE_V8
   test_option_with_no_value("--javascript", "initial-mode",
                             shell_mode_name(IShell_core::Mode::JavaScript));
   test_option_with_no_value("--js", "initial-mode",
@@ -956,6 +962,15 @@ TEST_F(Shell_cmdline_options, app) {
   test_option_equal_invalid_values("log-sql", {"filtered"},
                                    "--log-sql: The log level value must be any "
                                    "of {off, error, on, all, unfiltered}.\n");
+
+#ifdef _WIN32
+  test_option_with_value_list("plugin-authentication-kerberos-client-mode", "",
+                              {"GSSAPI", "SSPI"}, "", false, false);
+
+  test_option_equal_invalid_value(
+      "plugin-authentication-kerberos-client-mode", "GSSPI",
+      "Invalid value: GSSPI. Allowed values: SSPI, GSSAPI.\n");
+#endif
 }
 
 TEST_F(Shell_cmdline_options, test_session_type_conflicts) {
@@ -1118,31 +1133,6 @@ TEST_F(Shell_cmdline_options, test_deprecated_arguments) {
         "The --dbuser option was deprecated, please use --user instead.",
         cout.str());
   }
-  {
-    SCOPED_TRACE("TESTING: deprecated --recreate-schema argument");
-    firstArg = "root@localhost:3301/some-schema";
-    secondArg = "--recreate-schema";
-
-    char *argv[] = {const_cast<char *>("ut"),
-                    const_cast<char *>(firstArg.c_str()),
-                    const_cast<char *>(secondArg.c_str()), NULL};
-
-    // Redirect cout.
-    std::streambuf *cout_backup = std::cout.rdbuf();
-    std::ostringstream cout;
-    std::cout.rdbuf(cout.rdbuf());
-
-    Shell_options cmd_options(3, argv);
-
-    // Restore old cout.
-    std::cout.rdbuf(cout_backup);
-
-    EXPECT_EQ(0, cmd_options.get().exit_code);
-    EXPECT_STREQ(cmd_options.get().connection_options().as_uri().c_str(),
-                 "root@localhost:3301/some-schema");
-    MY_EXPECT_OUTPUT_CONTAINS("The --recreate-schema option was deprecated.",
-                              cout.str());
-  }
 }
 
 TEST_F(Shell_cmdline_options, test_positional_argument) {
